#include "DomainModel.h"
#include "GenAttrSetType.h"




DomainModel::DomainModel(const char* n): name(n)
{
   node_unknown = new GenericAttrSetType(""); 		//Always create the ""-type for the unknown node type
   addNodeType(node_unknown);				//...and add it to this domain model.

   arc_unknown = new GenericAttrSetType(""); 		//Always create the ""-type for the unknown arc type
   addArcType(arc_unknown);				//...and add it to this domain model.
}


DomainModel::~DomainModel()				//Destroy all attr-set-types we contain.
{							//We own them, so we can delete them
   for(map<STRING,AttrSetType*>::iterator i=node_types.begin();i!=node_types.end();i++)
      delete (*i).second;
   for(map<STRING,AttrSetType*>::iterator j=arc_types.begin();j!=arc_types.end();j++)
      delete (*j).second;
}


void DomainModel::save(FILE*)                           //Default (nop) version of saving Tcl-cmd to reload this
{  }

const AttrSetType* DomainModel::getNodeType(const char* n) const
{
   map<STRING,AttrSetType*>::const_iterator i = node_types.find(STRING(n));

   return (i!=node_types.end())? (*i).second : node_unknown;	//If the required type is not known, 
								//we return the unknown, empty type.
}

const AttrSetType* DomainModel::getNodeType(int i) const
{
   map<STRING,AttrSetType*>::const_iterator it=node_types.begin();
   for(;i>0;i--) it++;
   return (*it).second;
}


int DomainModel::numNodeTypes() const
{  
   return node_types.size();
} 


const AttrSetType* DomainModel::getArcType(const char* n) const
{
   map<STRING,AttrSetType*>::const_iterator i = arc_types.find(STRING(n));

   return (i!=arc_types.end())? (*i).second : arc_unknown;	//If the required type is not known, 
								//we return the unknown, empty type.
}


const AttrSetType* DomainModel::getArcType(int i) const
{
   map<STRING,AttrSetType*>::const_iterator it=arc_types.begin();
   for(;i>0;i--) it++;
   return (*it).second;
}


int DomainModel::numArcTypes() const
{  
   return arc_types.size();
} 


void DomainModel::addNodeType(AttrSetType* a)
{
    map<STRING,AttrSetType*>::iterator i = node_types.find(a->getName());

    if (i!=node_types.end())
    {
       delete (*i).second;
       node_types.erase(i);	    
    }

    map<STRING,AttrSetType*>::value_type v(a->getName(),a);

    node_types.insert(v);
}


void DomainModel::addArcType(AttrSetType* a)
{
    map<STRING,AttrSetType*>::iterator i = arc_types.find(a->getName());

    if (i!=arc_types.end())
    {
       delete (*i).second;
       arc_types.erase(i);	    
    }

    map<STRING,AttrSetType*>::value_type v(a->getName(),a);

    arc_types.insert(v);
}
