# Load a hierarchical graph.
#
#
pr_read_own_format DATA/test1.dat
pr_sel_new level_sel
pr_sel_new contain_sel

# Compute a level-attr and initialize level_sel to the bottommost level
# (Computing the level-attr is sometimes redundant as it is usually saved in the file)
#
pr_compute_level [pr_get_selection name All] "level" "type" "contain" in
pr_select_nodes_by_value All level_sel "level" 0


# Make viewers and display information
#
#

# For the containment graph, use a tree-layout where the node-ranks are constrained by
# the level-attr we compute. This will keep nodes on their levels in the layout.
#
pr_lay_dot -set ranksep 5.0 rankdir 1 rank_attr "level"
pr_lay_dot contain_sel pos
pr_view_new vw contain_sel pos hl_sel
pr_view vw set node_scale 0.1

# For the association graph, use a spring layout.
#
#
pr_lay_neato All pos3
pr_view_new vw3 All pos3 hl_sel
pr_view vw3 set node_scale 0.1   
pr_view vw3 set user_cb my_show_contain_tree


# Make two specific node and edge Tcl scripts to illustrate how we can customize a viewer.
#
#

proc my_node_glyph {node} {
    set id [pr_node $node id]
    if {$id<=15} { 
       return [pr_make_file file IV/file.iv]
    } else {	
       return [pr_make_file file IV/folder.iv color 0 $id/10 0 scale 0.8]
    }
}

proc my_edge_glyph {e x y z} {
    set n1 [pr_edge $e node from]
    set n2 [pr_edge $e node to] 
    set t  [pr_edge $e type]
    if {$t=="contain"} {
        set r 1; set g 0; set b 0
    } else {
        set r 1; set g 1; set b 1
    }
    return [pr_make_line edge $e x $x y $y z $z color 0 0 0 $r $g $b]
}

# Make two glyph-factories and use them for viewer 'vw3'.
# Use the above defined Tcl scripts for these factories.
#
pr_glyphs new IVTclNodeGlyphFactory my_node_factory
set gf [pr_glyphs get node_glyph name my_node_factory] 
pr_glyph_factory $gf set cb my_node_glyph

pr_glyphs new IVTclEdgeGlyphFactory my_edge_factory
set gf [pr_glyphs get edge_glyph name my_edge_factory] 
pr_glyph_factory $gf set cb my_edge_glyph

pr_view vw3 set node_glyph my_node_factory
pr_view vw3 set edge_glyph my_edge_factory
pr_view vw3 set edge_mapper basic

#  my_cluster {sel}: Clusters all nodes in 'sel' into a new parent and
#			   refreshes the app's windows.			
#
proc my_cluster {sel} {
  # 1. Cluster the nodes in 'sel' and call parent 'p'
  #
  set p [pr_cluster $sel]
  # 2. Add an attribute 'type' of value 'contain' to all edges
  #    from parent 'p' to its children.
  #
  set edg [pr_node $p edges [pr_get_selection name All]]
  pr_set $edg "type" "contain" string
  pr_selection $edg delete 
  
  # 3. Set parent level to 1 + highest level of its children
  #
  set l -1
  set i [pr_selection [pr_get_selection name $sel] iter_nodes]
  set n [pr_iterator $i next_node]
  while {$n!=0} {
    set nl [pr_node $n attr level]
    if {$nl>$l} { set l $nl }
    set n [pr_iterator $i next_node]
  }  
  incr l; pr_node $p set level $l int

  # 4. Finally, refresh the several windows of the application
  #
  pr_select_edges_by_value All contain_sel "type" "contain"
  pr_lay_dot -set ranksep 5.0 rankdir 1
  pr_lay_dot contain_sel pos
}

#   my_show_contain_tree {opt val}: Shows containment-tree under given node 'val'
#				    in the viewer 'vw'
#
proc my_show_contain_tree {opt val} {
   # Treat only nodeclick events here
   #
   if {$opt!="node"} { return }
   set nd $val
   # Make new selection, put node in it, then get containment-tree under it
   #
   set root [pr_sel_new root_sel]
   pr_selection $root add_node [pr_get_node All id $nd]
   pr_traverse_dag $root attr "type" "contain" contain_sel
   
   # Relayout the obtained tree
   #
   pr_lay_dot -set ranksep 5.0 rankdir 1 rank_attr "level"
   pr_lay_dot contain_sel pos
}



   