#  Load the RSF layered graph
#
#

pr_domain_load_rsf DATA2
pr_load DATA2/hava_high_level.rsf




#  Filter out the composite edges and obtain selection 'no_compo'
#
pr_select_edges_by_type All no_compo "composite" 0


#  Filter out all the non-association edges and all edges connected to Collapse nodes 
#  Since this selection is quite complex, we write a small procedure to implement it, as follows
#


# pr_my_filter: 
#		
#-------------------------------------

proc pr_my_filter {sel sel2} {

  set res [pr_sel_new $sel2]
  set inp [pr_get_selection name $sel]

  set i [pr_selection $inp iter_edges]
  set e [pr_iterator $i next_edge]

  while {$e!=0} {
      set  t [pr_edge $e type]	
      set b1 [string equal $t "invocation"]
      set b2 [string equal $t "access"]
      set b3 [string equal $t "include"]
      set b4 [string equal $t "inherit"]
      set b5 [string equal [pr_node [pr_edge $e node from] type] "Collapse"]
      set b6 [string equal [pr_node [pr_edge $e node to]   type] "Collapse"]
      if {[expr ($b1 || $b2 || $b3 || $b4) && ($b5==0) && ($b6==0)]==1} {
         pr_selection $res add_edge $e 
      }
      set e [pr_iterator $i next_edge]
  }

  set i [pr_selection $inp iter_nodes]
  set n [pr_iterator $i next_node]

  while {$n!=0} {
    if {[pr_node $n type]!="Collapse"} {
       pr_selection $res add_node $n
    }
    set n [pr_iterator $i next_node]
  }
}

# Ok, now we have the selection procedure, so apply it
#
pr_my_filter no_compo no_compo_level

#  Select now only the "level" edges to get a 'vertical slice' 
#
pr_select_edges_by_type All only_level "level" 1


#-------------------------------------------------------------

# Now produce several layouts for the three selections we have
# Make the default node size somewhat smaller to nicely fit with
# these layouts.
#
pr_lay -set node_size         0.02
pr_lay_neato All    		pos
pr_lay_neato no_compo 		pos2
pr_lay_neato no_compo_level 	pos3
pr_lay_dot  -set ranksep        50
pr_lay_dot   only_level 	pos4


# Make some fancy edge-glyph showing the edge type via its color
#
#
proc my_make_eglyph {e x y z} {
    set t [pr_edge $e type]
    if {$t=="composite"} {
       return [pr_make_line edge $e x $x y $y z $z color 1 0 0 1 0 0]
    }

    if {$t=="level"} {
       return [pr_make_line edge $e x $x y $y z $z color 0 0 1 0 0 1]
    }

    if {$t=="invocation"} {
       return [pr_make_line edge $e x $x y $y z $z color 1 1 0 1 1 0]
    }

    return [pr_make_line edge $e x $x y $y z $z color 1 1 1 1 1 1]
}

# Make two glyph-factories and use them for viewer 'vw2'.
# Use the above defined Tcl scripts for these factories.
#

pr_glyphs new IVTclEdgeGlyphFactory my_edge_factory
pr_glyph_factory [pr_glyphs get edge_glyph name my_edge_factory] set cb my_make_eglyph


# First, view ALL edges, using different colors to indicate type
# Use the same pick-selection to correlate pick in the different viewers
#
pr_view_new vw All pos hl_sel
pr_view vw set edge_glyph my_edge_factory


# Next, view all edges except composite ones. The picture is still messy...
#
#
pr_view_new vw2 no_compo pos2 hl_sel
pr_view vw2 set edge_glyph my_edge_factory


# Third, view only the association edges as well as non Collapse nodes.
# Here we get some more insight...
#
pr_view_new vw3 no_compo_level pos3 hl_sel
pr_view vw3 set edge_glyph my_edge_factory


# Finally, view a vertical aggregation-slice, showing the level edges
# 
#
pr_view_new vw4 only_level pos4 hl_sel
pr_view vw4 set edge_glyph my_edge_factory

