pr_load DATA/johan_result2.dat

set tree        "tree"
set assoc       "assoc"
pr_sel_new      "hl_sel"

#  my_selection: Selects a subgraph to examine. Basically, the 'component' nodes are not selected,
#		     nor are the 'Component_of_scenario' and '_Representation_of_component' arcs.
#		     Moreover, the '_Component_of_scenario' and 'Scenario_of_use_case' arcs are marked
#		     as 'containment_type' arcs and will be used as such in the nested layout.
#---------------------------------------------------------------------------------------------------
proc my_selection {inp out} {
   set so [pr_sel_new $out]
   set si [pr_get_selection name $inp]
   set i [pr_selection $si iter_nodes]
   set n [pr_iterator $i next_node]
   while {$n!=0} {
     set t [pr_node $n attr "type"]
     if {$t!="component"} { pr_selection $so add_node $n }
     set n [pr_iterator $i next_node]
   }
   set i [pr_selection $si iter_edges]
   set e [pr_iterator $i next_edge]
   while {$e!=0} {
     set t [pr_edge $e attr "type"]
     if {$t!="Component_of_scenario" && $t!="_Representation_of_component"} {
       set n1 [pr_edge $e node from]; set n2 [pr_edge $e node to]
       if {[expr [pr_selection $so contains node $n1] && [pr_selection $so contains node $n2]]} {
          pr_selection $so add_edge $e 
       }
     }
     if {$t=="_Component_of_scenario" || $t=="Scenario_of_use_case"} {
        pr_edge $e set "containment_type" "true" string
     } else {
        pr_edge $e set "containment_type" "false" string
     } 
     set e [pr_iterator $i next_edge]
   }
}

# Select the containment-tree from the input data. The tree is used for the nested layout.
# Select the tree-nodes + assoc-edges. This will be drawn in the viewer.
#------------------------------------------------------------------------------------------
my_selection All $tree
pr_select_edges_by_value $tree $assoc "type" "_Method_call"

# Do the nested layout (in 2D)
#---------------------------------------------------------------
pr_lay -set node_size 1.0
pr_lay -set keep_dims 0
pr_lay -set use_crt_pos 0
pr_lay -set use_crt_dim 0
pr_lay_neato -set edgelen 0.1
pr_lay_neato -set weight 10.0
pr_lay_nested -set zlayout 0
pr_lay_nested $tree pos traverse "containment_type" "true"

# Display the data in the viewer
#----------------------------------------------------------------
pr_view_new vw  $assoc     pos "hl_sel"
pr_view     vw  set        sweep_cb my_sweep_cb
pr_view     vw  set        user_cb  my_click_cb
pr_view     vw  set        edge_res smooth


# Connect the glyph-procedures to glyph-mappers and then to viewers
#-------------------------------------------------------------------
pr_glyphs new IVTclNodeGlyphFactory my_node_mapper
pr_glyph_factory [pr_glyphs get node_glyph name my_node_mapper] set cb my_node_glyph
pr_view vw set node_glyph my_node_mapper




# Sweep-proc for viewers. Displays node name and type and edge type
#-------------------------------------------------------------------
proc my_sweep_cb {opt val} {
  set ret ""
  if {$opt=="node"} { 
     pr_gui:attrset_refresh_id .nodes $val 
     set n  [pr_get_node All id $val]
     set nm [pr_node $n attr "name"]
     set tp [pr_node $n attr "type"]
     return [join [list $nm $tp] "\n"]
  }  
  if {$opt=="edge"} {
     pr_gui:attrset_refresh_id .nodes $val 
     set e  [pr_get_edge All id $val]
     set tp [pr_edge $e attr "type"]
     return $tp
  }
  return ""
}

# my_click_cb: Callback for nodeclick. Selects, in "hl_sel", all component-reps
# 		   corresponding to the clicked component-rep. For this, traverses from the clicked component-rep
#              (via a _Representation_of_Component edge) to the real component. Next, traverses from this
#		   component to all its component-reps, (via _Representation_of_Component edges). These
#		   component-reps are put then in "hl_sel". 
#-------------------------------------------------------------------------------------------------------------
proc my_click_cb {opt val} {
     if {$opt=="edge"} { return }
     if {$opt=="selection"} { return }
     set all [pr_get_selection name "All"]
     set hl  [pr_get_selection name "hl_sel"]
     set n   [pr_get_node All id $val]
     set se  [pr_node $n edges $all]
     set i   [pr_selection $se iter_edges]
     set e   [pr_iterator $i next_edge]
     while {$e!=0} {
        if {[pr_edge $e attr "type"]=="_Representation_of_Component"} {
           set c [pr_edge $e node to]
           set se2 [pr_node $c edges $all]
           set j   [pr_selection $se2 iter_edges]
           set e2  [pr_iterator $j next_edge]
           while {$e2!=0} {
		  if {[pr_edge $e2 attr "type"]=="_Representation_of_Component"} {
                 set n2 [pr_edge $e2 node from]
                 pr_selection $hl add_node $n2
              }
              set e2 [pr_iterator $j next_edge]
	     }
           pr_selection $se2 delete	
       }
       set e [pr_iterator $i next_edge]
     }
     pr_selection $se delete
}   






proc my_node_glyph {node} {
  set t [pr_node $node attr "type"]
  if {$t=="use_case"} {
       return [pr_make_cube color 0 0 1 style lines shading off]
  } elseif {$t=="scenario"} {
       return [pr_make_cube color 0 1 0 style lines shading off]
  } elseif {$t=="_ScenarioComponent"} {
       set se [pr_node $node edges [pr_get_selection name "All"]]
       set i  [pr_selection $se iter_edges]
       set e  [pr_iterator $i next_edge]
       while {$e!=0} {
          if {[pr_edge $e attr "type"]=="_Representation_of_Component"} {
             set c [pr_edge $e node to]
          }
          set e  [pr_iterator $i next_edge]
       }
       pr_selection $se delete
       set n1 [pr_node $c attr "Nr_of_called_services_of_component"]
       set n2 [pr_node $c attr "Nr_of_service-calls_of_component"]
	 set n3 [pr_node $c attr "Coupling"]
       set n4 [pr_node $c attr "Inverse_coupling"]
	 if {$n1==""} {set n1 0}
	 if {$n2==""} {set n2 0}
       if {$n3==""} {set n3 0}
	 if {$n4==""} {set n4 0}
	 set n1 [expr $n1 * 0.5]; set n2 [expr $n2 * 0.5]
       set n3 [expr $n3 * 0.5]; set n4 [expr $n4 * 0.5]
	 if {$n1==0} {set n1 0.1} 
       if {$n2==0} {set n2 0.1}
       if {$n3==0} {set n3 0.1}
       if {$n4==0} {set n4 0.1}
       return [pr_make_bar nv 1  nvalues 4 values $n1 $n2 $n3 $n4 colors 1 1 0  0 0 1  1 0 0  0 1 0]
  } 
  return [pr_make_cube color 1 1 1 style lines shading off]
}


