# First read the data
#
pr_read_own_format DATA/nested.dat

# Select nodes + association edges in 'sel_a'
# and nodes + containment edges in 'sel_c'
# We do this since we want to visualize them differently
#
pr_select_edges_by_value All sel_c "type" "contain"
pr_select_edges_by_value All sel_a "type" "assoc"

pr_sel_new hl
pr_sel_new tree

# Compute a level attribute for the read data
# We'll use this to visualize node-ranks further
pr_compute_level [pr_get_selection name All] "level" "type" "contain" in


# Visualize full containment as a tree in 'vw'
#
pr_lay_dot sel_c pos_c
pr_view_new vw sel_c pos_c hl
pr_view vw set node_scale 0.1
pr_view vw set user_cb my_nodeclick_cb 

# Visualize containment also using a nested layout in 'vw2'
#
pr_lay_nested All pos_c2 traverse "type" "contain"
pr_view_new vw2 sel_a pos_c2 hl
pr_view vw2 set user_cb my_nodeclick_cb
pr_view vw2 set output_only 1

# Visualize containment-tree from selected node in a separate viewer
# 
pr_view_new vw4 tree pos_c3
pr_view vw4 set node_mapper basic
pr_view vw4 set edge_mapper basic



# Make two glyph-factories and use them for viewer 'vw2' 
# Use the above defined Tcl scripts for these factories.
#
pr_glyphs new IVTclNodeGlyphFactory my_node_factory
set gf [pr_glyphs get node_glyph name my_node_factory] 
pr_glyph_factory $gf set cb my_make_nglyph

pr_glyphs new IVTclEdgeGlyphFactory my_edge_factory
set gf [pr_glyphs get edge_glyph name my_edge_factory] 
pr_glyph_factory $gf set cb my_make_eglyph

pr_view vw2 set node_glyph my_node_factory
pr_view vw2 set edge_glyph my_edge_factory


#  Private stuff
#
#--------------------------------------------------------------------------



proc my_make_nglyph {node} {
    set l [pr_node $node attr level]
    set r 1; set g 0; set b 0;
    if {$l==0} { set r 0; set g 1; set b 0; }
    if {$l==1} { set r 0.2; set g 0.2; set b 1; }
    if {$l==2} { set r 1; set g 1; set b 0; }
    return [pr_make_cube color $r $g $b style lines shading off] 
}

proc my_make_eglyph {e x y z} {
    set l1 [pr_node [pr_edge $e node from] attr level]
    set l2 [pr_node [pr_edge $e node to] attr level]
    set  l [expr int(($l1+$l2)/2)] 
    set wd [expr 4-$l]  
    set c  0.8

    return  [pr_make_line edge $e x $x y $y z $z color $c $c $c $c $c $c width $wd]
}

proc my_nodeclick_cb {opt nidx} {
  if {$opt!="node"} { return }
  set n [pr_get_node All id $nidx]

  set root [pr_sel_new root_sel]
  pr_selection $root add_node $n
  pr_traverse_dag $root attr "type" "contain" tree

  set ts [pr_get_selection name tree]

  pr_lay_dot tree pos_c3
   
  #!!! various updates/redraws must come in here
} 

