#include "Edge.h"
#include "Node.h"
#include "Selection.h"
#include "SelList.h"
#include "GraphManager.h"


Edge::Edge(Selection* s,Node* f,Node* t,const AttrSetType* ast,int id)
     : Element(ast,id),from(f),to(t),node_refcount(0),in_dtor(FALSE)
{  
   Selection* all = GraphManager::getInstance()->getSelList()->getFullSelection();
   if (s && s!=all) s->addEdge(this);		        //add this to specified selection, if any given (and if not full-sel)
   all->addEdge(this);                                  //in any case, add this to full-sel
}


Edge::~Edge()
{  }

Node* Edge::node(Node::DIRECTION d)
{  return (d==Node::IN)? from : to;  }


void Edge::destroy()
{
   int N = numSelections();
   for(int i=0;i<N;i++)					//Inform all selections that own this that this goes away
   {
      Selection* s = getSelection(0);			//Always get the 1st selection since delEdge() does remove
      s->delEdge(this);					//the registration of this by selections
   }
}


void Edge::addToSelection(Selection* s)			//Adds this to given selection. This is NOT in given selection yet.
{
    from->addOutEdge(this,s);				 
    to->addInEdge(this,s);
    Element::addToSelection(s);
}

void Edge::delFromSelection(Selection* s)
{
   if (in_dtor)
	   Element::delFromSelection(s);
   else
	   delFromNodes(s);
}


void Edge::delFromNodes(Selection* s)
{
   in_dtor = TRUE;	
   if (node_refcount)					//If this is still in use by any node, remove this usage
   {  from->delEdge(this,s); to->delEdge(this,s); }
   
   if (node_refcount)
   {
      s->removeEdge(this);				//remove this from 's' but do NOT ask 's' to update
      Element::delFromSelection(s);			//If this is still in use by any node (e.g. in context of some selection)
      in_dtor = FALSE;	
   }      
   else							//then release its use from selection 's' 
      destroy();					//else destroy this since not used by ANY other node.
}





