#include "Node.h"
#include "Filter.h"
#include "Edge.h"
#include "AttrSetType.h"

//----------  Filter  ------------------

Filter::~Filter()
{   }


//-----------  DirectionFilter  -------------------

EdgeSet	DirectionFilter::filter(Selection* s,Node* n)
{
    return n->getEdges(s,direction);
}

Filter* DirectionFilter::clone()
{  return new DirectionFilter(*this);  }


//------------  AttrFilter  --------------------

EdgeSet AttrFilter::filter(Selection* s,Node* n)	//Impl inh
{
   const EdgeSet& es = n->getEdges(s,direction);	//First get only edges in the desired direction	
   EdgeSet   r;

   BOOL unset = (value.getType()==Attr::UNSET);		
   for(int i=0;i<es.Count();i++)			//For all edges of this:
   {
      const Attr* b = es[i]->get(attr);			//If edge has no attr with given name, skip it
      if (!b) continue;					//If edge has attr with given name & value, get it
      if (value.compare(*b))  r.Add(es[i]);		//We use here Attr::compare() and not Attr::operator==()
							//since we want to compare only the VALUES, not the 
							//TYPES of the attrs.
   }
   return r;
}   

Filter* AttrFilter::clone()
{  return new AttrFilter(*this);  }


//-------------  TypeFilter  ----------------------

EdgeSet TypeFilter::filter(Selection* s,Node* n)	//Impl inh
{
   const EdgeSet& es = n->getEdges(s,direction);	//First get only edges in the desired direction	
   EdgeSet   r;

   
   for(int i=0;i<es.Count();i++)			//For all edges of this:
   {
      const STRING& t = es[i]->getType()->getName();	//Compare their type-name with filter's type-name
      if (t==type) r.Add(es[i]);
   }
   return r;
}   

Filter* TypeFilter::clone()
{  return new TypeFilter(*this);  }


//-------------   NodeFilter  ----------------------


EdgeSet NodeFilter::filter(Selection* s,Node* n)	//Impl inh
{
   const EdgeSet& es = node->getEdges(s,direction);	//First get only edges in the desired direction
   EdgeSet   r; int i;
   
   if (direction==Node::IN || direction==Node::BOTH)
   for(i=0;i<es.Count();i++)				//For all edges of this:
      if (es[i]->fromNode()==n) r.Add(es[i]);
   if (direction==Node::OUT || direction==Node::BOTH)
   for(i=0;i<es.Count();i++)				//For all edges of this:
      if (es[i]->toNode()==n) r.Add(es[i]);
   return r;
} 

Filter* NodeFilter::clone()
{  return new NodeFilter(*this);  }

 