#include "GUIManager.h"
#include <iostream.h>
#include "GraphManager.h"
#include "ObsList.h"
#include <tcl.h>
#include <tk.h>
#include <iostream.h>
#include "TclApi.h"
#include "UI.h"
#include "utils.h"
#include "PathQualifier.h"


#ifdef WIN32
#include <windows.h>
//STUFF BELOW NEEDED SINCE BOTH tk.h AND SoWin.h DEFINE THESE !!!
#undef ButtonPressMask 
#undef ButtonReleaseMask 
#undef PointerMotionMask 
#undef ButtonMotionMask  
#undef KeyPressMask      
#undef KeyReleaseMask
#include <Inventor/Win/SoWin.h>
#endif   


GUIManager* GUIManager::instance  = 0;
BOOL	    GUIManager::exit_flag = FALSE;
char        GUIManager::err_buf[1024];


void GUIManager::init()
{   if (!instance)  instance = new GUIManager;  }


static Tcl_Interp* interp = 0;		// We don't make these static in the class in order not to pollute	
#ifndef WIN32				// the header file with implementation types...
static XtAppContext appContext;
#endif


#ifdef WIN32
void bindXtTk(Tcl_Interp* interp, const char* winName, HWND parent)
#else
void bindXtTk(Tcl_Interp* interp, const char* winName, Window parent)
#endif
{
   char tCmd[1000];

   sprintf (tCmd, "toplevel %s -use %d", winName, parent);
   
   if (Tcl_Eval (interp, tCmd) != TCL_OK)
   {
      cerr<<"GUIManager::bindXtTk: error binding Xt to Tk: "<<Tcl_GetStringResult(interp)<<endl;
      cerr<<"Please try to run the application again."<<endl;
      exit(1);
   }
}


#ifdef WIN32
void fitTk(char* winName, HWND wid)
{
  Tk_Window twin = Tk_NameToWindow (interp, winName, Tk_MainWindow (interp));
  Tcl_Eval (interp, "update idletasks");
}
#else
void fitTk(char* winName, Widget wid) 	
{
  Arg arg[10];
  Tk_Window twin = Tk_NameToWindow (interp, winName, Tk_MainWindow (interp));
  Tcl_Eval (interp, "update idletasks");
  XtSetArg(arg[0], XmNwidth,  Tk_Width (twin));
  XtSetArg(arg[1], XmNheight, Tk_Height(twin));
  XtSetValues(wid,arg,2);
  XFlush (XtDisplay(wid));	
}
#endif



GUIManager::GUIManager()
{
   interp = Tcl_CreateInterp();			//construct Tcl interpreter

   if (Tcl_Init(interp)!=TCL_OK) 		//initialize Tcl system
   {
     cerr<<"Tcl_Init error - program aborted"<<endl; exit(1);
   }

   if (Tk_Init(interp)!=TCL_OK) 		//initialize Tk system	
   {
     cerr<<"Tk_Init error - program aborted"<<endl; exit(1);
   }

   pr_initialize(interp);                       //load names of precompiled Tcl-procs into interpreter
   SoUI::init();				//initialize Inventor system


#ifndef WIN32   			
   //SoXt::show(tl);				//this'll create an Xt toplevel shell.
						//we could actually use the X toplevel window
						//of Tk's own main-window, but that one is NOT
						//an Xt widget, just an X window. And we need an Xt
						//widget for Inventor... so we let it create one itself.

   XFlush(XtDisplay(tl));			//make toplevel window on-screen
						//so it gets an X window. We need to do this since
   XSync(XtDisplay(tl),False);			//we need the X window to associate it with
						//the toplevel window of Tk... Also, we must wait for the
   //bindXtTk(interp,".top",XtWindow(tl));	//X window to appear on the screen before we pass it to Tk.
#endif						

   exec("pr_read_tcl TCL/init.tcl");		//run the system-wide initialization script
                                                //(do it via pr_read_tcl to use path-qualification)

#ifndef WIN32
   //fitTk(".top",tl);				//this makes the Tcl widgets fit into the Xt parent
   appContext = XtWidgetToApplicationContext(tl);
#endif
}







#ifdef WIN32					//MS Windows

void processTclEvents()                         //process one Tcl event
{  while (Tk_DoOneEvent (TCL_DONT_WAIT));  }    


void CALLBACK timeout(HWND,UINT,UINT,DWORD)
{  processTclEvents();  } 

#else						//X11/UNIX

void processTclEvents()				//process one Tcl event
{   
   Tk_DoOneEvent(TCL_DONT_WAIT);  
   GraphManager::getInstance()->cleanup();
}

Boolean handleTclEvents(XtPointer)		//callback that allows Tcl events to be processed
{  processTclEvents();  return False;  }	//when or Xt-based loop gets idle	

void timeout(XtPointer,XtIntervalId*)		//as above, but invoked not when the Xt loop gets
{  						//idle, but periodically. This allows Tcl events
   processTclEvents();  			//to be processed even when the Xt loop is 'busy'
   XtAppAddTimeOut(appContext,10,timeout,0);
}
#endif






void GUIManager::mainLoop()			//Xt loop that services both native X and
{  						//Tcl events, as described above
   #ifdef WIN32
     SetTimer(0,0,250,&timeout);                //REMARK: the timeout-freq can be lowered to spare
     MSG msg;                                   //        processor time but then the Tcl GUI is less interactive.
     for(;exit_flag==FALSE;)			//Loop until we shutdown()
     {
       while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
       {  TranslateMessage(&msg); DispatchMessage(&msg);  }
       GraphManager::getInstance()->cleanup();
       WaitMessage();
       processTclEvents();
     }  
     delete instance;                           //Cleanup: delete this
   #else
     XtAppAddWorkProc(appContext,handleTclEvents,0);
     XtAppAddTimeOut(appContext,10,timeout,0);
     XtAppMainLoop(appContext);
   #endif					
}


int GUIManager::spawn(const char* s)
{
   return (system(s)!=0);
}

void GUIManager::shutdown()			//Exit procedure. This shots down in a rather contorted way
{						//now: see below the cases for Windows/X11...
  #ifndef WIN32
                        			//Once we're finished with the GUI, it means we're 
    delete instance;				//done with the application too, so destroy this.
    exit(0);					//Force shutdown. Actually we should be able to do this
    //XtAppSetExitFlag(*(XtAppContext*)cd);	//via XtAppSetExitFlag - but the Xt-Inventor event system	
						//is such now that it takes a little while until it gets
						//to read the Xt events. This is mildly annoying, so we
						//simply force an exit() here - it is safe.
  #endif
  exit_flag = TRUE;				//Signal that the event loop has to exit (Windows only so far)
}


GUIManager::~GUIManager()                       //Dtor
{
   GraphManager::destr();                       //First delete our own data        
   Tcl_DeleteInterp(interp);                    //Next release the Tcl interpreter
}

void GUIManager::error(const char* s1,const char* s2)
{
   strcpy(err_buf,s1);
   if (s2) strcat(err_buf,s2);
}

void GUIManager::error(const char* s1,int s2)
{
   sprintf(err_buf,"%s %d",s1,s2);
}



BOOL GUIManager::exec(const char* c)
{
   return (Tcl_Eval(interp,(char*)c)==TCL_OK)? TRUE:FALSE;	//REMARK: cast is needed by Tcl
}                                                               //but should be harmless
								

BOOL GUIManager::execFile(const char* c)
{
   return (Tcl_EvalFile(interp,(char*)c)==TCL_OK)? TRUE:FALSE;	//REMARK: cast is needed by Tcl
}



long GUIManager::getLongResult()
{
   long ret;
   Tcl_GetLongFromObj(interp, Tcl_GetObjResult(interp), &ret);
   return ret;
}

STRING GUIManager::getStringResult()
{
   STRING res = Tcl_GetString(Tcl_GetObjResult(interp));
   return res;
}

  
void GUIManager::saveTclProc(FILE* fp,const char* name)         //Save Tcl-proc-def given Tcl-proc-name
{
   STRING cmd,args,body;
   cmd = "info args "; cmd += name; 
   if (!exec(cmd)) return;                                      //Ask Tcl the proc-args. If failed, nothing to 
   args = getStringResult();                                    //do further;

   cmd = "info body "; cmd += name;                             //Ask Tcl the proc-body. If failed, nothing to
   if (!exec(cmd)) return;                                      //do further.
   body = getStringResult();

   fprintf(fp,"proc %s {",name);                                //Ok, we have all info, save the Tcl-proc
   fprintf(fp,"%s} {",args);
   fprintf(fp,"%s",body);
   fprintf(fp,"}\n\n");
}
