#include "Selection.h"
#include "Geometry.h"
#include "Node.h"
#include "Genrl/types.h"
#include <iostream.h>




void Geometry::bbox2D(Selection* sel,const char* pos,float& xm, float& ym,float& xM, float& yM)
//Computes bbox of given Selection of nodes.
{
   Selection::NodeIterator* i; Node* n;	 
   xm=ym=INFINITY; xM=yM=-INFINITY;

   STRING pos_x = getPositionX(pos),  pos_y = getPositionY(pos),
          dim_x = getDimensionX(pos), dim_y = getDimensionY(pos);

   for(i=sel->iterNodes();(n=i->next());)
   {
      float x  = n->getFloat(pos_x), y  = n->getFloat(pos_y),
            dx = n->getFloat(dim_x), dy = n->getFloat(dim_y);

      dx /= 2; dy /= 2;
	
      if (x-dx<xm) xm = x-dx; if (x+dx>xM) xM = x+dx;
      if (y-dy<ym) ym = y-dy; if (y+dy>yM) yM = y+dy;
   }
}

int* Geometry::spline(float* Px,float* Py,float* Pz,float t,int K,int& N)
//Generates spline
{
   static int buf1[1000],buf2[1000];
   int *idx = buf1, *nidx = buf2;
   int i; 
   
   for(i=0;i<N;i++) idx[i]=i; nidx[0] = 0;
   for(int k=0;k<K;k++)
   {
      float Plx=Px[0], Ply=Py[0], Plz=Pz[0]; int j=1; int L=N-1;
      for(int c=1;c<L;c++)
      {
         i   = idx[c];
         float P1x   = t*Plx + (1-t)*Px[i], 
               P1y   = t*Ply + (1-t)*Py[i],
               P1z   = t*Plz + (1-t)*Pz[i];
         Px[N] = (1-t)*Px[i] + t*Px[idx[c+1]]; 
         Py[N] = (1-t)*Py[i] + t*Py[idx[c+1]]; 
         Pz[N] = (1-t)*Pz[i] + t*Pz[idx[c+1]];
         Plx   = Px[i]; Ply = Py[i]; Plz = Pz[i];
         Px[i] = P1x; Py[i] = P1y; Pz[i] = P1z;
         nidx[j++] = i; nidx[j++] = N++;
      }
      nidx[N-1] = idx[L];
      int* tmp = idx; idx = nidx; nidx = tmp;
   }

   return idx;
}





void Geometry::rectLineClip(float x1, float y1, float x2, float y2, float& x3, float& y3, float& x4, float& y4)
{
    unsigned char code1,code2;
    for(;;)                                                             //iteratively clip...
    {
       rectLineIntersect(x1,y1,x2,y2,x3,y3,x4,y4,&code1,&code2);        //first determine the location of the points
                                                                        //w.r.t rectangle -> code1,code2
       if (!(code1|code2) || (code1&code2)) return;                     //trivial accept or reject, nothing to do

       if (code2)   //x4,y4 outside and x3,y3 surely inside
       {
         float delx = x3-x4, dely = y3-y4; 
         if (code2 & 8)      { y4 += (x1-x4)*dely/delx; x4 = x1; }
         else if (code2 & 2) { y4 += (x2-x4)*dely/delx; x4 = x2; }
         else if (code2 & 1) { x4 += (y1-y4)*delx/dely; y4 = y1; }
         else if (code2 & 4) { x4 += (y2-y4)*delx/dely; y4 = y2; }
       }
       else //code1 for sure, so x3,y3 outside and x4,y4 inside
       {
         float delx = x4-x3, dely = y4-y3; 
         if (code1 & 8)      { y3 += (x1-x3)*dely/delx; x3 = x1; }
         else if (code1 & 2) { y3 += (x2-x3)*dely/delx; x3 = x2; }
         else if (code1 & 1) { x3 += (y1-y3)*delx/dely; y3 = y1; }
         else if (code1 & 4) { x3 += (y2-y3)*delx/dely; y3 = y2; }
       }
    }
}


