#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "Node.h"
#include "Edge.h"
#include "Selection.h"
#include "AttrSet.h"
#include "Attr.h"
#include <iostream.h>

const int VMAX         = 10000;
const int GNAMESIZE    = 100;
const int PARSEBUFSIZE = 200;

static char  graphname[GNAMESIZE];

static const char *posx_name,*posy_name,*posz_name;

static int next_word(FILE *f, char *buf, int buflen)
{
   int   	i = 0;
   char 	ch = getc (f);

   while (isspace(ch)) ch = getc (f);

   if (ch == '"') 
   {
      ch = getc (f);
      while (!feof (f) && (ch != '"') && (i < buflen)) 
      { buf[i] = ch; ch = getc (f); i++; }
   }
   else 
   {
      while (!feof (f) && !isspace (ch) && (i < buflen) && (ch != '"')) 
      { buf[i] = ch; ch = getc (f); i++; }
      if (ch == '"')
	 ungetc (ch, f);
   }
   buf[i] = '\0';

   if (feof (f))
      return -1;
   else
      if (isspace (ch))  return i;
      else		 return 0;
}


int next_token(FILE *f, char *buf, int buflen)
{
   int ret = next_word (f, buf, buflen);
   while (strchr (buf, '{')) 
   {
      while (!strchr (buf, '}'))
	 ret = next_word (f, buf, buflen);
      ret = next_word (f, buf, buflen);
   }
   return ret;
}


static float* read_pos(FILE *f, Node* n)
//Reads NP-attrs from file 'f' into Node 'n'. Ignore all other attrs.
{
   static char  buf[PARSEBUFSIZE];
   static float xyz[3]; 

   next_word (f, buf, PARSEBUFSIZE);
   while (!strstr (buf, "NP"))			
      next_word (f, buf, PARSEBUFSIZE);

   next_word (f, buf, PARSEBUFSIZE);
   n->set(posx_name,xyz[0] = (float)atof(buf));
   next_word (f, buf, PARSEBUFSIZE);
   n->set(posy_name,xyz[1] = (float)atof(buf));
   n->set(posz_name,xyz[2] = 0);

   while (!strstr (buf, "$}"))
      next_word (f, buf, PARSEBUFSIZE);
   return xyz;
}

static float* read_pos(Selection* s, FILE *f, Node*& n)
//Reads NP and ID attrs from file f. Returns Node n in Graph g 
//with id==read-id. NP attrs are read into this Node.
{
   static char  buf[PARSEBUFSIZE];
   static float xyz[3]; 

   next_word (f, buf, PARSEBUFSIZE);
   for(;;)
   {
   while (!strstr (buf, "NP") && !strstr(buf,"ID") && !strstr(buf,"$}"))
      next_word (f, buf, PARSEBUFSIZE);

   if (strstr (buf, "$}")) break;

   if (strstr(buf,"NP"))			//NP attr found
   {
     next_word (f, buf, PARSEBUFSIZE);
     xyz[0] = (float)atof(buf);
     next_word (f, buf, PARSEBUFSIZE);
     xyz[1] = (float)atof(buf);
     xyz[2] = 0;
   }
   else if (strstr(buf,"ID"))			//ID attr found
   {
     next_word (f, buf, PARSEBUFSIZE);
     n = s->getNodeById(atoi(buf));
     n->set(posx_name,xyz[0]);
     n->set(posy_name,xyz[1]);
     n->set(posz_name,xyz[2]);
   }
   }
   return xyz;
}


int read_graphed(FILE *f, Selection* g, int max_id, int rdpos, const char* posx_name_,const char* posy_name_,
                 const char* posz_name_)
{
   char		buf[PARSEBUFSIZE];
   int		i, nw, u, v, vm, um, map[VMAX+1];
   Node*	nmap[VMAX+1];
   int 	        number_vertices=0;

   nw = next_token (f, buf, PARSEBUFSIZE);
   while ((nw >= 0) && strcmp ("GRAPH", buf) != 0)
      nw = next_token (f, buf, PARSEBUFSIZE);
   if (nw < 0) return 0;

   next_token(f, graphname, PARSEBUFSIZE);
   next_token(f, buf, PARSEBUFSIZE);
   next_token(f, buf, PARSEBUFSIZE);
   if ((strcmp(buf, "DIRECTED") == 0) || (strcmp(buf, "UNDIRECTED") == 0))
      next_token (f, buf, PARSEBUFSIZE);

   posx_name  = posx_name_;					//init the globals that communicate with read_pos()
   posy_name  = posy_name_;
   posz_name  = posz_name_;

   //3. Read the Graph from the file. Take care to offset read-ids with max_id.
   while (strcmp(buf,"END") != 0) 
   {
      v = atoi(buf); v += max_id;
      for (i = 1; (i <= number_vertices) && (map[i] != v); i++);
      if (i > number_vertices) 
      {
	 number_vertices++;					//new vertex discovered
	 nmap[number_vertices] = new Node(g,v);			//make new Node, record it in map[],nmap[]
	 map[number_vertices]  = v;
	 vm = number_vertices;
      }
      else vm = i;						//node exists at location 'vm' in map[],nmap[]

      if (rdpos) read_pos(f, nmap[vm]);				//if we wish to read the position-attribute, do it now

      next_token (f, buf, PARSEBUFSIZE);
      next_token (f, buf, PARSEBUFSIZE);
      while (!strchr(buf, ';')) 				//read the rest of stuff for this node (i.e. connected-nodes)
      {
	 u = atoi(buf); u += max_id;
	 for (i = 1; ((i <= number_vertices) && (map[i] != u)); i++) ;
	 if (i > number_vertices) 				
	 {							//new node discovered
	    number_vertices++;					//make new Node, record it in map[],nmap[]
	    nmap[number_vertices] = new Node(g,u);		//and name it with its idx-number
	    map[number_vertices]  = u;
	    um = number_vertices;
	 }
	 else um = i;						//node exists at location 'um' in map[],nmap[]

	 nmap[vm]->connectTo(nmap[um],g); 			//connect Nodes vm->um. By default, link is directional

	 next_token(f,buf,PARSEBUFSIZE);
	 if (!strchr(buf,';')) next_token(f,buf,PARSEBUFSIZE);
      }
      next_token(f,buf,PARSEBUFSIZE);
   }
   return 1;
}




int read_graphed_pos(FILE *f, Selection* g, const char* posx_name_,const char* posy_name_,
                     const char* posz_name_)
{
   char		buf[PARSEBUFSIZE];
   int		N=0;
   Node*	nmap[VMAX+1];   
   const float  MAXVAL = 1.0e8;
   float        minx = MAXVAL, maxx = -MAXVAL,
	        miny = MAXVAL, maxy = -MAXVAL,
		minz = MAXVAL, maxz = -MAXVAL; 
   const float  EPS_SIZE = 1.0e-4f;

   int nw = next_token (f, buf, PARSEBUFSIZE);
   while ((nw >= 0) && strcmp ("GRAPH", buf) != 0)
      nw = next_token (f, buf, PARSEBUFSIZE);
   if (nw < 0) return 0;

   next_token (f, graphname, PARSEBUFSIZE);
   next_token (f, buf, PARSEBUFSIZE);
   next_token (f, buf, PARSEBUFSIZE);
   if ((!strcmp (buf, "DIRECTED")) || (!strcmp (buf, "UNDIRECTED")))
      next_token (f, buf, PARSEBUFSIZE);

   posx_name  = posx_name_;			//init the globals that communicate with read_pos()
   posy_name  = posy_name_;
   posz_name  = posz_name_;

   //3. Read the node-positions from the file. 
   while (strcmp (buf, "END") != 0) 
   {
      Node*  n;
      float* xyz = read_pos(g,f,n);

      if (minx > xyz[0]) minx = xyz[0]; if (maxx < xyz[0]) maxx = xyz[0];
      if (miny > xyz[1]) miny = xyz[1]; if (maxy < xyz[1]) maxy = xyz[1];
      if (minz > xyz[2]) minz = xyz[2]; if (maxz < xyz[2]) maxz = xyz[2];

      nmap[N++] = n;	
      next_token (f, buf, PARSEBUFSIZE);
      next_token (f, buf, PARSEBUFSIZE);
      while (!strchr (buf, ';')) 		//skip the rest of stuff for this node (i.e. connected-nodes)
      {
	 atoi (buf);
	 next_token (f, buf, PARSEBUFSIZE);
	 if (!strchr (buf, ';'))
	    next_token (f, buf, PARSEBUFSIZE);
      }
      next_token (f, buf, PARSEBUFSIZE);
   }

   if (!N) return 1;
   float dx = maxx-minx, dy = maxy-miny, dz = maxz-minz;
   for(int i=0;i<N;i++)
   {
     nmap[i]->set(posx_name,(dx>EPS_SIZE)? (nmap[i]->getFloat(posx_name)-minx)/dx : 0.0f);
     nmap[i]->set(posy_name,(dy>EPS_SIZE)? (nmap[i]->getFloat(posy_name)-miny)/dy : 0.0f);
     nmap[i]->set(posz_name,(dz>EPS_SIZE)? (nmap[i]->getFloat(posz_name)-minz)/dz : 0.0f);
   }
   //!!! ALSO HERE COORDS ARE NORMALIZED IN 0..1,0..1 - IS THIS REALLY WHAT WE WANT???
   //
   //

   return 1;
}





int write_graphed(FILE *f, Selection* s,int wr_pos,int wr_ids,const char* posx,const char* posy) 
//
// Simple procedure that writes the nodes/edges in a selection to a GraphEd file.
// If wr_pos == 1, the integer attributes posx, posy are written as the GraphEd node positions.
// We assume these args exist for all nodes in selection. 
// Also, node-ids are written to ID-attrs in the file.
// 
{
   fprintf(f, "GRAPH \"%s\" = UNDIRECTED \n", "Unnamed");

   const Node* n; Selection::NodeIterator* i;  
   
   for(i=s->iterNodes();(n=i->next());)			//for all nodes in selection
   {
      if (wr_pos)					//write position args?
        if (wr_ids)					//write ID arg?
          fprintf(f,"%d {$NP %6f %6f ID %d$} \"\"", n->getId()+1, n->get(posx)->getFloat(), n->get(posy)->getFloat(),n->getId());
	else
	  fprintf(f,"%d {$NP %6f %6f} \"\"", n->getId()+1, n->get(posx)->getFloat(), n->get(posy)->getFloat());
      else						//write without position args
        if (wr_ids)					//write ID arg?
          fprintf(f,"%d {$NP 0 0 ID %d$} \"\"", n->getId()+1,n->getId());	
	else
	  fprintf(f,"%d {$NP 0 0} \"\"", n->getId()+1);		
							

      const EdgeSet& es = n->getEdges(s,Node::OUT);
      for(int j=0;j<es.Count();j++)			//for all edges of current node
      {
	 Edge* e = es[j];
	 if (!s->hasNode(e->toNode())) continue;	//skip writing edges going to nodes outside selection
	 fprintf(f, "\n%d \"\"", e->toNode()->getId()+1);	//!!! COULD BE ENHANCED to remove check hasNode
      }  
      fprintf(f, ";\n");
   }	 

   fprintf(f, "END\n");

   return 1;
}
