#include "IVGlyphFactory.h"
#include "Node.h"
#include "Edge.h"
#include "GUIManager.h"
#include "IVGlyphs.h"
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoVertexProperty.h>
#include <Inventor/nodes/SoCube.h>
#include "utils.h"
#include <math.h>



IVGlyphFactory::~IVGlyphFactory()
{  }

void IVGlyphFactory::save(FILE* fp)                     //Save this to Tcl-file (called by subclasses)
{
   fprintf(fp,"pr_glyphs new %s %s\n",getType(),(const char*)addQuotes(getName()));
}

void IVGlyphFactory::getTclDeps(std::set<STRING>&)      //Get Tcl-procs this uses (nop)
{  }

//-------------------  IVNodeGlyphFactory/IVEdgeGlyphFactory  -----------------------




//-------------------  IVBasicGlyphFactory and subclasses  ----------------------------------

SoNode* IVBasicNodeGlyphFactory::map(Node* n)
{  
   return IVGlyphs::defaultNode(n);
}

SoNode* IVBasicEdgeGlyphFactory::map(Edge* e,const char* posx,const char* posy,const char* posz)
{
   return IVGlyphs::defaultEdge(e,posx,posy,posz);
}






//-------------------  IVTclGlyphFactory and subclasses  ----------------------------------

IVTclGlyphFactory::IVTclGlyphFactory()
{  }


void IVTclGlyphFactory::save(FILE* fp,const char* opt,const char* nm) //save this's settings to Tcl-file (called by subclasses)
{
   fprintf(fp,"pr_glyph_factory [pr_glyphs get %s name %s] set cb %s\n",
          opt,
          (const char*)addQuotes(nm),
          (const char*)getTclCallback());
}

void IVTclGlyphFactory::getTclDeps(std::set<STRING>& s)          //enh inh
{
   const STRING& cb = getTclCallback();                          //This depends on its Tcl-callback-proc,
   if (cb!="") s.insert(cb);                                     //if it has one set      
}
   	

SoNode* IVTclNodeGlyphFactory::map(Node* n)
{
   static char cmd[1000];

   if (tcl_cb=="") return IVGlyphs::defaultNode(n);		//do we have any Tcl callback?

   sprintf(cmd,"%s %ld",(const char*)tcl_cb,(unsigned long)n);	//prepare calling Tcl callback with Node arg

   if (!GUIManager::exec(cmd))					//if Tcl callback somehow failed, return default
      return IVGlyphs::defaultNode(n);

   SoNode* ret = (SoNode*)GUIManager::getLongResult();		//get IV object back from Tcl callback's result
   if (!ret) ret = IVGlyphs::defaultNode(n);			//check if not NULL, i.e. if callback could make an IV object

   return ret;   						//return IV object to caller...
}

void IVTclNodeGlyphFactory::save(FILE* fp)
{
   IVNodeGlyphFactory::save(fp);                                //Ask 1st parent to save its info
   IVTclGlyphFactory::save(fp,"node_glyph",getName());          //Ask other parent to save its info
}
  
void IVTclNodeGlyphFactory::getTclDeps(std::set<STRING>& s)     //Ask IVTclGlyphFactory parent to get its Tcl-deps
{  IVTclGlyphFactory::getTclDeps(s);  } 


SoNode* IVTclEdgeGlyphFactory::map(Edge* e,const char* posx,const char* posy,const char* posz)
{
   static char cmd[1000];

   if (tcl_cb=="") return IVGlyphs::defaultEdge(e,posx,posy,posz);
								//do we have any Tcl callback?

   sprintf(cmd,"%s %ld %s %s %s",(const char*)tcl_cb,		//prepare calling Tcl callback with Edge arg
	   (unsigned long)e,posx,posy,posz);			//and position-attr names

   if (!GUIManager::exec(cmd))					//if Tcl callback somehow failed, return default
      return IVGlyphs::defaultEdge(e,posx,posy,posz);

   SoNode* ret = (SoNode*)GUIManager::getLongResult();		//get IV object back from Tcl callback's result
   if (!ret) ret = IVGlyphs::defaultEdge(e,posx,posy,posz);	//check if not NULL, just in case

   return ret;   						//return IV object to caller...
}

void IVTclEdgeGlyphFactory::save(FILE* fp)
{
   IVEdgeGlyphFactory::save(fp);                                //Ask 1st parent to save its info
   IVTclGlyphFactory::save(fp,"edge_glyph",getName());          //Ask other parent to save its info       
}
  
void IVTclEdgeGlyphFactory::getTclDeps(std::set<STRING>& s)     //Ask IVTclGlyphFactory parent to get its Tcl-deps
{  IVTclGlyphFactory::getTclDeps(s);  } 
