#include "IVGlyphs.h"
#include "Edge.h"
#include "Node.h"
#include "IVMapperImp.h"
#include "PathQualifier.h"
#include "SoCachedFile.h"
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoVertexProperty.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoScale.h>
#include <math.h>
#include <stdlib.h>

#include "UI_imp.h"
#include "IVUtils.h"




float        IVGlyphs::start_r,IVGlyphs::start_g,IVGlyphs::start_b;
float        IVGlyphs::end_r,IVGlyphs::end_g,IVGlyphs::end_b;
STRING       IVGlyphs::pos_x,IVGlyphs::pos_y,IVGlyphs::pos_z;
Edge*        IVGlyphs::edge=0;
SoNode*      IVGlyphs::def_icon=0;
SoNode*      IVGlyphs::def_node=0;
SoNode*	     IVGlyphs::def_cube=0;
SoMaterial*  IVGlyphs::material=0;
SoDrawStyle* IVGlyphs::draw_style=0;
float	     IVGlyphs::scale = 0.02f;				//global scale param




void IVGlyphs::initialize()					//Initialize various static class objects.
{								
   static float def_icon_coords[][3] = {{-1,0,0},{1,0,0},{0,2,0}};
   static int   def_icon_verts[]     = {0,1,2,-1};		//1. Initialize 'def_icon' to an icon for the edge-arrows

   if (def_icon) def_icon->unref();				//We delete any def_icon, this means initialize() was
								//previously called.
   SoGroup*       sep = new SoGroup;
   SoCoordinate3*   c = new SoCoordinate3;
   c->point.setValues(0,3,def_icon_coords);
   SoIndexedFaceSet* ls = new SoIndexedFaceSet;
   ls->coordIndex.setValues(0,4,def_icon_verts);
   sep->addChild(c);
   sep->addChild(ls);
   def_icon = sep;
   def_icon->ref();

   if (def_node) def_node->unref();				//2. Initialize 'def_node' to an icon for nodes	
   def_node = new SoCube;
   def_node->ref();

   if (def_cube) def_cube->unref();				//3. Initialize 'def_cube' to a cube-object	
   def_cube = new SoCube;
   def_cube->ref();
}



SoNode* IVGlyphs::cube(char* argv[])				//Makes a cube. The geometry is shared
{								//between all instances (see 'def_cube' initialization
   SoGroup* grp = base(); 					//in initialize()
   for(processStart();*argv;) processNodeDefaults(grp,argv);	
   grp->addChild(def_cube); 
   return grp;
}

SoNode* IVGlyphs::bar(char* argv[])				//Draws a histogram bar
{
   SoGroup* grp = base(); int n; float* val=0; float (*col)[3]=0;
   int nv=0,nh=0;						//nv = #bars stacked atop of each other (Z)
								//nh = #bars stacked along each other (XY)
   for(processStart();*argv;)
   {
      processNodeDefaults(grp,argv);				//Process default args
      if (!argv[0]) continue;
      if (!strcmp(argv[0],"nvalues"))				//Read # values to display
      {								//Allocate read-buffers for values and colors
	 argv++; if (!argv[0]) continue;
	 n = atoi(argv[0]); argv++; 
	 val = new float[n];
	 col = new float[n][3];
      }	
      else if (!strcmp(argv[0],"values"))			//Read 'nvalues' data values
      {
	 argv++; if (!argv[0]) continue;
	 for(int i=0;i<n;i++)
	 {
	    val[i] = atof(argv[0]); 
	    argv++; if (!argv[0]) break;
	 }
      }
      else if (!strcmp(argv[0],"colors"))			//Read 'nvalues' colors
      {
         argv++; if (!argv[0]) continue;
	 for(int i=0;i<n;i++)
	 {
	   for(int c=0;c<3;c++)
	   {
	      col[i][c] = atof(argv[0]);
	      argv++; if (!argv[0]) break;
	   }
	 }
      }
      else if (!strcmp(argv[0],"nv"))				//Read 'nv' vertical stacking
      {
	 argv++; if (!argv[0]) continue;
	 nv = atoi(argv[0]); argv++;
      }
      else if (!strcmp(argv[0],"nh"))				//Read 'nh' horizontal stacking
      {
	 argv++; if (!argv[0]) continue;
	 nh = atoi(argv[0]); argv++;
      }
   }

   if (!nv && !nh) nv=nh=1;                                     //If both nv and nh not given, default both to 1
   if (!nv) nv = ceil(float(n)/nh);	                        //If nv not given, determine it from n and nh
   if (!nh) nh = ceil(float(n)/nv);	                        //If nh not given, determine it from n and nv
   if (nv*nh<n) nh = ceil(float(n)/nv);				//If nv,nh s.th. not enough to draw n data values,
								//adjust nh and take nv as granted.

   float sz = 2.0/nh; 
   SoTranslation* t = new SoTranslation;			//Translate for the 1st bar
   t->translation = SbVec3f(-1+sz/2,0,val[0]/2);
   grp->addChild(t);
   for(int i=0;i<n;)						//Draw 'nh' horizontal groups of bars
   {				
       float dz=val[i]/2,z=0;
       for(int j=0;i<n && j<nv;i++,j++)				//Draw 'nv' bars atop of each other
       {
         SoBaseColor* c = new SoBaseColor;
	 c->rgb = SbColor(col[i][0],col[i][1],col[i][2]);
         grp->addChild(c);
         float ht = val[i];
         SoCube* cube = new SoCube;
         cube->width = sz; cube->height = sz; cube->depth = ht;
	 grp->addChild(cube);
	 if (i<n-1 && j<nv-1)					//If more bars to be drawn vertically,
	 {							//shift upwards and accumulate, in z,
	   SoTranslation* t = new SoTranslation;		//the total vertical shift we do
	   t->translation   = SbVec3f(0,0,dz + val[i+1]/2);
	   grp->addChild(t);
	   dz = val[i+1]/2; z += ht;
	 } else z += ht/2;					//Last vertical bar drawn
       }
       if (i<n)							//If more bars to be drawn horizontally,
       {							//shift downwards (back to z==0) and forwards
         SoTranslation* t = new SoTranslation;
         t->translation = SbVec3f(sz,0,val[i]/2-z);
	 grp->addChild(t);
       }
   }
   
   delete[] val; delete[] col;					//Ready with buffers

   return grp;
}


SoNode* IVGlyphs::file(char* argv[])				//Makes a node by reading an Inventor-file
{								//The file is actually read and cached (see SoCachedFile)
   SoGroup* grp = base(); grp->ref();				//so instances of the read scenegraphs are shared.
   STRING file;							

   for(processStart();*argv;)
   {
      processNodeDefaults(grp,argv);	
      if (!argv[0]) continue;
      if (!strcmp(argv[0],"file"))
      {
	 argv++; if (!argv[0]) continue;
	 file = argv[0]; 
	 argv++; if (!argv[0]) continue;
      }	
   }
 
   if (file=="") { grp->unref(); return 0; }

   STRING crtp = PathQualifier::setCurrentPath("");             //We must qualify the path we load the IV-files from.
   file = PathQualifier::qualify(file);                         //However, don't use the 'current path' for qualification
   PathQualifier::setCurrentPath(crtp+"/");         

   SoNode* ret = SoCachedFile::read(file);                      //The filename put/used for the SoCachedFile is now 
   if (!ret) { grp->unref(); return 0; }                        //the fully-qualified filename, so it is unique
   grp->addChild(ret);
   grp->unrefNoDelete();
   return grp;     
}


SoNode* IVGlyphs::sphere(char* argv[])
{
   SoGroup* grp = base(); 	

   for(processStart();*argv;)
       processNodeDefaults(grp,argv);

   SoSphere* c = new SoSphere;
   grp->addChild(c);
   return grp;
}


SoNode* IVGlyphs::line(char* argv[])
{
   SoGroup* grp = base();

   SoNode* icon=0; BOOL center_arrow=FALSE,end_arrow=FALSE;
   for(processStart();*argv;)
   {
      processEdgeDefaults(grp,argv);
      if (!argv[0]) continue;
      if (!strcmp(argv[0],"dir"))
      {
	 argv++; if (!argv[0]) continue;
	 center_arrow = (!strcmp(argv[0],"center_arrow"));
	 end_arrow    = (!strcmp(argv[0],"end_arrow"));
	 argv++;
      }	
      else
      if (!strcmp(argv[0],"icon"))
      {
	 argv++; if (!argv[0]) continue;		
	 icon = (SoNode*)atol(argv[0]);
	 argv++;
      }
   }
    
   SoLineSet*        edges       = new SoLineSet; 	 	
   SoVertexProperty* edge_coords = new SoVertexProperty;

   edges->numVertices.insertSpace(0,1);                 //make a single line
   edges->numVertices.setNum(1);                        //
   edges->vertexProperty = edge_coords;

   //Get coords of edge-points in v[]. We want to use edge-attrs, if these are set,
   //otherwise the edge-nodes' attrs. For this, we call the utility IVEdgeMapper::getEdgeCoords()
   static float v[500]; 

   int sz = IVEdgeMapper::currentActiveMapper()->getEdgeCoords(edge,pos_x,pos_y,pos_z,v);
   edge_coords->vertex.insertSpace(0,sz);                //the line will use 2 coords from 'edge_coords'
   edge_coords->vertex.setNum(sz);                       //and also 2 colors from 'edge_coords'
   edge_coords->orderedRGBA.insertSpace(0,sz);           //
   edge_coords->orderedRGBA.setNum(sz);
   edge_coords->materialBinding = SoVertexProperty::PER_VERTEX;
   edges->numVertices.set1Value(0,sz);     
   edge_coords->vertex.SoMFVec3f::setValues(0,sz,(float(*)[3])v); 

   for(int i=0;i<sz;i++)                                 //Linearly interpolate RGB colors of the edge-points
   {                                                     //between the given values for the start and end points
      float t = float(i)/(sz-1);
      float r = (1-t)*start_r + t*end_r;
      float g = (1-t)*start_g + t*end_g;
      float b = (1-t)*start_b + t*end_b;
      edge_coords->orderedRGBA.set1Value(i,makeRGBA(r,g,b));
   }

   grp->addChild(edges);
   grp->addChild(edge_coords);

   if (center_arrow || end_arrow)
   {
      SoTransform* r = new SoTransform;
      SbVec3f dir = edge_coords->vertex[1]-edge_coords->vertex[0];
      float   len = dir.length()*getScale();		
      SbVec3f cnt = (center_arrow)? (edge_coords->vertex[1]+edge_coords->vertex[0])/2 
				  : edge_coords->vertex[1];
      r->rotation.setValue(SbRotation(SbVec3f(0,1,0),dir));
      r->translation.setValue(cnt);
      r->scaleFactor.setValue(len,len,len);
      SoNode* ic = (icon)? icon : def_icon;
      SoBaseColor* col = new SoBaseColor;
      col->rgb = SbColor(1,1,1);
      SoVertexProperty* vp = new SoVertexProperty;
      vp->materialBinding = SoVertexProperty::OVERALL;
      grp->addChild(vp);
      grp->addChild(col);
      grp->addChild(r);
      grp->addChild(ic);
   }

   return grp; 
}

 	

SoGroup* IVGlyphs::base()
{
    SoGroup*  	  grp = new SoSeparator;
    return 	  grp;
}


void IVGlyphs::processStart()
{
   material = 0;
   draw_style = 0;
   start_r=start_g=start_b=end_r=end_g=end_b=1;
}


void IVGlyphs::processNodeDefaults(SoGroup* grp,char** &argv)
{
   if (!argv[0]) return;

   if (!strcmp(argv[0],"color"))
   {
      argv++; if (!argv[0]) return;
      float r = atof(argv[0]); argv++; if (!argv[0]) return;
      float g = atof(argv[0]); argv++; if (!argv[0]) return;
      float b = atof(argv[0]); argv++;
      if (!material) grp->addChild(material=new SoMaterial);
      material->diffuseColor = SbColor(r,g,b);
   }
   else
   if (!strcmp(argv[0],"style"))
   {
      argv++; if (!argv[0]) return;
      char* s = argv[0]; argv++;
      if (!draw_style) grp->addChild(draw_style=new SoDrawStyle);
      if (!strcmp(s,"filled"))	
	 draw_style->style = SoDrawStyle::FILLED;
      else if (!strcmp(s,"lines"))	
	 draw_style->style = SoDrawStyle::LINES;
      else if (!strcmp(s,"points"))	
	 draw_style->style = SoDrawStyle::POINTS;
      else if (!strcmp(s,"invisible"))	
	 draw_style->style = SoDrawStyle::INVISIBLE;
   }
   else 
   if (!strcmp(argv[0],"shading"))
   {
      argv++; if (!argv[0]) return;
      SoLightModel* lm = new SoLightModel;
      char* onoff = argv[0]; argv++;
      lm->model = (!strcmp(onoff,"on"))? SoLightModel::PHONG : SoLightModel::BASE_COLOR;
      grp->addChild(lm);
   }
   else
   if (!strcmp(argv[0],"width"))
   {
      argv++; if (!argv[0]) return;
      float w = atof(argv[0]); argv++;
      if (!draw_style) grp->addChild(draw_style=new SoDrawStyle);
      draw_style->lineWidth = w;
   }
   else
   if (!strcmp(argv[0],"scale"))
   {
      argv++; if (!argv[0]) return;
      float sx = atof(argv[0]); argv++; if (!argv[0]) return;
      float sy = atof(argv[0]); argv++; if (!argv[0]) return;
      float sz = atof(argv[0]); argv++;
      SoScale* s = new SoScale;
      grp->addChild(s);
      s->scaleFactor = SbVec3f(sx,sy,sz);
   }
   else
   if (!strcmp(argv[0],"text"))
   {
      argv++; if (!argv[0]) return; 
      char* txt = argv[0]; argv++;
      SoText2* t = new SoText2;
      grp->addChild(t);
      t->string = txt;
   }
}



SoNode*  IVGlyphs::defaultNode(Node*)			//The default node is shared for efficiency
{
   return def_node;
}

/*
SoNode*  IVGlyphs::defaultEdge(Edge* e,const char* pos_x,const char* pos_y,const char* pos_z)
{
   SoGroup*            sep 	 = new SoGroup;
   SoLineSet*        edges       = new SoLineSet; 	 	
   SoVertexProperty* edge_coords = new SoVertexProperty;

   edges->numVertices.insertSpace(0,1);
   edges->numVertices.setNum(1);
   edges->vertexProperty = edge_coords;
   edge_coords->vertex.insertSpace(0,2);
   edge_coords->vertex.setNum(2);

   Node* n1 = e->fromNode();
   Node* n2 = e->toNode();

   edges->numVertices.set1Value(0,2); 
   edge_coords->vertex.set1Value(0,n1->getFloat(pos_x),n1->getFloat(pos_y),n1->getFloat(pos_z));
   edge_coords->vertex.set1Value(1,n2->getFloat(pos_x),n2->getFloat(pos_y),n2->getFloat(pos_z));

   sep->addChild(edges);
   sep->addChild(edge_coords);
   return sep;
}
*/


SoNode*  IVGlyphs::defaultEdge(Edge* e,const char* pos_x,const char* pos_y,const char* pos_z)
{
   SoGroup*          sep         = new SoGroup; 
   SoLineSet*        edges       = new SoLineSet; 	 	
   SoVertexProperty* edge_coords = new SoVertexProperty;

   edges->numVertices.insertSpace(0,1);                 //make a single line
   edges->numVertices.setNum(1);                        //
   edges->vertexProperty = edge_coords;

   //Get coords of edge-points in v[]. We want to use edge-attrs, if these are set,
   //otherwise the edge-nodes' attrs. For this, we call the utility IVMapperImp::getEdgeCoords()
   static float v[500]; 
   IVEdgeMapper* mm = IVEdgeMapper::currentActiveMapper();
   int sz = mm->getEdgeCoords(e,pos_x,pos_y,pos_z,v);

   edge_coords->vertex.insertSpace(0,sz);                //the line will use 2 coords from 'edge_coords'
   edge_coords->vertex.setNum(sz);                       //and also 2 colors from 'edge_coords'
   edges->numVertices.set1Value(0,sz);     
   edge_coords->vertex.SoMFVec3f::setValues(0,sz,(float(*)[3])v); 

   sep->addChild(edges);
   sep->addChild(edge_coords);
   return sep; 
}


void IVGlyphs::processEdgeDefaults(SoGroup* grp,char** &argv)
{
   if (!argv[0]) return;

   if (!strcmp(argv[0],"color"))
   {
      argv++; if (!argv[0]) return;
      start_r = atof(argv[0]); argv++; if (!argv[0]) return;
      start_g = atof(argv[0]); argv++; if (!argv[0]) return;
      start_b = atof(argv[0]); argv++; if (!argv[0]) return;
      end_r = atof(argv[0]); argv++; if (!argv[0]) return;            
      end_g = atof(argv[0]); argv++; if (!argv[0]) return;
      end_b = atof(argv[0]); argv++; if (!argv[0]) return;
   }
   else
   if (!strcmp(argv[0],"style"))
   {
      argv++; if (!argv[0]) return;
      char* s = argv[0]; argv++;
      if (!draw_style) grp->addChild(draw_style=new SoDrawStyle);
      if (!strcmp(s,"filled"))	
	 draw_style->style = SoDrawStyle::FILLED;
      else if (!strcmp(s,"lines"))	
	 draw_style->style = SoDrawStyle::LINES;
      if (!strcmp(s,"POINTS"))	
	 draw_style->style = SoDrawStyle::POINTS;
      if (!strcmp(s,"invisible"))	
	 draw_style->style = SoDrawStyle::INVISIBLE;
   }
   else 
   if (!strcmp(argv[0],"width"))
   {
      argv++; if (!argv[0]) return;
      float w = atof(argv[0]); argv++;
      if (!draw_style) grp->addChild(draw_style=new SoDrawStyle);
      draw_style->lineWidth = w;
   }

   else if (!strcmp(argv[0],"x"))
   {
      argv++; if (!argv[0]) return;
      pos_x = argv[0]; argv++;
   }
   else if (!strcmp(argv[0],"y"))
   {
      argv++; if (!argv[0]) return;
      pos_y = argv[0]; argv++;
   }
   else if (!strcmp(argv[0],"z"))
   {
      argv++; if (!argv[0]) return;
      pos_z = argv[0]; argv++;
   }
   else if (!strcmp(argv[0],"edge"))
   {
      argv++; if (!argv[0]) return;
      edge = (Edge*)atol(argv[0]); argv++;
   }
}

