#include "IVMapperImp.h"
#include "IVMapper.h"
#include "IVGlyphFactory.h"
#include "GlyphFactoryList.h"
#include "IVGlyphs.h"
#include "GraphManager.h"
#include "SelList.h"
#include "Selection.h"
#include "Geometry.h"
#include "AttrSetType.h"
#include "Node.h"
#include "Edge.h"
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/details/SoLineDetail.h>
#include <Inventor/details/SoFaceDetail.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoVertexProperty.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/SoPath.h>
#include <Inventor/SoPickedPoint.h>
#include "glut.h"
#include "UI_imp.h"
#include "IVUtils.h"




IVHighlightMarker::IVHighlightMarker(): highlight_marker(0), owner(0)
{  }

IVHighlightMarker::~IVHighlightMarker()		//Dtor - releases contents of this
{  clear();  }


SoGroup* IVHighlightMarker::getSelMarker()	//make, on demand, the 'highlight_marker' field
{						//put inside some IV stuff that, if added before any 
   if (!highlight_marker)			//geometry, it makes it look 'selected'.
   {
	highlight_marker = new SoGroup;
	SoBaseColor* col = new SoBaseColor;
	float r,g,b; owner->mapper->getHighlightColor(r,g,b);
	col->rgb.setValue(r,g,b);		//selected objects are red. We force the override flag on the color	
	col->setOverride(TRUE);			//to supress any own color-settings the glyphs themselves may have
	highlight_marker->addChild(col);
	SoDrawStyle* ds = new SoDrawStyle;	//selected objects are thick wireframe
	ds->style = SoDrawStyle::LINES;
        ds->lineWidth = 2;
	highlight_marker->addChild(ds);
	SoLightModel* lm = new SoLightModel;	//selected objects are not shaded
	lm->model = SoLightModel::BASE_COLOR;
	highlight_marker->addChild(lm);
	highlight_marker->ref();		//we _own_ the highlight_marker s.th. it doesn't 
						//get destroyed when removed from scenegraph
   }
   return highlight_marker;
}

void IVHighlightMarker::clear()			//Releases contents of this (if any already created)
{  if (highlight_marker) { highlight_marker->unref(); highlight_marker = 0; } }


IVMapperImp::IVMapperImp(IVMapper* m): mapper(m)
{  root = new SoSeparator; root->ref();  }

IVMapperImp::~IVMapperImp()
{  root->unref(); }

std::map<Node*,int>& IVMapperImp::nodes()
{  return mapper->nodes; }

std::map<Node*,int>& IVMapperImp::hl_nodes()
{  return mapper->hl_nodes; }


std::map<Edge*,int>& IVMapperImp::edges()
{  return mapper->edges; }

std::map<Edge*,int>& IVMapperImp::hl_edges()
{  return mapper->hl_edges; }

SoGroup* IVMapperImp::getSceneGraph()
{  return root;  }


//---------------------------  IVMapperImp  ------------------------------------

void  IVMapperImp::map(Selection*) { }
void  IVMapperImp::clearHighlight(){ }		
void  IVMapperImp::highlight(int,BOOL) { }	
void  IVMapperImp::moveNode(Node*,int,float,float) { }		
void  IVMapperImp::moveHighlight(float,float) { }



int IVEdgeMapper::getEdgeCoords(Edge* edge,const char* pos_x,const char* pos_y,const char* pos_z,float* v)
//Extracts the edge-coords (x1,y1,z1,x2,y2,z2) in v[]. Uses the edge's own pos-attrs, if it has them,
//else uses the pos-attrs of the edge's nodes.
{
   Node* n1 = edge->fromNode(), *n2 = edge->toNode();
   float *vx=0,*vy=0,*vz=0; int sz,szx,szy,szz=0;

   IVMapper::EDGE_RESOLUTION edge_res = mapper->getEdgeResolution();
   int splineK; float splineT; mapper->getEdgeSmoothingParams(splineK,splineT);

   if (edge_res!=IVMapper::NODE_CENTERS)                        //If we explicitly want an edge between node-centers, 
   {                                                            //don't bother getting the edge-coords
     vx = edge->getFloatVec(pos_x,szx);                         //Look for two float-vec attrs calls pos_x,pos_y
     if (vx) vy = edge->getFloatVec(pos_y,szy);                 //Get size s.th. both x and y coords exist
     sz = (szx<szy)? szx : szy;                                 //Get separately the z coord-vec
     vz = edge->getFloatVec(pos_z,szz);
   }

   float z1 = n1->getFloat(pos_z), z2 = n2->getFloat(pos_z);    //Get z-coords of node-endpoints. We will need them later.

   if (vx && vy)                                                //If vx,vy exist (and we want sth else than NODE_CENTERS)
   { 
      int sz_est = (edge_res==IVMapper::STRAIGHT)?      2 :     //Evaluate how many vertices we'll generate
                   (edge_res==IVMapper::DEFAULT_RES)?  sz :
                   /*(edge_res==SMOOTH)*/ Geometry::splineSize(splineK,sz);  
      if (!v) return sz_est;                                    //Return v[] not given, simply report 'sz_est'

      if (edge_res==IVMapper::DEFAULT_RES)                      //If DEFAULT_RES, simply copy x,y(,z) coords to v[] 
        for(int i=0;i<sz;i++)                                   //(generate z via interpolation if not available)
        { 
           float t = float(i)/(sz-1);
           *v++ = vx[i]; *v++ = vy[i]; *v++ = (i<szz)? vz[i] : (1-t)*z1 + t*z2; 
        }
      else if (edge_res==IVMapper::STRAIGHT)                    //If STRAIGHT, use just the 1st and last x,y(,z) coords
        for(int i=0;i<sz;i+=sz-1)                               //(generate z via interpolation if not available)        
        { 
           float t = float(i)/(sz-1);
           *v++ = vx[i]; *v++ = vy[i]; *v++ = (i<szz)? vz[i] : (1-t)*z1 + t*z2; 
        }
      else  /*edge_res==SMOOTH*/                                //If SMOOTH, generate a spline based on the x,y(,z) coords
      {                                                         //and copy it into v[]        
         static float Px[300],Py[300],Pz[300];                  //(generate z via interpolation if not available)
         
         for(int i=0;i<sz;i++)                                  //1. Copy control-points into Px,Py,Pz to pass to spline()
         { 
            float t = float(i)/(sz-1);
            Px[i] = vx[i]; Py[i] = vy[i]; Pz[i] = (i<szz)? vz[i] : (1-t)*z1 + t*z2; 
         }
 
         int* idx = Geometry::spline(Px,Py,Pz,splineT,splineK,sz); //2. Call spline() to fill-in Px..Pz and return index-vector

         for(i=0;i<sz;i++)                                      //3. Pass generated points to caller in v[]
         { *v++ = Px[idx[i]]; *v++ = Py[idx[i]]; *v++ = Pz[idx[i]]; }
      }
   }
   else                                                         //If vx,vy not both found or want to use NODE_CENTERS
   {
     sz = 2;  
     if (!v) return sz;      
     v[0] = n1->getFloat(pos_x); v[3] = n2->getFloat(pos_x); 
     v[1] = n1->getFloat(pos_y); v[4] = n2->getFloat(pos_y);
     v[2] = z1;                  v[5] = z2; 
   }
   
   return sz;
}


void shiftEdge(Edge* e,Node* n,const char* pos_x,const char* pos_y,float dx,float dy)
//Updates pos-attrs of edge 'e' which has an endpoint 'n' which was shifted with dx,dy. 
//The other endpoint wasn't moved so we have to re-layout the edge. We do this simply by
//computing pos-attrs on a straight line between the 1st attr (corresponding to the unshifted
//endpoint) and the last attr (corresponding to the endpoint shifted with dx,dy). This guarantees that:
// - the edge's pos-attrs still start and end at the correct positions, taking node n's shifting into account
// - the number of edge-pos-attrs remains unchanged 
{
   int szx,szy; 
   float* vx = e->getFloatVec(pos_x,szx); 
   if (!vx) return;                                     //Any x pos-attrs?
   float* vy = e->getFloatVec(pos_y,szy);
   if (!vy) return;                                     //Any y pos-attrs?
   int    sz = (szx<szy)?szx:szy;                       //Determine overlap of vx,vy
   int p = (e->fromNode()==n)? 0 : sz-1;                //Determine attr-idx corresponding to shifted node 'n'
   vx[p] += dx; vy[p] += dy;                            //Shift pos-attr for idx 'p'
   float vx0 = vx[0],vy0 = vy[0],vxn = vx[sz-1],vyn = vy[sz-1];
   for(p=1;p<sz-1;p++)                                  //Linearly interpolate all other pos-attrs to account 
   {                                                    //for a straight line
     float t = float(p)/(sz-1);
     vx[p]   = vx0*t + vxn*(1-t);
     vy[p]   = vy0*t + vyn*(1-t);
     //!!! Must do the same for Z !!!
   }
}


void  IVNodeMapper::setNodeScale(float) { }	
void  IVNodeMapper::delNode(Node*,int) { }
Node* IVNodeMapper::pickNode(SoPickedPoint*,float,float,float) { return 0; }
void  IVEdgeMapper::delEdge(Edge*,int) { }
Edge* IVEdgeMapper::pickEdge(SoPickedPoint*,float,float,float) { return 0; }

IVNodeMapper* IVNodeMapper::active_mapper = 0;
IVEdgeMapper* IVEdgeMapper::active_mapper = 0;

void IVNodeMapper::map(Selection* s)
{  
   active_mapper = this;
   mapImp(s);
   active_mapper = 0;
}

void IVEdgeMapper::map(Selection* s)
{  
   active_mapper = this;
   mapImp(s);
   active_mapper = 0;
}

void IVNodeMapper::shiftNode(Node* n,float dx,float dy)
{
   const char* posx = mapper->getPosAttrX();
   n->set(posx,n->getFloat(posx)+dx);
   const char* posy = mapper->getPosAttrY();
   n->set(posy,n->getFloat(posy)+dy);
}

//----------------------------  IVFactoryNodeMapper  ---------------------------------

IVFactoryNodeMapper::IVFactoryNodeMapper(IVMapper* m)
		   : IVNodeMapper(m), cache_glyph_factory(0)
{  marker.setMapper(this);  }

void IVFactoryNodeMapper::clearHighlight()
{
   for(std::map<Node*,int>::iterator it = hl_nodes().begin();it!=hl_nodes().end();it++) 		
   {
     int idx = (*it).second;
     SoGroup* sel = getNodeSelect(idx);	
     if (sel->getNumChildren()) sel->removeChild(0);
   }
}

void IVFactoryNodeMapper::highlight(int idx,BOOL hl)
{
   SoGroup* sel = getNodeSelect(idx);				//get IV group managing highlight of 'nd'
   if (hl) sel->addChild(marker.getSelMarker());
   else	   sel->removeChild(0);
}

void IVFactoryNodeMapper::moveNode(Node* n,int idx,float dx,float dy)
{
   IVNodeMapper::shiftNode(n,dx,dy);

   float tx = n->getFloat(mapper->getPosAttrX()), 
	 ty = n->getFloat(mapper->getPosAttrY()), 
	 tz = n->getFloat(mapper->getPosAttrZ());
   SoTransform* t = getNodeTransform(idx);
   t->translation.setValue(tx,ty,tz);				//move node itself
}

void IVFactoryNodeMapper::moveHighlight(float dx, float dy)
{ 
   for(std::map<Node*,int>::iterator it = hl_nodes().begin();it!=hl_nodes().end();it++) 		
   {
         int idx = (*it).second;
	 Node* n = (*it).first;
	 float tx = n->getFloat(mapper->getPosAttrX())+dx;
	 float ty = n->getFloat(mapper->getPosAttrY())+dy;
	 n->set(mapper->getPosAttrX(),tx);
	 n->set(mapper->getPosAttrY(),ty);
	 SoTransform* t = getNodeTransform(idx);
         t->translation.setValue(tx,ty,0);
   }
}

void IVFactoryNodeMapper::setNodeScale(float scale)
{
   for(std::map<Node*,int>::iterator i=nodes().begin();i!=nodes().end();i++)  //do our own job
   {
	SoTransform* t = getNodeTransform((*i).second);		//modulate the node's own	
	Node* n = (*i).first;					//size-attrs (if they're there)
	float sx = n->getFloat(mapper->getSizeAttrX());		//with the global mapper scale
        float sy = n->getFloat(mapper->getSizeAttrY());
        float sz = n->getFloat(mapper->getSizeAttrZ());
        sx = (sx)? sx*scale : scale;
        sy = (sy)? sy*scale : scale;
        sz = (sz)? sz*scale : scale;
        t->scaleFactor.setValue(sx,sy,sz);
   }
}


void IVFactoryNodeMapper::mapImp(Selection* sel)
{
     root->removeAllChildren();			//clean up whatever we have constructed previously
     marker.clear();				//clear marker since map() might update its color/shape too

     //Get the IVNodeGlyphFactory by name. We do this here and not in mapNode() for speed.		
     cache_glyph_factory = GraphManager::getInstance()->getGlyphFactoryList()->getNGFactory(getNodeGlyphFactory());

     Selection::NodeIterator* i = sel->iterNodes();     
     for(Node* n;(n=i->next());)		//construct all node-reps for the selection
        root->addChild(place(n));		//make and place a new glyph for node 'n'
}


void IVFactoryNodeMapper::delNode(Node* n,int idx)
{	
   root->removeChild(idx);
}




void my_cb(void* p,SoAction* a)			//!!! TEST CODE for text-label drawing
{
	if (a->isOfType(SoGLRenderAction::getClassTypeId()))
	{	
	  glPushAttrib(GL_CURRENT_BIT|GL_LIGHTING_BIT);
	  glDisable(GL_LIGHTING);
	  glColor3f(1,1,1);
	  glRasterPos2f(0,-1);
	  const char* s = "Test!";
	  while(*s) 
	     glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, *s++);
	  glPopAttrib();
	}
	
}

SoSeparator* IVFactoryNodeMapper::place(Node* n)//make and place glyph for a given node
{
   SoSeparator* sep = new SoSeparator;		//this's graph
   SoTransform*   t = new SoTransform;		//this's translation and scaling
   SoGroup*     sel = new SoGroup;		//placeholder for the selection-marker
   SoNode*       sn = mapNode(n); 
   sep->addChild(t);				//1. add xform as 1st child - the order is important
						//as other methods rely on it
   sep->addChild(sel);				//2. add selection-placeholder as 2nd child

   float tx = n->getFloat(mapper->getPosAttrX());
   float ty = n->getFloat(mapper->getPosAttrY());
   float tz = n->getFloat(mapper->getPosAttrZ());
   t->translation.setValue(tx,ty,tz);		//translate node where its layout puts it

   float sx = n->getFloat(mapper->getSizeAttrX());
   float sy = n->getFloat(mapper->getSizeAttrY());
   float sz = n->getFloat(mapper->getSizeAttrZ());
   float sc  = mapper->getNodeScale();

   sx = (sx)? sx*sc : sc;
   sy = (sy)? sy*sc : sc;
   sz = (sz)? sz*sc : sc;
   t->scaleFactor.setValue(sx,sy,sz);		//scale node as the mapper says
  
   if (sn) sep->addChild(sn);			//3. add node-glyph as 3rd child, if any
  
   //SoCallback* cb = new SoCallback;		//!!!  TEST CODE for text-label drawing
   //cb->setCallback(my_cb,0);
   //sep->addChild(cb);

   return sep;  
}

Node* IVFactoryNodeMapper::pickNode(SoPickedPoint* pt,float,float,float)
{
   //REMARK: identifying the picked node is done by looking if its group is on the pick path.
   //	     we don't directly look for its glyph, since that one could be shared between several
   //        nodes. The group (one step above the glyph) is, however, unique.
   SoPath* p = pt->getPath();
   for(std::map<Node*,int>::iterator i=nodes().begin();i!=nodes().end();i++)
      if (p->containsNode(getNodeGroup((*i).second))) return (*i).first;
   return 0;
}

SoGroup* IVFactoryNodeMapper::getNodeGroup(int i)
{
   if (i>=nodes().size()) return 0;
   return (SoSeparator*)root->getChild(i);
}


SoTransform* IVFactoryNodeMapper::getNodeTransform(int i)
{
   if (i>=nodes().size()) return 0;
   SoSeparator* s = (SoSeparator*)root->getChild(i);
   return (SoTransform*)s->getChild(0);
}

SoGroup* IVFactoryNodeMapper::getNodeSelect(int i)
{
   if (i>=nodes().size()) return 0;
   SoSeparator* s = (SoSeparator*)root->getChild(i);
   return (SoGroup*)s->getChild(1);
}

SoNode* IVFactoryNodeMapper::mapNode(Node* n)		//simply delegate to this's GlyphFactory, if any
{
   return cache_glyph_factory->map(n);
}


IVNodeMapper::IVNodeMapper(IVMapper* m): IVMapperImp(m)
{    }

IVNodeMapper::~IVNodeMapper()
{   }


//---------  IVFactoryEdgeMapper  -----------------------------------------------------------------

IVFactoryEdgeMapper::IVFactoryEdgeMapper(IVMapper* m)
                   : IVEdgeMapper(m), cache_glyph_factory(0)
{ marker.setMapper(this); }

void IVFactoryEdgeMapper::clearHighlight()
{
   for(std::map<Edge*,int>::iterator it = hl_edges().begin();it!=hl_edges().end();it++) 		
   {
     int idx = (*it).second;
     SoGroup* sel = getEdgeSelect(idx);	
     if (sel->getNumChildren()) sel->removeChild(0);
   }
}

void IVFactoryEdgeMapper::highlight(int idx,BOOL hl)
{
   SoGroup* sel = getEdgeSelect(idx);				//get IV group managing highlight of 'nd'
   if (hl) sel->addChild(marker.getSelMarker());
   else	   sel->removeChild(0);
}

void IVFactoryEdgeMapper::moveNode(Node* n,int idx,float dx,float dy)	//Remaps edges that must be changed since a node was moved
{
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(mapper->getSelection()); 
   EdgeSet&   es  = n->getEdges(sel); 

   //Cache various stuff for speed		
   cache_glyph_factory = GraphManager::getInstance()->getGlyphFactoryList()->getEGFactory(getEdgeGlyphFactory());
   BOOL         hi_res = (mapper->getMoveStyle()==IVMapper::HIGH_RES);
   const char*   pos_x = mapper->getPosAttrX();
   const char*   pos_y = mapper->getPosAttrY();
   const char*   pos_z = mapper->getPosAttrZ();

   active_mapper = this;                                        //set active_mapper since mapEdge() will need it
   for(int j=0;j<es.Count();j++)				//remap/recompute coords for this node's edges
   {
      Edge* e = es[j];                                          
      if (hl_edges().find(e)==hl_edges().end())                 //If 'e' isn't a highlighted-edge, we must recompute its
      {  shiftEdge(e,n,pos_x,pos_y,dx,dy); continue;  }         //pos-attrs to account for the shifting of 'n', but we shouldn't
                                                                //remap it
        
      std::map<Edge*,int>::iterator i=edges().find(e);	        //Locate idx of edge
      if (i==edges().end()) continue;                           //Should not happen
      int idx = (*i).second;
      SoNode* eg = getEdgeGlyph(idx);                           //Locate Inventor-glyph of edge
      if (!eg) break;
      SoGroup* s = getEdgeGroup(idx);

      if (hi_res)	                                	//Move edges as full glyphs only in the
         s->replaceChild(eg,mapEdge(e));			//high-res style since this is slow
      else
         s->replaceChild(eg,IVGlyphs::defaultEdge(e,pos_x,pos_y,pos_z));
   }    
   active_mapper = 0;                                           //reset active_mapper since we're done

   

}



void IVFactoryEdgeMapper::moveHighlight(float dx,float dy)
{
   //Get the IVEdgeGlyphFactory by name. We do this here and not in mapEdge() for speed.		
   cache_glyph_factory = GraphManager::getInstance()->getGlyphFactoryList()->getEGFactory(getEdgeGlyphFactory());

   IVMapper::MOVE_STYLE mv = mapper->getMoveStyle();
   active_mapper = this;                                        //set active_mapper since we'll call mapEdge()
   for(std::map<Edge*,int>::iterator jt = hl_edges().begin();jt!=hl_edges().end();jt++)
   {								//move edges
      Edge* e = (*jt).first;
      
      int p,sz; float* v;                                       //shift the edge's coords since we moved both its nodes
      v = e->getFloatVec(mapper->getPosAttrX(),sz);             //we must do this here otherwise mapEdge() will 
      if (v) for(p=0;p<sz;p++) v[p] += dx;                      //incorrectly work on the old edge-coords
      v = e->getFloatVec(mapper->getPosAttrY(),sz);
      if (v) for(p=0;p<sz;p++) v[p] += dy;

      //!!! also update coords of all edges going to ONE node exactly in hl_nodes...
      //    this might be hard since we don't have knowledge of the node-mapper here

      if (mv==IVMapper::LOW_RES) continue;                      //in low-res, we only update coords, but remap nothing

      int idx = (*jt).second;
      SoNode* eg = getEdgeGlyph(idx);
      if (!eg) break;
      SoGroup* s = getEdgeGroup(idx); 
 
      if (mv==IVMapper::HIGH_RES)		                //Move edges as full glyphs only in the
         s->replaceChild(eg,mapEdge(e));			//high-res style since this is slow
      else 
         s->replaceChild(eg,IVGlyphs::defaultEdge(e,mapper->getPosAttrX(),mapper->getPosAttrY(),mapper->getPosAttrZ()));
   }  
   active_mapper = 0;                                           //Done with active_mapper, reset it
}

void IVFactoryEdgeMapper::mapImp(Selection* sel)
{
     root->removeAllChildren();			//clean up whatever we have constructed previously
     marker.clear();				//clear marker since map() might update its color/shape too

     //Get the IVEdgeGlyphFactory by name. We do this here and not in mapEdge() for speed.		
     cache_glyph_factory = GraphManager::getInstance()->getGlyphFactoryList()->getEGFactory(getEdgeGlyphFactory());

     Selection::EdgeIterator* i = sel->iterEdges();
     for(Edge* e;(e=i->next());)		//construct all node-reps for the selection
        root->addChild(place(e));		//make and place a new glyph for node 'n'
}

void IVFactoryEdgeMapper::delEdge(Edge*,int idx)
{  root->removeChild(idx);  }

IVEdgeMapper::IVEdgeMapper(IVMapper* m): IVMapperImp(m)
{  }

SoSeparator* IVFactoryEdgeMapper::place(Edge* e)	
{						//make and place glyph for all edges
   SoSeparator* sep = new SoSeparator;
   SoGroup*     sel = new SoGroup;
   SoNode*       se = mapEdge(e);
   sep->addChild(sel);				//1. add selection-placeholder as 1st child
  
   if (se) sep->addChild(se);			//2. add edge-glyph as 2nd child, if any
   return sep;
}

Edge* IVFactoryEdgeMapper::pickEdge(SoPickedPoint* pt,float,float,float)
{
   //REMARK: the same as for IVFactoryNodeMapper::pickNode()
   //
   SoPath* p = pt->getPath();
   for(std::map<Edge*,int>::iterator i=edges().begin();i!=edges().end();i++)
      if (p->containsNode(getEdgeGroup((*i).second))) return (*i).first;
   return 0;
}

SoGroup* IVFactoryEdgeMapper::getEdgeGroup(int i)
{
   if (i>=edges().size()) return 0; 
   return (SoSeparator*)root->getChild(i);
}

SoNode* IVFactoryEdgeMapper::getEdgeGlyph(int i)
{  
   if (i>=edges().size()) return 0;
   SoSeparator* s = (SoSeparator*)root->getChild(i);
   if (s->getNumChildren()<2) return 0;
   return s->getChild(1);
}

SoGroup* IVFactoryEdgeMapper::getEdgeSelect(int i)
{
   if (i>=edges().size()) return 0;
   SoSeparator* s = (SoSeparator*)root->getChild(i);
   return (SoGroup*)s->getChild(0);
}

SoNode* IVFactoryEdgeMapper::mapEdge(Edge* e)
{
   return cache_glyph_factory->map(e,mapper->getPosAttrX(),	//simply delegate to this's EdgeGlyphFactory
				     mapper->getPosAttrY(),
				     mapper->getPosAttrZ());	
}



IVEdgeMapper::~IVEdgeMapper()
{  }

//-------------  class IVBasicNodeMapper  -----------------------------------


IVBasicNodeMapper::IVBasicNodeMapper(IVMapper* m): IVNodeMapper(m)
{
   SoShapeHints* sh = new SoShapeHints;
   sh->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
   root->addChild(sh);
   vprop = new SoVertexProperty; root->addChild(vprop);
   vprop->materialBinding = SoVertexProperty::PER_VERTEX_INDEXED;
   faces = new SoIndexedFaceSet; root->addChild(faces);
   faces->vertexProperty = vprop;
}

void IVBasicNodeMapper::clearHighlight()
{
   unsigned int* col = vprop->orderedRGBA.startEditing();
   int N = vprop->orderedRGBA.getNum();
   for(int i=0;i<N;i++) *col++ = makeRGBA(1,1,1);
   vprop->orderedRGBA.finishEditing();
}

void IVBasicNodeMapper::mapImp(Selection* sel)
{
   vprop->vertex.insertSpace(0,8*sel->numNodes());		//Allocate coordinates and colors
   vprop->vertex.setNum(8*sel->numNodes());
   vprop->orderedRGBA.insertSpace(0,8*sel->numNodes());
   vprop->orderedRGBA.setNum(8*sel->numNodes());
   clearHighlight();						//Needed to initialize vprop->orderedRGBA
   
   faces->coordIndex.insertSpace(0,30*sel->numNodes());		//Allocate 'faces' SoIndexedFaceSet
   faces->coordIndex.setNum(30*sel->numNodes());
   nodeidx.clear();

   Selection::NodeIterator* j = sel->iterNodes(); Node* n;
   for(int idx=0;(n=j->next());idx++)				//construct mappings for all nodes in sel
      mapNode(n,idx);
}

void IVBasicNodeMapper::mapNode(Node* n,int idx)
{
   std::map<int,Node*>::value_type v(idx,n);			//add entry (idx,node) to nodeidx
   nodeidx.insert(v);						

   placeNode(n,idx);						//place and scale the node
   
   int* t = faces->coordIndex.startEditing();			//edit the 30 (= 6 faces * 5 entries per face) indexes
   t += 30*idx;							//needed for describing the cube

   const int db[] = {0,1,2,3,-1, 1,5,6,2,-1, 0,3,7,4,-1, 3,2,6,7,-1, 0,4,5,1,-1, 5,4,7,6,-1};   	
   for(int i=0;i<30;i++) t[i] = (db[i]==-1)? -1 : 8*idx + db[i];
   faces->coordIndex.finishEditing();
}

void IVBasicNodeMapper::highlight(int idx, BOOL hl)
{ 
   float r,g,b; mapper->getHighlightColor(r,g,b);
   unsigned int col = (hl)? makeRGBA(r,g,b) : makeRGBA(1,1,1);	//highlight/unhighlight it
   unsigned int*  c = vprop->orderedRGBA.startEditing();
   c += 8*idx;
   for(int i=0;i<8;i++) *c++ = col;
   vprop->orderedRGBA.finishEditing();
}


void IVBasicNodeMapper::moveNode(Node* n,int idx,float dx,float dy)
{
   IVNodeMapper::shiftNode(n,dx,dy);
   placeNode(n,idx);
}


void IVBasicNodeMapper::placeNode(Node* n,int idx)	                //updates position of node 
{                                                                       //(does nothing for its edges)
   float tx = n->getFloat(mapper->getPosAttrX());			//get position and scale of the
   float ty = n->getFloat(mapper->getPosAttrY());			//node to be mapped
   float tz = n->getFloat(mapper->getPosAttrZ());
   float sx = n->getFloat(mapper->getSizeAttrX());
   float sy = n->getFloat(mapper->getSizeAttrY());
   float sz = n->getFloat(mapper->getSizeAttrZ());
   float sc  = mapper->getNodeScale();					//combine mapper's scale with own node's size
   sx = (sx)? sx*sc : sc;
   sy = (sy)? sy*sc : sc;
   sz = (sz)? sz*sc : sc;

   SbVec3f* f = vprop->vertex.startEditing();				//edit the 8 coordinates in 'vprop'
   f += 8*idx; int i;							//that correspond to the cube-vertices for this node
									//don't allocate space, this has been done in map()
   const float cb[] = {-1,-1,-1, 1,-1,-1, 1,1,-1, -1,1,-1, 
	               -1,-1, 1, 1,-1, 1, 1,1, 1, -1,1, 1};
   for(i=0;i<24;i+=3,f++)						//build a default cube, then translate & scale it where
       f->setValue(sx*cb[i]+tx,sy*cb[i+1]+ty,sz*cb[i+2]+tz);		//the node says
   vprop->vertex.finishEditing();
}


void IVBasicNodeMapper::moveHighlight(float dx,float dy)		//move complete highlight-sel with (dx,dy)
{ 
   for(std::map<Node*,int>::iterator jt = hl_nodes().begin();jt!=hl_nodes().end();jt++)
   {									//move nodes
      Node* n = (*jt).first;
      int idx = (*jt).second;
      moveNode(n,idx,dx,dy);						
   } 
}
      
void  IVBasicNodeMapper::setNodeScale(float)			    //Setting a new node-scale means, in our case,
{								    //remapping the whole scenegraph
   map(GraphManager::getInstance()->getSelList()->getSelection(mapper->getSelection()));
}

void IVBasicNodeMapper::delNode(Node* n,int idx)		    //Delete rep of Node n 	
{								    //i.e. delete its vertices, faces,
  vprop->vertex.deleteValues(8*idx,8);				    //and colors
  vprop->orderedRGBA.deleteValues(8*idx,8);
  faces->coordIndex.deleteValues(30*idx,30);			    //
  nodeidx.erase(nodeidx.find(idx));
}

Node*  IVBasicNodeMapper::pickNode(SoPickedPoint* pt,float,float,float)
{
   SoPath* p = pt->getPath();

   if (!p->containsNode(faces)) return 0;			    //first see if we picked any face
   if (!pt->getDetail()->isOfType(SoFaceDetail::getClassTypeId()))  //next check we picked indeed a SoIndexedFaceSet
       return 0;						    //(maybe this check is redundant but we do it)	

   
   SoFaceDetail* fd = (SoFaceDetail*)pt->getDetail();		    //ok, now get the face-index of the SoFaceDetail.
								    //since we know we have only a SoIndexedFaceSet, this idx is	
								    //the idx of the picked face. We go from face-idex to node-index by
								    //dividing by 6 since a cube has 6 faces. We use the 'edgeidx' ma[
								    //to go back from idx to Edge and return the latter.
   int idx = fd->getFaceIndex()/6;
   std::map<int,Node*>::iterator it = nodeidx.find(idx);
   return (it!=nodeidx.end())? (*it).second : 0;
}


//-------------------------------  IVBasicEdgeMapper  --------------------------------------------------


IVBasicEdgeMapper::IVBasicEdgeMapper(IVMapper* m): IVEdgeMapper(m)
{
   vprop = new SoVertexProperty; root->addChild(vprop);
   vprop->materialBinding = SoVertexProperty::PER_VERTEX;
   lines = new SoLineSet; root->addChild(lines);
   lines->vertexProperty = vprop;
}

void IVBasicEdgeMapper::clearHighlight()			//clearing all highlight
{								//means changing the color-property
                                                	        //of all edge-vertices to the unhighlighted color
   unsigned int* col = vprop->orderedRGBA.startEditing();
   int N = vprop->orderedRGBA.getNum();
   for(int i=0;i<N;i++) *col++ = makeRGBA(1,1,1);
   vprop->orderedRGBA.finishEditing();
}

void IVBasicEdgeMapper::mapImp(Selection* sel)			//Maps all edges in given Selection
{	
     lines->numVertices.insertSpace(0,sel->numEdges());		//
     lines->numVertices.setNum(sel->numEdges());		//Allocate coordinates and colors

     int idx; Edge* e;
     const char *posx = mapper->getPosAttrX(), *posy = mapper->getPosAttrY(), *posz = mapper->getPosAttrZ(); 
     int st = 0;

     start.InitC(sel->numEdges());                              //Allocate start[]
     Selection::EdgeIterator* j = sel->iterEdges();
     for(idx=0;(e=j->next());idx++)				//Set start[] to start-idxs, in vprop[], of the
     {                                                          //lines for edges. For this, we call getEdgeCoords() to estimate
        start[idx] = st;                                        //how many vertices each edge will have
        int sz     = IVEdgeMapper::currentActiveMapper()->getEdgeCoords(e,posx,posy,posz);
        lines->numVertices.set1Value(idx,sz);
        st += sz;
     }

     vprop->vertex.insertSpace(0,st);                           //Insert space for all edges
     vprop->vertex.setNum(st);
     vprop->orderedRGBA.insertSpace(0,st);
     vprop->orderedRGBA.setNum(st);
     clearHighlight();						//Needed to initialize vprop->orderedRGBA
     edgeidx.clear();						//Clear any stuff in the edgeidx map

     j = sel->iterEdges();
     for(idx=0;(e=j->next());idx++)				//Map each edge at its location
     {
	std::map<int,Edge*>::value_type v(idx,e);		//add entry (idx,edge) to edgeidx
        edgeidx.insert(v);						
	mapEdge(e,idx);	
     }	
}

void IVBasicEdgeMapper::highlight(int idx, BOOL hl)		//simply change the color-property
{								//of all the vertices of the given line to
   float r,g,b; mapper->getHighlightColor(r,g,b);		//highlight/unhighlight it
   unsigned int col = (hl)? makeRGBA(r,g,b) : makeRGBA(1,1,1);	
   
   int pos = start[idx];                                        //determine which are the vertices of line 'idx'  
   int end = (idx<start.Count()-1)? start[idx+1]                //they go from start[idx] till start[idx+1] or
             : vprop->orderedRGBA.getNum();                     //till the end of the vertex-array if this is the last line
   for(int i=pos;i<end;i++)
      vprop->orderedRGBA.set1Value(i,col);
}

void IVBasicEdgeMapper::moveNode(Node* n,int idx,float dx,float dy)
{
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(mapper->getSelection());
   EdgeSet&   es  = n->getEdges(sel); 

   active_mapper = this;
   const char* pos_x = mapper->getPosAttrX();
   const char* pos_y = mapper->getPosAttrY();
   for(int j=0;j<es.Count();j++)
   {
      Edge* e = es[j];                                                          
      if (hl_edges().find(e)==hl_edges().end())                 //If 'e' isn't a highlighted-edge, we must recompute its
         shiftEdge(e,n,pos_x,pos_y,dx,dy);                      //pos-attrs to account for the shifting of 'n', but we shouldn't remap it
      else                                                      
      {
         std::map<Edge*,int>::iterator i=edges().find(e);	  
         if (i==edges().end()) continue;
         int idx = (*i).second;
         mapEdge(e,idx);
      }
   }    
   active_mapper = 0;
}

void IVBasicEdgeMapper::moveHighlight(float dx,float dy)
{
   active_mapper = this;
   IVMapper::MOVE_STYLE mv =  mapper->getMoveStyle();
   for(std::map<Edge*,int>::iterator jt = hl_edges().begin();jt!=hl_edges().end();jt++)
   {								    //move edges
      Edge* e = (*jt).first;
      
      int p,sz; float* v;                                           //shift the edge's coords of since we moved both its nodes
      v = e->getFloatVec(mapper->getPosAttrX(),sz);                 //we must do this here otherwise mapEdge() will 
      if (v) for(p=0;p<sz;p++) v[p] += dx;                          //incorrectly work on the old edge-coords
      v = e->getFloatVec(mapper->getPosAttrY(),sz);
      if (v) for(p=0;p<sz;p++) v[p] += dy;

      //!!! also update coords of all edges going to ONE node exactly in hl_sel...
      //    this might be hard since we don't have knowledge of the node-mapper here

      if (mv==IVMapper::LOW_RES) continue;                          //In low-res, only update coords but remap nothing
      
      int idx = (*jt).second;
      mapEdge(e,idx);
   } 
   active_mapper = 0;
}


void IVBasicEdgeMapper::delEdge(Edge*,int idx)			    //Erase Edge given as (ptr,idx)
{
  int sz = (idx<start.Count()-1)? start[idx+1]-start[idx]           //Determine # vertices of current line    
           : vprop->vertex.getNum()-start[idx]; 
  vprop->vertex.deleteValues(start[idx],sz);		            //Erase the vertices of the edge
  vprop->orderedRGBA.deleteValues(start[idx],sz);                   //Erase the colors of the edge
  lines->numVertices.deleteValues(idx,1);			    //Erase the line for the edge
  edgeidx.erase(edgeidx.find(idx));				    //Erase the entry (idx,edge)
  start.Delete(idx);                                                //Erase the edge-vertex-count for this edge
}


Edge* IVBasicEdgeMapper::pickEdge(SoPickedPoint* pt,float,float,float)
{
   SoPath* p = pt->getPath();

   if (!p->containsNode(lines)) return 0;			    //first see if we picked any edge-line
   if (!pt->getDetail()->isOfType(SoLineDetail::getClassTypeId()))  //next check we picked indeed a SoLineSet
       return 0;						    //(maybe this check is redundant but we do it)	

   SoLineDetail* ld = (SoLineDetail*)pt->getDetail();		    //ok, now get the 1st-point-index of the SoLineDetail.
   int c0 = ld->getPoint0()->getCoordinateIndex(); 		    //since we know we have only a SoLineSet, this idx is
								    //the idx of a vertex of the picked edge in its SoVertexProperty. 
   int lt=0,rt=start.Count()-1;                         
   while(lt<rt)                                                     //find idx of edge containing vertex 'c0'. This is done by simple    
   {                                                                //bisection-search for the interval (start[i],start[i+1]) that   
      int m = (lt+rt)/2;                                            //contains 'c0'. We can do this since start[] is sorted in ascending
      if (c0<start[m]) rt=m-1; else lt=m+1;                         //order (see construction).
      if (c0==start[m]) break;
   }
   if (lt==start.Count() || start[lt]>c0) lt--;                     //adjust 'lt' to be exactly the edge-idx
   std::map<int,Edge*>::iterator it = edgeidx.find(lt);             //use edgeidx-map to go from edge index to Edge*.    
   return (it!=edgeidx.end())? (*it).second : 0;
}

void IVBasicEdgeMapper::mapEdge(Edge* e,int idx)		    //Makes an edge and puts it in this's scenegraph
{
   //Get n triplets of xyz-coords of edge in v[]. We want to use edge-attrs, if these are set,
   //otherwise the edge-nodes' attrs. For this, we call the utility IVMapperImp::getEdgeCoords()
   
   static float v[1000]; 
   int n = IVEdgeMapper::currentActiveMapper()->getEdgeCoords(e,mapper->getPosAttrX(),mapper->getPosAttrY(),mapper->getPosAttrZ(),v);

   idx = start[idx];                                                //Determine where in the vertex-array does the current
                                                                    //line segment start. Space is already allocated (see map())

   vprop->vertex.SoMFVec3f::setValues(idx,n,(float(*)[3]) v);       //Finally, set the position of the line segment in
                                                                    //the Inventor scene graph		    
}





//-------------  class IVSplatNodeMapper  -----------------------------------


IVSplatNodeMapper::IVSplatNodeMapper(IVMapper* m): IVNodeMapper(m), 
                   verts_per_dim(80),splat(0),splat_sz(0)
{
   SoShapeHints* sh = new SoShapeHints;
   sh->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
   root->addChild(sh);
   vprop = new SoVertexProperty; root->addChild(vprop);
   vprop->materialBinding = SoVertexProperty::PER_VERTEX_INDEXED;
   mesh = new SoQuadMesh; root->addChild(mesh);
   mesh->vertexProperty = vprop;
}

void IVSplatNodeMapper::clearHighlight()
{
   //!!! to be done
}

void IVSplatNodeMapper::highlight(int idx, BOOL hl)
{ 
   //!!! to be done
}

void IVSplatNodeMapper::moveHighlight(float dx,float dy)		//move complete highlight-sel with (dx,dy)
{ 
   //!!! to be done
}
   
void  IVSplatNodeMapper::setNodeScale(float)			    //Setting a new node-scale means, in our case,
{								    //remapping the whole scenegraph
   map(GraphManager::getInstance()->getSelList()->getSelection(mapper->getSelection()));
}

void IVSplatNodeMapper::moveNode(Node* n,int idx,float dx,float dy)  //Moving a node means remapping the whole scenegraph   
{
   IVNodeMapper::shiftNode(n,dx,dy);
   map(GraphManager::getInstance()->getSelList()->getSelection(mapper->getSelection()));
   //!!!could be sped up
}

Node*  IVSplatNodeMapper::pickNode(SoPickedPoint* pt,float,float,float)
{
   //!!! to be done
   return 0;
}

void IVSplatNodeMapper::delNode(Node* n,int idx)		    //Delete rep of Node n 	
{								    //So far we do a complete remap
  map(GraphManager::getInstance()->getSelList()->getSelection(mapper->getSelection()));
  //!!!could be sped up
}


IVSplatNodeMapper::~IVSplatNodeMapper()
{  delete[] splat;  }

void IVSplatNodeMapper::mapImp(Selection* sel)                  
{
   Geometry::bbox2D(sel,mapper->getPosAttr(),xm,ym,xM,yM);      //Get 2D bbox of nodes
   
   if (xM-xm<1.0e-6 && yM-ym<1.0e-6)                            //Initialize verts_per_col and verts_per_row s.th.
      verts_per_col = verts_per_row = 2;                        //the mesh has approximately square quads. This is done
   else                                                         //by setting the verts_per_longest_dim to verts_per_dim.
   if (xM-xm>yM-ym)                                             //The case when the bbox is degenerate is treated separately.
   {                                                            
      verts_per_row = verts_per_dim;                            
      verts_per_col = (yM-ym)*verts_per_dim/(xM-xm);
      l2c           = verts_per_dim/(xM-xm);
   }
   else //yM-ym>xM-xm
   {
      verts_per_col = verts_per_dim;
      verts_per_row = (xM-xm)*verts_per_dim/(yM-ym);
      l2c           = verts_per_dim/(yM-ym);
   }

   int nv = verts_per_col*verts_per_row;
   if (splat_sz != nv)
   {
     vprop->vertex.insertSpace(0,nv);	                        //Allocate coordinates and colors
     vprop->vertex.setNum(nv);                                  //if we resized anything
     vprop->orderedRGBA.insertSpace(0,nv);
     vprop->orderedRGBA.setNum(nv);
   }
   
   int i;

   SbVec3f*      f = vprop->vertex.startEditing();              //Initialize the mesh coords
   float y = ym, dy = (yM-ym)/verts_per_col;                    //to the regular structure in
   for(i=0;i<verts_per_col;i++)                                 //the z=0 plane determined by
   {                                                            //the nodes' 2D bbox
      float x = xm, dx = (xM-xm)/verts_per_row;                 
      for(int j=0;j<verts_per_row;j++)
      { f->setValue(x,y,0); f++; x += dx; }
      y += dy;
   }
   vprop->vertex.finishEditing();
   mesh->verticesPerColumn = verts_per_col;                     //Set the mesh size
   mesh->verticesPerRow    = verts_per_row;

   if (splat_sz!=nv)                                            //Initialize the work-array splat[] only if
      splat = new float[splat_sz=nv];                           //we need a different size than what we already have.
   splat_max=0;                                                 //(this to prevent frequent memo allocations0;
   for(i=0;i<nv;i++) splat[i] = 0;                              //We compute splat[] and
                                                                //not the RGBA colors directly since incremental accumulation 
                                                                //of floats is much more accurate than color accumulation.

   Selection::NodeIterator* j = sel->iterNodes(); Node* n;
   for(int idx=0;(n=j->next());idx++)				//construct mappings for all nodes in sel
      mapNode(n,idx);                                           //(this fills splat[] in)

   unsigned int* c = vprop->orderedRGBA.startEditing();         //map splat[] to RGBA scalars in the mesh
   for(i=0;i<nv;i++)
   {
      float s = splat[i]/splat_max;                             //'s' is now normalized between 0 and 1
      float r,g,b;
      rainbowColormap(s,r,g,b);                                 //map 's' via the colormap to RGB
      *c++ = makeRGBA(r,g,b);                                   //assign RGB colors to the mesh
   }

   vprop->orderedRGBA.finishEditing();                          //ready updating colors  
}

void IVSplatNodeMapper::mapNode(Node* n,int idx)
{
   placeNode(n,idx);						//place and scale the node
}

void IVSplatNodeMapper::placeNode(Node* n,int idx)	                //updates position of node 
{                                                                       //(does nothing for its edges)
   float tx = n->getFloat(mapper->getPosAttrX());			//get position and scale of the
   float ty = n->getFloat(mapper->getPosAttrY());			//node to be mapped
   float tz = n->getFloat(mapper->getPosAttrZ());
   float sx = n->getFloat(mapper->getSizeAttrX());
   float sy = n->getFloat(mapper->getSizeAttrY());
   float sz = n->getFloat(mapper->getSizeAttrZ());
   float sc  = mapper->getNodeScale();					//combine mapper's scale with own node's size
   sx = (sx)? sx*sc : sc;                                               //this will tell the size of the splat for this node
   sy = (sy)? sy*sc : sc;
   sz = (sz)? sz*sc : sc;

   int cx     = int(0.5+verts_per_row*((xM-xm > 1.0e-5)? (tx-xm)/(xM-xm) : 0.5)); //determine mesh-coords of splat-center
   int cy     = int(0.5+verts_per_col*((yM-ym > 1.0e-5)? (ty-ym)/(yM-ym) : 0.5));        
   
   const float splat_min = 0.007;                                       //splat_min = min value we want to accumulate in a splat
   float    a = sx*sy/4;                                                //a         = approx. area of splat
   int cell_x = int(l2c*sqrt(-a*log(splat_min))) + 1;                   //cell_x,_y = radius of splat, in cells, until its
   int cell_y = cell_x;                                                 //            value decreases under splat_min, plus 1 (for safety)
   a         = l2c*l2c*a;                                               //a         = now approx. area of splat, in cells

   for(int i=cy-cell_y;i<=cy+cell_y;i++)                               //perform splat by sampling exp-func over subgrid of
   {                                                                   //cell_x*cell_y cells. 
     if (i<0 || i>=verts_per_col) continue;
     for(int j=cx-cell_x;j<=cx+cell_x;j++)
     {
       if(j<0 || j>=verts_per_row) continue;
       float d2 = (cx-j)*(cx-j)+(cy-i)*(cy-i);                         //d2 = sqr-dist from splat center to crt-point 
       float v;
       v = (splat[i*verts_per_row + j] += exp(-d2/a)); //!!!add fact before exp...
       if (splat_max<v) splat_max=v;                                   //update splat_max, needed for later normalization 
     }
   }
}
  

   
