#include "IVUtils.h"
#include <Inventor/SbLinear.h>
#include "minmax.h"
#include "glut.h"
#include <math.h>



SbBool IVViewVolumeIntersect(const SbViewVolume& v,const SbVec3f& point)
//This is the original Open Inventor implementation of SbViewVolume::intersect(SbVec3f)
{
	SbVec3f llfO = v.llf-v.projPoint;
	SbVec3f lrfO = v.lrf-v.projPoint;
	SbVec3f ulfO = v.ulf-v.projPoint;

    // Transform point to origin:
    SbVec3f pt = point - v.projPoint;

    // Compare the point with all 6 planes of the view volume
    SbVec3f origin(0,0,0);

    if (v.type == SbViewVolume::PERSPECTIVE) 
	{
	// Left plane is formed from origin, llfO, ulfO
	SbPlane leftPlane(origin, llfO, ulfO);
	if (! leftPlane.isInHalfSpace(pt)) return FALSE;

	// Figure out urf point:
	SbVec3f urfO = lrfO + (ulfO - llfO);
	// Right is origin, urfO, lrfO
	SbPlane rightPlane(origin, urfO, lrfO);
	if (! rightPlane.isInHalfSpace(pt)) return FALSE;

	// Near is lrfO, llfO, ulfO:
	SbPlane nearPlane(lrfO, llfO, ulfO);
	if (! nearPlane.isInHalfSpace(pt)) return FALSE;

	// Far is near points in opposite order, translated to far plane:
	SbVec3f farOffset = v.nearToFar * v.projDir;
	SbPlane farPlane(ulfO+farOffset, llfO+farOffset, lrfO+farOffset);
	if (! farPlane.isInHalfSpace(pt)) return FALSE;

	// Bottom is origin, lrfO, llfO
	SbPlane bottomPlane(origin, lrfO, llfO);
	if (! bottomPlane.isInHalfSpace(pt)) return FALSE;

	// Finally, top is origin, ulfO, urfO
	SbPlane topPlane(origin, ulfO, urfO);
	if (! topPlane.isInHalfSpace(pt)) return FALSE;
    }
    else 
	{			// type == ORTHOGRAPHIC
	// Left plane is formed from llfO, lff+projDir, ulfO
	SbPlane leftPlane(llfO, llfO+v.projDir, ulfO);
	if (! leftPlane.isInHalfSpace(pt)) return FALSE;

	// Figure out urfO point:
	SbVec3f urfO = lrfO + (ulfO - llfO);
	// Right is urfO+projDir, lrfO, urfO
	SbPlane rightPlane(urfO+v.projDir, lrfO, urfO);
	if (! rightPlane.isInHalfSpace(pt)) return FALSE;

	// Near is lrfO, llfO, ulfO:
	SbPlane nearPlane(lrfO, llfO, ulfO);
	if (! nearPlane.isInHalfSpace(pt)) return FALSE;

	// Far is near points in opposite order, translated to far plane:
	SbVec3f farOffset = v.nearToFar * v.projDir;
	SbPlane farPlane(ulfO+farOffset, llfO+farOffset, lrfO+farOffset);
	if (! farPlane.isInHalfSpace(pt)) return FALSE;

	// Bottom is lrfO, lrfO+projDir, lrfO, llfO
	SbPlane bottomPlane(lrfO, lrfO+v.projDir, llfO);
	if (! bottomPlane.isInHalfSpace(pt)) return FALSE;

	// Finally, top is urfO, ulfO, ulfO+projDir
	SbPlane topPlane(urfO, ulfO, ulfO+v.projDir);
	if (! topPlane.isInHalfSpace(pt)) return FALSE;
    }

    return TRUE;
}


STRING rgb2tkcolor(float r,float g,float b)			//Make a Tk-compliant #rrggbb color spec
{								//from a RGB color spec
   static char buf[10];
   r = (r<0)? 0 : (r>1)? 1 : r;
   g = (g<0)? 0 : (g>1)? 1 : g;
   b = (b<0)? 0 : (b>1)? 1 : b; 
   sprintf(buf,"#%02x%02x%02x",int(r*255),int(g*255),int(b*255));
   return buf;
}


int tkcolor2rgb(const char* c,float& r,float& g,float& b)	//Extracts RGB components of Tk-compliant color spec	
{								//The Tk color can come as a 3,6,or 9 char string, i.e.
   if (!c || !c[0] || c[0]!='#') return 0;			//as a 4,8 or 16 bit RGB color spec.

   c++; int s = strlen(c); int n=0,R,G,B;
   if (s==3)
      n = sscanf(c,"%1x%1x%1x",&R,&G,&B);
   else if (s==6)
      n = sscanf(c,"%02x%02x%02x",&R,&G,&B);
   else if (s==9)
      n = sscanf(c,"%02x%02x%02x",&R,&G,&B); 
   if (n!=3) return 0;

   float m = float(pow(16,s/3)-1);				//Normalize components in 0..1
   r = R/m; g =G/m; b = B/m;

   return 1;
}


void rainbowColormap(float value,float& R,float& G,float& B)    //Returns v mapped via a standard rainbow colormap
{                                                               //as three RGB values.
   const float dx = 0.8;
   value = (value<0.0)? 0 : (value>1.0)? 1 : value;
   R = max(0.0,(3-fabs(value-4)-fabs(value-5))/2);
   G = max(0.0,(4-fabs(value-2)-fabs(value-4))/2);
   B = max(0.0,(3-fabs(value-1)-fabs(value-2))/2);
}


void glutBitmapString(float x,float y,const char* t)
{
   glRasterPos2f(x,y);
   while(*t) 
         glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, *t++);
}

int glutStringLength(const char* t)
{
    int r=0;
    while(*t) r += glutBitmapWidth(GLUT_BITMAP_HELVETICA_10,*t++);
    return r;
}

int glutStringHeight(const char*)                               //This routine is correlated with
{                                                               //glutStringLength() and glutBitmapString(),
    return 10;                                                  //i.e. a change in the font-type in the former 
}                                                               //two has to be correlated with the change in font-height herein.



void IVZBufferHit::init(int x,int y,int dx_,int dy_)            //Initializes zbuffer object by copying z-values from current
{                                                               //OpenGL buffer to it. The window (x,y,x+dx,y+dy) is copied.
   delete zb; zb = new float[(dx=dx_)*(dy=dy_)];
   glReadPixels(x,y,dx,dy,GL_DEPTH_COMPONENT,GL_FLOAT,zb);      //Read z-values normalized in [0..1]

   float zmin=1;                                                //Determine if zbuffer is empty, i.e. has nothing rendered in it.
   for(int i=0,sz=dx*dy;i<sz;i++)                               //This is the case if the smallest value in it is 1.0, since
      if (zb[i]<zmin) zmin = zb[i];                             //this is the 'background' max-value
   is_empty = (zmin>=1);
}
