#include "Ops.h"
#include "GraphManager.h"
#include "Node.h"
#include "Edge.h"
#include "Geometry.h"
#include "Selection.h"
#include "SelList.h"
#include "Filter.h"
#include "Genrl/string.h"
#include "Genrl/minmax.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include "AttrSetType.h"


float   NestedLayoutOp::xyborder = 0.2f;				//Initialize static class members
float   NestedLayoutOp::zborder  = 0.2f;				//
BOOL    NestedLayoutOp::do_z_layout = TRUE;
DirectionFilter* NestedLayoutOp::out_filter = 0;
DirectionFilter* NestedLayoutOp::in_filter  = 0;


int NestedLayoutOp::layout()						//Lays out a selection as a set of nested 2D layouts
{									//
    SelectionList* sl  = GraphManager::getInstance()->getSelList();
    Selection*     s   = getSelection();				//Get input ("what we want to lay out")
    if (!s->numNodes()) return 1;					//Anything to do?
    Selection*    root = 0;						//'root' holds the parent-less nodes in 's'
    SelectSetOp::roots(s,in_filter,root);				//Select 'roots' of selection to lay out

    cross = new Selection;                                              //'cross' will hold the assoc-edges between nodes in different parents
                                                                        //(which have to be layed out separately)

    if (root->numNodes())						//Anyhting to do?
    {
      tree = 0;
      SelectSetOp::tree(root,s,out_filter,0,tree);			//Make a strict tree-restriction from the roots of the input-selection 's'
									//This is needed to ensure that the nested-layout will work 
      Selection::NodeIterator* i; Node* n;                              //(a node must have EXACTLY one parent).
      for(i=root->iterNodes();(n=i->next());)				//Select all edges between the roots. The assoc-edges
      {									//between roots and e.g. children-of-roots are separately selected in
         EdgeSet& es = n->getEdges(s,Node::OUT);			//'cross' and layed out separately. The same happens in layout().
         for(int j=0;j<es.Count();j++)     				//	
	 {								//
	   Edge* e = es[j];						
           if (root->hasNode(e->toNode())) root->addEdge(e); 
             else 
             if (!tree->hasEdge(e)) cross->addEdge(e);
	 }								
      }

      
      float bbx,bby;							//Perform the nested layout
      detect_bottom = FALSE;						//Set flag for detecting first call of layoutNodes()
      layoutNodes(root,bbx,bby,maxlevel=0);				//starting from the root-selection


      normalize(root,0,0,bbx,bby);					//Finish work by normalizing coords

      layoutCrossEdges();                                               //Now lay out all assoc-edges between nodes which are
                                                                        //in different parents.

      delete root; delete tree; delete cross;				//Done with the local selections
   }

   return 1;
}


void NestedLayoutOp::layoutCrossEdges()
//Lays out all assoc-edges (collected in 'cross') that connect nodes which are in different parents. Such edges could
//not be layed out by the normal process, since their end-nodes simply don't belong to the same 'subgraph' in the 
//recursive layout process. We lay them here as follows:
// a) the edge gets the node-center-coords of its two nodes
// b) if the nodes' bboxes don't overlap, the edge-line is clipped against these bboxes and the outside piece is kept
// c) if one node-bbox (say N1) is contained in he other (say N2) and N2's center is not in N1's bbox, 
//    we clip the edge against N1's bbox only. This seems to be a nice heuristic.
// d) if the above fail, simply let the edge connect the nodes' centers.
{
   Selection::EdgeIterator* i; Edge* e;
   
   for(i=cross->iterEdges();(e=i->next());)                             //for all cross-edges...
   {
       Node *n1 = e->fromNode(), *n2 = e->toNode();                     //get various coords of the nodes        
       float vx[2],vy[2],ex[2],ey[2],bx[2],by[2],rxm[2],rxM[2],rym[2],ryM[2];
       vx[0]  = n1->getFloat(getPositionX());           vx[1] = n2->getFloat(getPositionX());
       vy[0]  = n1->getFloat(getPositionY());           vy[1] = n2->getFloat(getPositionY());
       bx[0]  = n1->getFloat(getDimensionX());          bx[1] = n2->getFloat(getDimensionX());
       by[0]  = n1->getFloat(getDimensionY());          by[1] = n2->getFloat(getDimensionY()); 
       rxm[0] = vx[0]-bx[0]/2; rxM[0] = vx[0]+bx[0]/2; rym[0] = vy[0]-by[0]/2; ryM[0] = vy[0]+by[0]/2;
       rxm[1] = vx[1]-bx[1]/2; rxM[1] = vx[1]+bx[1]/2; rym[1] = vy[1]-by[1]/2; ryM[1] = vy[1]+by[1]/2;

       Geometry::OVERLAP_TYPE ov = Geometry::rectOverlap(rxm[0],rym[0],rxM[0],ryM[0],rxm[1],rym[1],rxM[1],ryM[1]);
                                                                        //test how the nodes' bboxes overlap
       if (ov!=Geometry::NONE)                                          //there is some overlap:                                
       {                                
         ex[0]=vx[0]; ex[1]=vx[1]; ey[0]=vy[0]; ey[1]=vy[1];            //default: edge connects nodes' centers
         
         if (ov==Geometry::CONTAIN_FIRST)                               //bbox of n1 contains bbox of n2
         {   
            if (!Geometry::rectContains(rxm[1],rym[1],rxM[1],ryM[1],vx[0],vy[0]))  
            {                                                           //center of n1 not in bbox of n2
              ex[1] = ex[0]; ey[1] = ey[0];                             //edge = center-n1 -> bbox-n2
              Geometry::rectLineClip(rxm[1],rym[1],rxM[1],ryM[1],vx[1],vy[1],ex[1],ey[1]);
            }
         }
         else if (ov==Geometry::CONTAIN_SECOND)                         //bbox of n2 contains bbox of n1
         {   
            if (!Geometry::rectContains(rxm[0],rym[0],rxM[0],ryM[0],vx[1],vy[1])) 
            {                                                           //center of n2 not in bbox of n1
              ex[0] = ex[1]; ey[0] = ey[1];                             //edge = bbox-n1 -> center-n2
              Geometry::rectLineClip(rxm[0],rym[0],rxM[0],ryM[0],vx[0],vy[0],ex[0],ey[0]);
            }
         }
       }
       else                                                             //no overlap between bboxes of n1,n2
       {
         ex[0] = vx[1]; ey[0] = vy[1];                                  //clip the centerline against the two bboxes
         Geometry::rectLineClip(rxm[0],rym[0],rxM[0],ryM[0],vx[0],vy[0],ex[0],ey[0]); 
         ex[1] = vx[0]; ey[1] = vy[0];  
         Geometry::rectLineClip(rxm[1],rym[1],rxM[1],ryM[1],vx[1],vy[1],ex[1],ey[1]); 
       }

       e->set(getPositionX(),ex,2);                                     //finally, write the edge-coords to the edge
       e->set(getPositionY(),ey,2);
       ex[0] = n1->getFloat(getPositionZ());                            //the z-coord is, so far, simply a copy of the
       ex[1] = n2->getFloat(getPositionZ());                            //nodes' z-coord (done for completeness)
       e->set(getPositionZ(),ex,2);
   }

}


void NestedLayoutOp::layout(Node* n,int lev)				//Lays out children of 'n' that are in 's', too.
{									//This is part of the nested-layout process.
   Selection*      s  = getSelection();
   Selection*      ch = new Selection;					//This will hold the children of 'n'
   Node* n2;

   n->set("_level",lev);						//Give a level ('depth') to the node so we can later
   if (lev>maxlevel) maxlevel = lev;					//normalize its Z coordinate. Also recall how deep we go.

   EdgeSet es = n->getEdges(tree,out_filter);				//Select children of 'n' that are in 'tree'
   for(int i=0;i<es.Count();i++) ch->addNode(es[i]->otherNode(n));
   Selection::NodeIterator* it;
   for(it=ch->iterNodes();(n2=it->next());)				//Select 'horizontal' edges, i.e. those edges
   {									//(between n's children) that are in 's' too
      EdgeSet& es = n2->getEdges(s,Node::OUT);				//Select horiz-edges between n's children and
      for(int j=0;j<es.Count();j++)                                     //other nodes in 's' in 'cross', since they have to
      {                                                                 //be layed out separately.
	 Edge* e = es[j];
         if (ch->hasNode(e->toNode()))  ch->addEdge(e); 
            else 
            if (!tree->hasEdge(e)) cross->addEdge(e);
      }
   }

   if (!ch->numNodes())							//no children: simply prescribe sizes
   {  n->set(getDimensionX(),1); n->set(getDimensionY(),1);  }		//of 'n' to (1,1) - by default
   else   	   							//lay out children-subgraph, which was determined above
   {									//This subgraph is a 'horizontal' graph, given the way it was
      float bbx,bby;							//constructed above.
      layoutNodes(ch,bbx,bby,lev+1);				
      n->set(getDimensionX(),bbx);					//size of this is bbox of its layout
      n->set(getDimensionY(),bby);
   }
   delete ch;								//done with 'ch'
}


void NestedLayoutOp::layoutNodes(Selection* ch,float& bbx,float& bby,int lev)
//Lays out nodes in 'ch' intersected with 's'. 'ch' nodes are on same level.
{
      Selection::NodeIterator* i; Node* n; Edge* e;
      for(i=ch->iterNodes();(n=i->next());)		//lay out contents of every child separately
         layout(n,lev);					//(this'll set the dx,dy attrs for the children)

      if (ch->numNodes()>1)				//Call Neato layout only if we have at least 2 nodes (laying out 1 node
      {							//is done separately to eliminate overhead of external layout call
	 NeatoLayoutOp l;				//lay out children with their given sizes
         l.useBoundingBox(FALSE);			//don't force layout in bbox, let it be set by the nodes' dimensions
         l.keepDimensions(TRUE);			//don't set the dim-attrs in 'l', we do it ourselves.
         BOOL ov = NeatoLayoutOp::getDefaultOverlap();
         BOOL ud = LayoutOp::getDefaultExtLayoutUseCrtDim();
	 BOOL le = LayoutOp::getDefaultLayEdges();
	 BOOL cp = NeatoLayoutOp::getDefaultCompact();
         NeatoLayoutOp::setDefaultOverlap(FALSE);	//use Neato without overlapping, so save global-param
	 NeatoLayoutOp::setDefaultCompact(TRUE);	//force Neato to compact unconnected nodes to get a tighter layout
         LayoutOp::setDefaultExtLayoutUseCrtDim(TRUE);	//pass the nodes' dims to Neato, this is crucial for the nesting
	 LayoutOp::setDefaultLayEdges(TRUE);		//edges MUST be layed out explicitly for a nice node-edge connection point
	 l.LayoutOp::layout(ch,getPosition());		//apply Neato layout
	 NeatoLayoutOp::setDefaultOverlap(ov);		//restore global layout params we modified
	 NeatoLayoutOp::setDefaultOverlap(cp);		//
         LayoutOp::setDefaultExtLayoutUseCrtDim(ud);	//
	 LayoutOp::setDefaultLayEdges(le);		//
      }
      else						//Laying out 1 node: simply assign the node's position as
      {							//being its center (its dimensions are set)
	 Selection::NodeIterator* i = ch->iterNodes(); 
	 Node* n = i->next();
	 n->set(getPositionX(),n->getFloat(getDimensionX())/2); 
	 n->set(getPositionY(),n->getFloat(getDimensionY())/2);
      }

      float xm,ym,xM,yM;				//Get bounding-box of whatever Neato layed out for 'ch'
      Geometry::bbox2D(ch,getPosition(),xm,ym,xM,yM);

      if (!detect_bottom)				//This code gets executed when we call layoutNested() the FIRST time for a given layout.
      {							//This means when we layout the 'deepest' node. Then we compute 'xyborder_abs' as a 
         xyborder_abs = max(xM-xm,yM-ym)*xyborder;	//fraction 'xyborder' of the size of this node. This is a good heuristics of computing 
	 detect_bottom = TRUE;				//an ABSOLUTE-size border between a nested layout and its container-node. The border will
      }							//have this size for all levels of nesting. This gives a pleasant aspect to the nesting. 

      float dx = xyborder_abs-xm, dy = xyborder_abs-ym;
      for(i=ch->iterNodes();(n=i->next());)		//Also, reposition nodes in the current level to match
      {							//the increase of the bounding-box with 'xyborder_abs'
         n->set(getPositionX(),n->getFloat(getPositionX())+dx);
         n->set(getPositionY(),n->getFloat(getPositionY())+dy);
	 //REMARK: we subtract xm,ym from the layed out nodes. So all nodes in 'ch' will be now in (0,0,bbx,bby).
	 //	   bbx,bby is passed to caller so it can set the parent's size. We could not subtract xm,ym and
	 //	   pass bbx+xm,bby+ym to caller. The difference would be that we'd leave the nodes where the internal
	 //	   (neato) layout puts them, thus wasting the xm,ym space. We choose not to do that since it has no use.
      }
      
      //Reposition edges in the current level as well. This does the same to the edges in 'ch' as
      //we did to the nodes. The only difference is that we check for the existence of the edge-pos
      //attributes, as these are not mandatory. If any exist, reposition them, else do nothing.
      for(Selection::EdgeIterator* j=ch->iterEdges();(e=j->next());)
      {
	 int sz; float* v = e->getFloatVec(getPositionX(),sz);
	 if (!v) continue;				//Add dx,dy to the x,y pos-attrs of the edges. These must ve
	 for(sz--;sz>=0;sz--) v[sz] += dx;		//float-vector-attrs. If not found as such, skip this step
	 v = e->getFloatVec(getPositionY(),sz);
	 if (!v) continue;
	 for(sz--;sz>=0;sz--) v[sz] += dy;
      }

      bbx = (xM-xm)+2*xyborder_abs;			//pass bounding-box info to caller	
      bby = (yM-ym)+2*xyborder_abs; 			//after we increased it with 'xyborder_abs'
}


void NestedLayoutOp::normalize(Selection* ch,float ox,float oy,float bbx,float bby)
// Final pass in the layout procedure. Converts all nodes' positions/sizes such that the 
// whole layout is normalized in [0..1], for the x and y coords.
// The z coord is made proportional with the nodes' level.
{
    Selection::NodeIterator* i; Node* n; Edge* e;
    Selection* s = getSelection();
    for(i=ch->iterNodes();(n=i->next());)
    {
       float cx   = n->getFloat(getPositionX()),	//normalized [0..1] pos of n, relative to its parent
             cy   = n->getFloat(getPositionY());		
       float x    = ox + cx, y = oy + cy;		//abs pos of n 
       float bbx_ = n->getFloat(getDimensionX()),	//abs size of n
	     bby_ = n->getFloat(getDimensionY());
       float  ox_ = x - bbx_/2,oy_ = y - bby_/2;	//abs pos of n's coordsys-orig or bounding-box

       n->set(getPositionX(),x);			//produce absolute, [0..1]-normalized coords for n
       n->set(getPositionY(),y);	
       n->set(getPositionZ(),(maxlevel && do_z_layout)? float(n->getInt("_level"))/maxlevel : 0.0);
       n->del("_level");				//ready with local-attr "_level", don't let it hang around
       n->set(getDimensionZ(),				//scale Z axis to fit in bbox of depth 1, and scale sizes on Z with 'zborder'	
	      (1-zborder)*((maxlevel)? 1.0/maxlevel : 1.0));

       Selection* ch_ = new Selection;			//normalize children of n (and horiz-edges) to fit in n
       EdgeSet es = n->getEdges(tree,out_filter);	//first, add all children of n to ch_
       for(int j=0;j<es.Count();j++) ch_->addNode(es[j]->otherNode(n));
       for(Selection::NodeIterator* k=ch_->iterNodes();(n=k->next());)	
       {						//next, add all edges between n's children to ch_ too			
         EdgeSet& es = n->getEdges(s,Node::OUT);	//(this is needed since these edges might need to be normalized too)
         for(int j=0;j<es.Count();j++)     					
	 {								
	   Edge* e = es[j];						
           if (ch_->hasNode(e->toNode())) ch_->addEdge(e);		
	 }								
       }
       if (ch_->numNodes()) normalize(ch_,ox_,oy_,bbx_,bby_);
       delete ch_;					//now normalize recursively n's contents and then dispose of ch_
    }

    //Do the position-normalization for the x,y edge-coords in 'ch' too. This is just as for the
    //nodes, with the difference that edge-pos-attrs might not exist, so we first check for them.
    //Edge-pos-attrs are supposed to be float-vectors. If none found, just skip this step.
    //REMARK: We also generate edge z-coords by linearly interpolating between the node-endpoints' z coords.
    //        This is done so that this layout guarantees that, if edge-coords exist, then all x,y, and z exist.
    //        This is a bit redundant as later in the pipeline IVEdgeMapper::getEdgeCoords() can also generate
    //        linearly interpolated z-coords if not available.

    for(Selection::EdgeIterator* j=ch->iterEdges();(e=j->next());)
    {
	int i,szx,szy; float* v = e->getFloatVec(getPositionX(),szx);
	if (!v) continue;				        //Add ox,oy to the x,y pos-attrs of the edges. These must ve
	for(i=0;i<szx;i++) v[i] += ox;		                //float-vector-attrs. If not found as such, skip this step
	v = e->getFloatVec(getPositionY(),szy);
	if (!v) continue;
	for(i=0;i<szy;i++) v[i] += oy;        
        
        static float vz[200];                                   //Generate z-coords as linear interpolation between       
        Node *n1 = e->fromNode(), *n2 = e->toNode();            //the node-endpoints' z-coords (which exist)
        float vz0 = n1->getFloat(getPositionZ());          
        float vzn = n2->getFloat(getPositionZ());
        int   szz = (szx<szy)? szx : szy;
        
        for(i=0;i<szz;i++)                                      //Linearly interpolate the z-coord 
        {                                                   
          float t = float(i)/(szz-1);
          vz[i]   = vz0*t + vzn*(1-t);
        }
        e->set(getPositionZ(),vz,szz);                          //Set the z-coord-attrs to the edge
    } 
}


void NestedLayoutOp::setFilter(DirectionFilter* f)		//Sets filter to use to determine containment
{								//relations. Makes a copy of the passed filter and
   delete out_filter; delete in_filter;				//stores it locally to avoid problems with ownership
   out_filter = (DirectionFilter*)f->clone();			//Also makes a reverse-filter, as we need to navigate 
   in_filter  = (DirectionFilter*)f->clone();			//the containment-relation in both directions
   in_filter->setDirection((in_filter->getDirection()==Node::IN)?	
	                   Node::OUT:Node::IN);
}