#include "ObsList.h"
#include "Observer.h"
#include "GraphManager.h"
#include "SelList.h"


ObserverList::ObserverList(SelectionList* l):sel(l),dirty(FALSE)	//Ctor
{     }

ObserverList::~ObserverList()				//Delete all observers owned by this
{
    for(int i=0;i<obs.Count();i++)
       delete obs[i];
}	

Observer* ObserverList::getObserver(const char* n)	//Search Observer by name
{
    for(int i=0;i<obs.Count();i++)
       if ((*obs[i])==n) return obs[i];
    return 0;
}

void ObserverList::addObserver(Observer* s)
{   
    const char* name = s->getName();

    for(int i=0;i<obs.Count();i++)		//search for Observer with same name as 's'
      if ((*obs[i])==name)			//if found, delete it (we own it), and put 's' in its place
      {
	 delete obs[i];
	 obs[i] = s;
	 return;
      }
      
    obs.Add(s);					//if not found, simply add 's' to list
}



void ObserverList::delObserver(const char* n)
{
    Observer* o = getObserver(n);		//search for named observer

    if (!o) return;

    obs.Remove(o);				//remove observer from list and destroy it
    delete o;
}	


void ObserverList::delAllObservers()
{
   for(int i=0;i<obs.Count();)			//delete all non persistent observers
    if (!obs[i]->persistent())			//from this
       delObserver(obs[i]->getName());
    else i++;
}


void ObserverList::update()			//lazy-update all observers
{  dirty = TRUE;  }				//when we'll have time for this

void ObserverList::doUpdates()			//execute all pending updates
{
   if (!dirty) return;				//is everything already updated? 
   int i;
   SelectionList* sl = GraphManager::getInstance()->getSelList();
   for(i=0;i<obs.Count();i++)			//update all observers:	
   {
      if (obs[i]->mustBeUpdated())		//ask each observer whether it needs updating
      {						//(the logic of checking is specific to every observer...)
	 obs[i]->update();  
      }
   }
   
   for(i=0;i<sl->numSelections();i++)		//mark all selections as untouched, since everybody has updated
      sl->getSelection(i)->untouch();
   dirty = FALSE;				//mark this as untouched since now all updates are done	
}

