#include "Ops.h"
#include "GraphManager.h"
#include "RSFDomainModel.h"
#include "RSFAttrSetType.h"
#include "Node.h"
#include "Edge.h"
#include "Filter.h"
#include "AttrType.h"
#include "Selection.h"
#include "SelList.h"
#include "Geometry.h"
#include "GraphEd.h"
#include "utils.h"
#include "Genrl/string.h"
#include "Genrl/types.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>


BOOL LoadOp::clear_at_load = TRUE;                      //Init class member


//***************   GRAPH OPERATIONS   ***************************************

void LoadOp::clearAtLoad(BOOL b)
{
   clear_at_load = b;
}


BOOL LoadOp::isClearAtLoad()
{
   return clear_at_load;
}


void LoadOp::reset()                    //resets this to initial state
{
   clearAtLoad(TRUE);
}


int LoadOp::readGraphEd(const char* fname,const char* pos_name)
{
   FILE* fp = fopen(fname,"r");
   if (!fp) return 0;   

   begin();

   BOOL rdpos = (pos_name!=0);		//determine if we wish to read pos-attributes

   if (isClearAtLoad()) 
      GraphManager::getInstance()->getSelList()->reset();
   Selection* g = GraphManager::getInstance()->getSelList()->getFullSelection();

   STRING posx = Geometry::getPositionX(pos_name),
          posy = Geometry::getPositionX(pos_name),
   	  posz = Geometry::getPositionX(pos_name);
   int max_id = Element::getFreeId();   //get 1st free-id to add read elements from
   int ret    = read_graphed(fp,g,max_id,rdpos,posx,posy,posz);

   end();

   fclose(fp);
   return ret;
}

int LoadOp::readString(FILE* fp,char* s)                //Like fscanf(fp,"%s",s) but reads all stuff between quotes "..."
{                                                       //if these are present in the input stream. Useful to read single keywords
   int c; *s = '\0';                                    //or optionally quoted strings containing spaces.
   
   for(c=fgetc(fp);c!=EOF && isspace(c);c=fgetc(fp));   //First skip all spaces
   
   for(int q=0;(q==1 || !isspace(c)) && (c!=EOF);c=fgetc(fp)) //Read all till closing quote or till space if no quote was found
   {
      if (c=='"') q = 1-q;                              //Quote found? Remember to read till closing quote
      else
      *s++ = (char)c;                                   //No quote - simply copy character in return buffer 's'
   }
   *s = '\0';                                           //Close string
   return c!=EOF;                                       //Return 1 if all fine and 0 if we encountered EOF
}




int LoadOp::readOwnFormat(const char* fname,Selection* all)   //Read own format file into given selection
{                                                             //('All' is used if no selection given)    

   FILE* fp = fopen(fname,"r");
   if (!fp) return 0;   

   begin();

   if (isClearAtLoad())                         //clear existing data, if we have to                
      GraphManager::getInstance()->getSelList()->reset();
   DomainModel* dom = GraphManager::getInstance()->getDomainModel();
   if (!all) 
      all = GraphManager::getInstance()->getSelList()->getFullSelection();

   char buf[255],buf2[255],nm[255]; AttrSet* a;
   float v[1024]; 

   int max_id = Element::getFreeId();           //start adding the new elements from the 1st free id onwards
                                                //(if we've done a reset(), max_id==0). Note the ids read from
                                                //file are offset with max_id, to leave the range [0..max_id-1] untouched.

   for(;;)					//read all file
   {
      int id,id2,id3; int e=0,ei=0; BOOL is_node;
      if (fscanf(fp,"%s",buf)!=1) break;	//stop when EOF	
      if (!strcmp(buf,"node"))			//process "node <node id>" statement
      {
	 fscanf(fp,"%d",&id); id+=max_id;
         a = all->getNodeById(id);		//make/retrieve node by id
         if (!a) a = new Node(all,id); is_node = TRUE; 
      }
      else if ((e=(!strcmp(buf,"edge"))) || (ei=(!strcmp(buf,"edgei"))))		
      {                                         //read edge decl ("edge" or "edgei")
	 fscanf(fp,"%d %d",&id,&id2);		//create/retrieve 2 nodes of edge
         id += max_id; id2 += max_id;
         if (ei) 
         { fscanf(fp,"%d",&id3); id3+=max_id; } //if "edgei" decl, read also edge-id
         else    id3=-1;                        //else the edge-id is -1, i.e. dynamically created
	 Node* n  = all->getNodeById(id);
	 if (!n) n = new Node(all,id); 
         Node* n2 = all->getNodeById(id2);
	 if (!n2) n2 = new Node(all,id2); 
	 a = n->connectTo(n2,all,0,id3);	//connect this node->new node, since we know
	 is_node = FALSE;			//that edge-decls are always OUTGOING.
      }
      else if (buf[0]=='/' && buf[1]=='/')	//skip "//" comment-lines
      {  for(int c=fgetc(fp);c!=EOF && c!='\n';c=fgetc(fp)); continue;  }
      else break;				//unknown keyword found...

      for(;;)					//process node/edge-body declaration
      {
         fscanf(fp,"%s",buf);			//get keyword (attrdecl/typedecl)
         if (!strcmp(buf,"end")) break;		//node/edge-body decl finished, break
         if (!strcmp(buf,"int"))
         {
           int x; 				//read int attr (name may contain quoted text w. spaces)
           readString(fp,nm);
           fscanf(fp,"%d",&x); a->set(nm,x);
         }
         else if (!strcmp(buf,"float"))
	 {
	   float x;				//read float attr (name may contain quoted text w. spaces)	
           readString(fp,nm);
	   fscanf(fp,"%f",&x); a->set(nm,x);
	 }
	 else if (!strcmp(buf,"string"))
	 {					//read string attr (may contain quoted text w. spaces)
           readString(fp,nm);
           readString(fp,buf2);
	   a->set(nm,buf2);
	 }
         else if (!strcmp(buf,"floatv"))        //read float-vector attr
         {
           readString(fp,nm);
           int n; fscanf(fp,"%d",&n);           //read # elems in vector, then read all vector in buffer v[]
           for(float *vp=v,*ve=v+n;vp<ve;vp++) fscanf(fp,"%f",vp);
           a->set(nm,v,n);                              
         }
         else if (!strcmp(buf,"type"))          //read domain-model type
         {
           readString(fp,nm);                   //read string attr (type-name)
           const AttrSetType* ast = (is_node)? dom->getNodeType(nm):dom->getArcType(nm);
           a->setType(ast);                     //set the node/edge type with the read name
         }
      }				
   }
   end();

   fclose(fp);
   return 1;
}



//-------------------------------------------------------------------------

int LoadOp::readRSF(const char* fname)		//Read RSF unstructured graph.
{						//We assume the current domain model is a RSF model
   FILE* fp = fopen(fname,"r");			//and that the names in the RSF file correspond with the domain model
   if (!fp) return 0;   

   begin();

   if (isClearAtLoad()) 
      GraphManager::getInstance()->getSelList()->reset();
   DomainModel* dom = GraphManager::getInstance()->getDomainModel();
   Selection* all = GraphManager::getInstance()->getSelList()->getFullSelection();

   char buf[300], buf2[300], buf3[300], line[1024]; 

   STRING cache_type;
   const AttrSetType* cache_ast = 0;
   STRING cache_node_name;
   Node* cache_node = 0;
   STRING cache_buf,cache_buf2,cache_buf3;

   const AttrSetType* ntyp = dom->getNodeType(dom->numNodeTypes()-1); 
							//get the nodetype that the RSF file will use.
							//since we know that RSF domains have the same attrtypes
							//for all nodetypes, we can simply get the last nodetype in the domain.
							//REMARK: we don't get the first since that is incidentally the default
							//'unknown' node-type created by the DomainModel class
   const AttrSetType* untyp = dom->getUnknownNodeType();//get the unknown node-type and arc-type too
   const AttrSetType* uatyp = dom->getUnknownArcType(); 

   map<STRING,Node*> ntable;				//Accelerates lookup of nodes by name by using a fast name-node map.

   for(int lnum=1;;lnum++)				//read all file, line by line:
   {
      if (!fgets(line,1023,fp)) break;			//stop if EOF
      int num = sscanf(line,"%s%s%s",buf,buf2,buf3);
      if (num==1 || num==2)				//check we have indeed 3 entries per line (mandatory)
	 cerr<<"LoadOp::readRSF: line "<<lnum<<" has only "<<num<<" entries instead of 3, aborting."<<endl;    
      if (num!=3) break;				//stop when EOF or less than 3 keywords per line
      if (cache_buf==buf && cache_buf2==buf2 && cache_buf3==buf3)
         continue;					//trap duplicate lines (simple speedup)

      if (!strcmp(buf,"type"))				//1. process "type <nodename> <nodetype>"
      {
	const AttrSetType* ast =0;
	if (cache_type==buf3) ast = cache_ast;		//search for AttrSetType called buf3.
	else						//first, check our local cache to avoid full getNodeType() call
	{						//cache didn't work, must call getNodeType().
           ast = dom->getNodeType(buf3); 		//determine type of new Node. Must exist, otherwise
           cache_type = buf3; cache_ast = ast;		//the current domain doesn't correspond with the RSF file.
	}						//store current (buf3,ast) data in cache 

	map<STRING,Node*>::const_iterator it = ntable.find(STRING(buf2));	
	Node* n = (it!=ntable.end())? (*it).second : 0;
	
						
	if (!n)						//check that we haven't a duplicate declaration
	{						//(it seems .rsf files may allow this...)
           n = new Node(all,-1,ast);			//node "buf2" doesn't exist yet:
	   n->set("name",buf2);				//make new node and add type and name attrs to it
	   map<STRING,Node*>::value_type v(buf2,n);
	   ntable.insert(v);				//also add the new node with its name to map
        }
	else						//We encountered a node-declaration for an already existing node 
	{						//This can mean two things:
  	   const AttrSetType* ast0=n->getType();	//a) the node was constructed since it was used BEFORE its declaration in an arc-decl
	   if (ast0!=ast)				//   (RSF seems to allow this). Here we have 2 subcases. If the node's type is the unknown type
	   { 						//   all is fine, the current decl actually sets its type, nothing more. If the node already had
							//   a type (not the unknown but anything else) then the current decl is a type-redeclaration 
							//   - so we issue a warning.
	      //!!!if (ast0!=untyp) cerr<<"LoadOp::readRSF: warning: node "<<n->getString("name")<<" of type "<<ast0->getName()<<endl
	      //!!!	                  <<"                 redeclared with type "<<ast->getName()<<endl;
	      n->setType(ast); 
           }
	   else						//b) the node exists and has the same type as in the current declaration
           {						//   We simply ignore the new declaration, which is a duplicate of the first
	      //!!!cerr<<"LoadOp::readRSF: warning: duplicate declaration of node "<<n->getString("name")<<" of type "<<ast0->getName()<<", ignored"<<endl;
	   }
	} // "type" declaration

	cache_node_name = buf2;				//cache the (n,buf2) association for possible speedup
	cache_node      = n;
      }
      else						//2. determine whether we have an attr or arc specification
      {							//we do this by simply looking up buf as a potential
	    						//nodeattr; the RSF manual states that nodeattrs and 
							//arcattrs are separate namespaces
	 if (ntyp->getAttrType(buf))
	 {						//2.1. process "<nodeattr> <nodename> <attrvalue>"
	    Node* n = (cache_node_name==buf2)? cache_node : 0;
	    if (!n)
	    {
	      map<STRING,Node*>::const_iterator it = ntable.find(STRING(buf2));
	      n = (it!=ntable.end())? (*it).second : 0; 			     
	    }
	    if (n) n->set(buf,buf3);			//set <nodeattr> to <attrvalue> for the found node
	    else cerr<<" LoadOp::readRSF: Ignoring attr for inexistent node "<<buf2<<endl;
	 }
         else 						//2.2. process "<arctype> <nodename1> <nodename2>"
	 {
	    Node* n1 = (cache_node_name==buf2)? cache_node : 0;
	    if (!n1)					//retrieve/create 1st node by name
	    {
	      map<STRING,Node*>::const_iterator it = ntable.find(STRING(buf2));
	      n1 = (it!=ntable.end())? (*it).second : 0;
	      if (!n1)
	      {
	        n1 = new Node(all); n1->set("name",buf2);
                map<STRING,Node*>::value_type v(buf2,n1); ntable.insert(v);	
	      }
	    }

	    Node* n2 = (cache_node_name==buf3)? cache_node : 0;
	    if (!n2)					//retrieve/create 2nd node by name
	    {
	       map<STRING,Node*>::const_iterator it = ntable.find(STRING(buf3));
	       n2 = (it!=ntable.end())? (*it).second : 0;
	       if (!n2)
	       {
	         n2 = new Node(all); n2->set("name",buf3);
	         map<STRING,Node*>::value_type v(buf3,n2); ntable.insert(v);
	       }
	    }

	    if (n1!=n2)					//!!!WARNING: self-arcs not yet allowed
 	    {	
	      const AttrSetType* ast = dom->getArcType(buf);
	      EdgeSet& es = n1->getEdges(all,Node::OUT);//See if n1,n2 are already connected by some edge
	      Edge* e = es.to(n2);	                //(might not be the only edge n1->n2, but it doesn't matter)
	      BOOL  f = FALSE;				
	      if (e)					//If n1,n2 already connected, find out whether
	      {						//we have to set the new type 'ast' for the already existing
		 const AttrSetType* oldt = e->getType();//edge (done if the new type is not unknown but the old one
		 if (ast!=uatyp)			//was) or to make a new edge (done if the new type is not unknown
		    if (oldt==uatyp) e->setType(ast);	//and the old one is also not unknown, so both edges are valid)
		    else if (ast!=oldt) f = TRUE;
	      } 	
	      if (!e || f)				//n1,n2 not connected at all or new edge n1->n2 encountered:
	      {
	         e = n1->connectTo(n2,all);		//make new horiz-edge between the two found nodes
	         if (ast==uatyp)			//set the type attr for the new edge, if it isn't the unknown-type
                    cerr<<"LoadOp::readRSF: unknown arc type "<<buf<<" encountered on line "<<lnum<<endl;
	         else
	            e->setType(ast);
	      }
              STRING x = "System.exit()";
              int dbg = (x==n1->getString("name") || x==n2->getString("name"));
              
              if (ast!=uatyp)                           //if the new arc has a not-unknown type,
              {                                         //check whether it enforces node-types on its nodes
                 RSFArcType* rsfa = (RSFArcType*)ast;   //If so and if the nodes don't have types yet,
                 const char* ftyp = rsfa->fromNodeType();  //set the node-types spec'd by the arc on them.
                 const char* ttyp = rsfa->toNodeType();    //(this can happen on one or both of the arc's nodes...)
                 if (ftyp[0]!='\0' && ttyp[0]!='\0')    //Check the arc node-type-spec is ok       
                 {
                    if (n1->getType()==untyp)           //n1 has no type set, enforce it
                    {
                       const AttrSetType* ast_n1 = dom->getNodeType(ftyp);
                       n1->setType(ast_n1);
                    }
                    if (n2->getType()==untyp)           //n2 has no type set, enforce it
                    {
                       const AttrSetType* ast_n2 = dom->getNodeType(ttyp);
                       n2->setType(ast_n2);
                    }
                 }
              }
	    }
	 }					
      }
      cache_buf  = buf;					//remember currently parsed line for speedup checks
      cache_buf2 = buf2;
      cache_buf3 = buf3;			 	
   }

   fclose(fp);

   end();
   return 1;
}



//-------------------------------------------------------------------------


void EditOp::renameSelection(Selection* sel,const char* newsel)
{
   begin();

   GraphManager* gm = GraphManager::getInstance();
   gm->getSelList()->renameSelection(sel,newsel);

   end();
}



void EditOp::reset()
{
   begin();
   GraphManager::getInstance()->reset();
   end();
}


BOOL EditOp::cluster(Selection* sel,Node*& parent,int id)
{
   if (*sel==Selection::NONE) return FALSE;				//can't add parent to empty-selection

   begin();

   GraphManager* gm = GraphManager::getInstance();
   Selection*   all = gm->getSelList()->getFullSelection();

   parent = new Node(all,id);						//make parent node
   Node* n;
   for(Selection::NodeIterator* i=sel->iterNodes();(n=i->next());)
   {
	Edge* e = parent->connectTo(n,all);				//add edges to given nodes as containment-edges
   }

   end();
   return TRUE;								//all done ok
}


void AttrOp::computeLevel(Selection* sel,DirectionFilter* f,const char* lev_attr)
{									//Computes a level-attribute for nodes in given selection.
   begin();								//Leaf nodes have level 0, their parents level 1, and so on.
									//The traversal is determined by the directional-filter 'f'
   Selection* r = 0; Node* n; Selection::NodeIterator* i;
   DirectionFilter* f_rev = (DirectionFilter*)f->clone();		//Clone the filter f and make a reverse-filter rev_f
   f_rev->setDirection((f_rev->getDirection()==Node::IN)?		//We need it for the implementation of the level-computation
	               Node::OUT:Node::IN);
   SelectSetOp::roots(sel,f_rev,r);					//Find the leaves (descendant-less) nodes in sel

   for(int level=0;;level++)						//Recursively set level-attr from leaves 'upwards'
   {
     Selection* t = new Selection;
     for(i=r->iterNodes();(n=i->next());)
     {
       n->set(lev_attr,level);
       EdgeSet es = n->getEdges(sel,f);					//Go upwards	
       for (int j=0;j<es.Count();j++) t->addNode(es[j]->otherNode(n)); 
     }
     delete r;
     r = t;
     if (!r->numNodes()) { delete r; break; }
   }
   delete f_rev;
   end();
}

     



//**********  VIEW OPERATIONS  *****************************************


int SaveOp::writeGV(Selection* sel,const char* fname, const char* pos,BOOL wr_pos,
		    BOOL wr_dim,BOOL flip_edges,const char* level,float edgelen)
{
   FILE* fp = fopen(fname,"w");
   if (!fp) return 0;
   if (!pos) pos="";

   fprintf(fp,"digraph G\n{\n");
   
   STRING    posx = Geometry::getPositionX(pos), 
  	     posy = Geometry::getPositionY(pos), 
   	     posz = Geometry::getPositionZ(pos),
   	      pin = Geometry::getPinned(pos),
   	       dx = Geometry::getDimensionX(pos),
	       dy = Geometry::getDimensionY(pos);	
   BOOL        wr = wr_pos && sel->nodesHaveAttr(posx) && sel->nodesHaveAttr(posy);
   BOOL        sz = wr_dim && sel->nodesHaveAttr(dx) && sel->nodesHaveAttr(dy); 
   BOOL      attr = wr || sz;					//any attrs to be written?
   Node::DIRECTION						//write edges in the same/inverse order than they are in our graph?
	   dir = (flip_edges)? Node::OUT : Node::IN;		//(inverse order is useful for passing this file to the layout-engine 'dot')

   //REMARK: in case we use writeGV() as part of a layout,
   //        we must set an empty node label to stop scaling node on label size.
   //	     If not, node-dims will be set to match their label-size, which is	
   //	     clearly against our wish to impose node-dims.
   //

   fprintf(fp," node [label=\"\"];\n");
   if (edgelen>0)						//write desired avg edgesize if valid one given
      fprintf(fp," edge [len=%6f,dir=none];\n", edgelen);
   else
      fprintf(fp," edge [dir=none];\n");	

   const Node* n; Selection::NodeIterator* i;  			//write nodes with optional xy-pos/size attrs
   int min_level = -1, max_level = -1;				//If a level-attribute is given, write a set of 'fake' nodes/edges
   if (level && level[0]=='\0') level=0;			//that will constrain the real nodes to stay on their levels as given
   if (level)							//by their level-attr. Useful for layout purposes.	
   {								
      const Attr* a;   
      for(i=sel->iterNodes();(n=i->next());)
         if ((a=n->get(level)))					//first compute the min & max of the level-attr
         {  
            int l = a->getInt();
            if (min_level==-1 || min_level>l) min_level = l;
	    if (max_level==-1 || max_level<l) max_level = l;
         }	
      for(int num=0;min_level<max_level;min_level++)		//then write a 'skeleton' containing a node-path from min to max. 
      {								//This will be used to constrain the actual levels of the graph nodes.
	 fprintf(fp," _node%d -> ",min_level);			//(constraining in .dot works only by specifying that a group of nodes 
	 num++; if (num==10) { fprintf(fp,"\n"); num=0; }	//must be on the same level)
      }
      if (max_level!=-1) fprintf(fp," _node%d;\n",max_level);
   }

   	
   for(i=sel->iterNodes();(n=i->next());)			//write nodes with optional xy-pos/size attrs
   {
      const Attr* a=0;
      if (level && (a=n->get(level)))				//if we're using levels and this node has a level-attr
         fprintf(fp,"{ rank=same; _node%d; ",a->getInt());	//then use it to force node on its given level.	

      fprintf(fp,"  %d ",n->getId());				//write node-id
      if (attr) 						//any attrs to be written? 
      {
	fprintf(fp,"[");					//ok, start writing attrs
        if (wr) fprintf(fp,"pos=\"%6f,%6f%c\"",			//write position-attr 
		      n->getFloat(posx), n->getFloat(posy),
		      (n->getInt(pin))? '!':' ');
        if (sz) 						//write size-attr
        { 
          if (wr) fprintf(fp,",");
          fprintf(fp,"width=%6f,height=%6f",n->getFloat(dx),
                  n->getFloat(dy));
        }
        fprintf(fp,"]");					//end writing attrs
      }
      if (a) fprintf(fp," }");					//end rank-scope if we started one
      fprintf(fp,";\n");					//end writing node
   }

   Edge* e; Selection::EdgeIterator* j;				//write edges in selection		
   for(j=sel->iterEdges();(e=j->next());)			
      fprintf(fp,"  %d -> %d;\n",e->node(dir)->getId(),e->otherNode(e->node(dir))->getId());

   fprintf(fp,"}\n");

   fclose(fp);
   return 1;
}



int SaveOp::writeGraphEd(Selection* sel,const char* fname,const char* pos,BOOL wr_pos_,BOOL wr_ids)
//Writes selection to GraphEd file. The GraphEd node-names are the node-ids.
//Position-attrs are written if they exist for all nodes in selection.
//
{
   FILE* fp = fopen(fname,"w"); if (!fp) return 0;
   if (!pos) pos="";

   STRING    posx = Geometry::getPositionX(pos),
   	     posy = Geometry::getPositionY(pos);
   BOOL    wr_pos = wr_pos_ && sel->nodesHaveAttr(posx) && sel->nodesHaveAttr(posy);

   //!!! Here we check if the nodes have a position attribute. If so, we write it
   //	 to the file. We should generalize this check/attr-writing for all node-attrs
   //	 (but for this we should examine the GraphEd syntax)

   write_graphed(fp,sel,wr_pos,wr_ids,posx,posy);
   
   fclose(fp);
   return 1;
}


//--------------  SaveOp::writeOwnFormat routines  -------------------------



static void writeOwnFormatAttrs(FILE* fp,AttrSet* as,DomainModel* dom)
{
     const AttrSetType* ast = as->getType();            //save attrset-type if it's something else than the default one
     if (ast!=dom->getUnknownNodeType() && ast!=dom->getUnknownArcType())  //(since loading an attrset without type will
        fprintf(fp,"type %s\n",(const char*)ast->getName());               //by default set the default one on it)

     for(int i=0;i<as->numAttrs();i++)			//save all attrs of a set:
     {
	Attr* a = as->getAttrVal(i);
	Attr::TYPE t = a->getType();
	if (t==Attr::PTR || t==Attr::UNSET) continue;	//do NOT save ptr-attrs or default-values   
	
        fprintf(fp,(t==Attr::INT)?	 "int":         //save attr-typename
		   (t==Attr::FLOAT)?	 "float":
                   (t==Attr::FLOAT_VEC)? "floatv":
		   /*t==ATTR::STRING*/   "string");

        STRING s = addQuotes(as->getAttrName(i));       //save attr-name, quoted if needed
	fprintf(fp," %s",(const char*)s);

        if (t==Attr::FLOAT_VEC)                         //saving a FLOAT_VEC-attr value means
        {                                               //writing the number of floats followed by all
           int n;                                       //the floats in the vector with a decent number of
           float* v = a->getFloatVec(n);                //decimals
           fprintf(fp," %d",n);
           for(;n;n--,v++) fprintf(fp," %7f",*v);
           fprintf(fp,"\n");
        }
        else                                             //saving all other attr-values means
        {                                                //writing the string-rep of the attr, quoted if needed
           s = addQuotes(a->getString());
	   fprintf(fp," %s\n",(const char*)s);             
        }							
     }
}


static void writeOwnFormatNode(FILE* fp,Node* n,DomainModel* dom)
{
      fprintf(fp,"node %d\n",n->getId());		//write node id
      writeOwnFormatAttrs(fp,n,dom);			//write node attrs	
      fprintf(fp,"end\n");				//ready with writing node
}

static void writeOwnFormatEdge(FILE* fp,Edge* e,Selection* sel,DomainModel* dom)
{
      Node *n1 = e->fromNode(), *n2 = e->toNode();	//don't write edge if its nodes aren't in 'sel'
      if (!sel->hasNode(n1) || !sel->hasNode(n2)) return;
      fprintf(fp,"edgei %d %d %d\n",n1->getId(),n2->getId(),e->getId()); //write edge id
      writeOwnFormatAttrs(fp,e,dom);			//write edge attrs	
      fprintf(fp,"end\n");				//ready with writing node
}

int SaveOp::writeOwnFormat(Selection* inp,const char* fname)
//Writes selection to a file in our own format. 
{
    FILE* fp = fopen(fname,"w");			//open file to write to
    if (!fp) return 0;
    DomainModel* dom = GraphManager::getInstance()->getDomainModel();

    Node* n; Selection::NodeIterator* i;  
    for(i=inp->iterNodes();(n=i->next());)		//for all nodes in selection: write them to file						
	writeOwnFormatNode(fp,n,dom);
    Edge* e; Selection::EdgeIterator* j;
    for(j=inp->iterEdges();(e=j->next());)		//for all edges in selection: write them to file						
	writeOwnFormatEdge(fp,e,inp,dom);

    fclose(fp);						//ready with writing file
    return 1;
}


void SaveOp::writeRSFNode(FILE* fp,Node* n)		//Write a single node to an RSF file
//REMARK: This method could be done more efficiently by skipping writing the node-decl altogether if
//        its type were inferrable from the Rigiarc-decl of one of its arcs. For ex, if 'n1' were of 
//        type 'T' and involved in an arc of type 'arctype T U', then finding 'arctype n1 whatever'
//        is enough to say that n1 is of type T. 
//        We don't do this sheck yet since we don't assume the current domain-model is RSF necessarily...
{
    const AttrSetType* ast = n->getType();
    STRING node_name  = getRSFNodeName(n);              //Use node name-attr or id, if no name-attr
    STRING type_name  = ast->getName();
    if (type_name=="") type_name = "_UnknownNodeType_"; //If this has an unknown type, write this explicitly in the RSF file

    fprintf(fp,"type %s %s\n",(const char*)node_name,(const char*)type_name);

    for(int i=0;i<ast->numAttrTypes();i++)		//write node's attrs (as strings):
    {
       const char* attr_name = ast->getAttrType(i)->getName();
       const Attr*         a = n->get(attr_name);
       if (a && a->getType()!=Attr::UNSET)		//write to file only attributes that a) occur both in the node-instance
       {						//AND its type and b) have been set, i.e. don't have their default values
          const char* val = a->getString();		//a) is to ensure the saved stuff is indeed compatible to the node-type's declaration.	
	  if (val[0]!='\0')				//b) is to limit file size
	     fprintf(fp,"%s %s %s\n",attr_name,(const char*)node_name,val);
       }
    }
}

STRING SaveOp::getRSFNodeName(Node* n)                  //Given a node, returns a textual-rep of its
{                                                       //name as it should appear in the RSF file.
   STRING ret = n->getString("name");                   //First we try the "name" attr. If this is not available,
   if (ret=="") ret = n->getId();                       //we simply convert its id to a textual-rep
   return ret;
}


void SaveOp::writeRSFEdge(FILE* fp,Edge* e,Selection* sel) //Write a single edge to an RSF file
{
    Node*          f = e->fromNode();
    Node*          t = e->toNode();
    STRING type_name = e->getType()->getName();
    if (type_name=="") type_name = "_UnknownArcType_";     //If this has an unknown type, write this explicitly in the RSF file     
							   
    if (!sel->hasNode(f) || !sel->hasNode(t)) return;      //skip writing edges leading to nodes outside 'sel'
    
    STRING n1 = getRSFNodeName(f);                         //Use nodes' name-attrs or ids, if no name-attrs
    STRING n2 = getRSFNodeName(t);
    fprintf(fp,"%s %s %s\n",(const char*)type_name,(const char*)n1,(const char*)n2);

    //REMARK: We could save arc-attrs as well - but how to store them in RSF's syntax?
    //
    //
}



int SaveOp::writeRSF(Selection* inp,const char* fname)	//Write a selection to an RSF file
{
    FILE* fp = fopen(fname,"w"); if (!fp) return 0;

    Node* n; Selection::NodeIterator* i;  
    for(i=inp->iterNodes();(n=i->next());)		//for all nodes in selection:
    {							//write them to file
	writeRSFNode(fp,n);
    } 	

    Edge* e; Selection::EdgeIterator* j;		//write edges in selection
    for(j=inp->iterEdges();(e=j->next());)		//(only those that connect nodes in that selection because
    {							//those are the only nodes we write as well...)
	writeRSFEdge(fp,e,inp);
    }

    fclose(fp);
    return 1;
}



//--------------- SaveOp::writeChaco routines  ------------------------------

static char* idx_name = "_idx";                         //Attr-name for the attrs we generate locally

static void writeChacoNode(FILE* fp,Node* n,Selection* inp)
{
    EdgeSet& es = n->getEdges(inp,Node::BOTH);		//writing a node means writing its nb-list, i.e.
    
    for(int i=0;i<es.Count();i++)			//the idxs of all nodes it is connected with
       fprintf(fp,"%d ",es[i]->otherNode(n)->getInt(idx_name));
    fprintf(fp,"\n");
    
}


int SaveOp::writeChaco(Selection* inp,const char* fname)
{
    FILE* fp = fopen(fname,"w"); if (!fp) return 0;

    Node* n; Selection::NodeIterator* i;  
    int idx = 1;					//number nodes from 1
    int ne  = 0;					//'ne' counts #edges we shall write
    for(i=inp->iterNodes();(n=i->next());)		//for all nodes in selection:
    {							//set the attribute 'idx_name' for them and also
							//count the #edges we shall write
       int e = n->getEdges(inp,Node::BOTH).Count();
       ne += e;
       n->set(idx_name,idx); idx++;
    }

    ne /=2;                                             //we counted the edges twice...

    fprintf(fp,"% Graph file in Chaco 2.2 format\n");
    fprintf(fp,"%d %d\n",inp->numNodes(),ne);		//output #nodes and #edges we shall write

    for(i=inp->iterNodes();(n=i->next());)		//for all nodes in selection:
    {							//write the nb-list for every node
	writeChacoNode(fp,n,inp);
    } 	

    inp->delNodeAttr(idx_name);				//done with the idx_name, delete it from all nodes

    fclose(fp);
    return 1;
}