#include "PathQualifier.h"
#include "GUIManager.h"
#include "utils.h"
#include <tcl.h>


DARRAY<STRING> PathQualifier::paths;
STRING         PathQualifier::crt_path;


STRING  PathQualifier::qualify(const char* p)           //Given a filename, relative or not, this returns a
{                                                       //qualified (absolute) filename that can be passed to
   STRING res;                                          //file ops. 

   Tcl_PathType pt = Tcl_GetPathType((char*)p);         //1. if fname already absolute, nothing to do
   if (pt==TCL_PATH_ABSOLUTE) res = p;
   else
   {
      for(int i=0;i<paths.Count()+2;i++)                //2. try all qualification options:
      {
         STRING cp = (i==0)?             crt_path   :   //2.1. first, qualify with the 'current path'
                     (i<=paths.Count())? paths[i-1] :   //2.2. next, qualify with the 'system paths'
                                         ".";           //2.3. finally, qualify with the process's own path (pwd)
         if (cp=="") continue;
         
         res = "file exists "; res += addQuotes(cp+"/"+p);
         if (GUIManager::exec(res))
            if (GUIManager::getLongResult()==1) 
            {  res = cp; res += "/"; res += p; return res; }
      }
      res = p;                                          //we arrive here if we can't qualify 'p' in any way
   }

   return res;
}



STRING  PathQualifier::dir(const char* p)
{
    STRING s = "file dirname "; s += addQuotes(p);
    if (GUIManager::exec(s)) 
       return GUIManager::getStringResult();
    return p;
}

STRING PathQualifier::cannonical(const char* p)
{
    STRING s = p;
    for(int i=0;i<s.length();i++)
       if (s[i]=='\\') s[i] = '/';
    return s;
}


STRING  PathQualifier::setCurrentPath(const char* p)
{                                                       //sets the current path from a filename
    STRING old_crt_path = crt_path;                     //also, returns the current value of the crt_path
    crt_path = "";                                      //if anything fails, the crt-path will be ""

    if (!p || !p[0]) return old_crt_path;               //'p' empty, nothing to do.
    
    STRING s = p;
    if (s[s.length()-1]=='/')                           //p already a path, ends with '/', don't process it further
    {
       s[s.length()-1] = '\0'; crt_path = s;            //remember p, but without trailing '/'
    }
    else                                                //p not a path, process it via "file dirname"
    {
       s = "file dirname "; s += addQuotes(p);          //try to get the path-part from 'p'
       if (GUIManager::exec(s))                         //if succeeded, remember it
          crt_path = GUIManager::getStringResult();
    }
    return old_crt_path;
}



STRING PathQualifier::tail(const char* p)               //get tail (filename+ext) out of path
{
    STRING res = "file tail "; res += addQuotes(p);
    if (!GUIManager::exec(res)) return "";
    return GUIManager::getStringResult();
}




STRING PathQualifier::removeExtension(const char* p)    //remove extension from arg, if any
{
     char* dot = strrchr((char*)p,'.');                 //search for last '.' in arg
     if (!dot) return p;                                //none found, return arg itself, it has no extension

     STRING ret = p; ret[dot-p] = '\0';                 //cut off extension, return rest
     return ret;

}


void PathQualifier::reset()                             //reset whatever info this might have set
{                                                       //(so far, not very much)
     
     crt_path = "";                                     //reset current-path. Must be "" anyway, but we do it to be sure
}



void PathQualifier::addSystemPath(const char* p)
{
    paths.Add(p);
}

