#include "RSFDomainModel.h"
#include "RSFAttrSetType.h"
#include "GenAttrSetType.h"
#include "AttrType.h"
#include "GUIManager.h"
#include "utils.h"
#include <iostream.h>



RSFDomainModel::RSFDomainModel(const char* n): DomainModel(n),nodetype(0),arctype(0)
{  }


RSFDomainModel::~RSFDomainModel()	
{
    delete nodetype;				//Destroy the nodetype/arctype implementation objs
    delete arctype;				//We can safely do this since we're the only owners of them
}



int RSFDomainModel::constr(const char* nodef,const char* arcf,const char* attrf)
{
    FILE* ndf = fopen(nodef,"r");		//1. All 3 Rigi files must be readable
    if (!ndf) 
    { GUIManager::error("RSFDomainModel::constr: can not open file",nodef); return 0; }
    FILE* arf = fopen(arcf,"r");
    if (!arf)
    { GUIManager::error("RSFDomainModel::constr: can not open file",arcf); return 0; }
    FILE* atf = fopen(attrf,"r");
    if (!atf)
    { GUIManager::error("RSFDomainModel::constr: can not open file",attrf); return 0; }

    delete nodetype;				//2. Delete and remake unique types for nodes and edges
    delete arctype;				//   These type-objects will be shared among all RSF types

    nodetype = new GenericAttrSetType("RSF node type");
    arctype  = new GenericAttrSetType("RSF arc type");

    int b1=read_node_file(ndf);			//3. Read the 3 Rigi files. This will populate the
    fclose(ndf);
    int b2=read_attr_file(atf);			//   superclass with AttrSetTypes as well as the
    fclose(atf);
    int b3=read_arc_file(arf);    		//   nodetype/arctype objects of this.
    fclose(arf);


    return b1 && b2 && b3;
}


void RSFDomainModel::save(FILE* fp)             //Impl inh - writes cmd to reload this to Tcl-file
{                                               //We save this ONLY if we have a valid domain_file.
    if (domain_file!="")                        //If not, it means we use a DomainModel NOT loaded from a file, 
                                                //e.g. the default one, which we shouldn't save.
       fprintf(fp,"pr_domain_load_rsf %s\n",(const char*)addQuotes(domain_file));
}


int RSFDomainModel::read_node_file(FILE* fp)	//Read contents of Riginode file
{						//For every entry (nodetype-name), make a new
    char line[512],buf[128];			//RSFAttrSetType entry with that name, and add it to this
						//That entry will use the unique 'nodetype' object for
    for(;fgets(line,511,fp);)			//its attrs' declarations, since the RSF model specifies 	
    {						//that all nodetypes have the same attrs.
       if (line[0]=='#' || line[0]=='\n') continue;
       if (sscanf(line,"%s",buf)!=1) break;
       addNodeType(new RSFAttrSetType(buf,nodetype));
    }
    return 1;

    						// Add the default node-type "Composite"
						//(the RSF manual says it is automatically added if 
						//not explicitly declared in the domain-model files
    if (getNodeType("Composite")==getUnknownArcType())
	addNodeType(new RSFAttrSetType("Composite",nodetype));
}


int RSFDomainModel::read_attr_file(FILE* fp)	//Read contents of Rigiattr file
{						//Make+add to this's nodetype/arctype new AttrTypes,
    char line[512],attr[128],			//as read from the file. The attr-types are STRING for RSF.
         node_arc[128],val[128];

    for(;fgets(line,511,fp);)
    {
       if (line[0]=='#' || line[0]=='\n') continue;
       if (sscanf(line,"%s%s%s",attr,node_arc,val)!=3) break;
       if (strcmp(attr,"attr"))
       {
	  GUIManager::error("RSFDomainModel::read_attr_file: attr expected but found: ",attr);
	  return 0;
       }
       else
       {
         if (!strcmp(node_arc,"Node"))		//new node-attr found:
	 {
	    nodetype->add(new AttrType(val,Attr::STRING));
	 }

         else
         if (!strcmp(node_arc,"Arc"))		//new arc-attr found:
            arctype->add(new AttrType(val,Attr::STRING));
         else
         {
	   GUIManager::error("RSFDomainModel::read_attr_file: Node/Arc expected but found: ",node_arc);
	   return 0;
         }
       }
    }

    return 1;
}


int RSFDomainModel::read_arc_file(FILE* fp)		//Read contents of Rigiarc file	
{							//For every entry (arctype-name + 2 optional args) 
    char line[256],arct[128],				//make a new RSFArcType entry with that name.
         ntype1[128],ntype2[128];   			//That entry will use the unique 'arctype' entry
							//for its attr-names, since the RSF model specifies that
    for(;fgets(line,255,fp);)				//all arctypes have the same attrs.
    {
       if (line[0]=='#' || line[0]=='\n') continue;	//Skip comments
       ntype1[0] = ntype2[0] = '\0';			//Reset buffers		
	
       int n = sscanf(line,"%s%s%s",arct,ntype1,ntype2);
       if (n!=1 && n!=3) 
       {
	  GUIManager::error("RSFDomainModel::read_arc_file: incorrect argument count: ",n);
	  return 0;
       }	
       addArcType(new RSFArcType(arct,arctype,ntype1,ntype2));
    }
							// Add the default arc-types "level" and "composite"
							//(the RSF manual says they are automatically added if they are
							//not explicitly declared in the domain-model files
    if (getArcType("level")==getUnknownArcType())
	addArcType(new RSFArcType("level",arctype,"",""));
    if (getArcType("composite")==getUnknownArcType())
        addArcType(new RSFArcType("composite",arctype,"",""));

    return 1;
}

