#include "SoCachedFile.h"
#include <Inventor/SoDB.h>
#include <Inventor/nodes/SoSeparator.h>

std::map<STRING,SoNode*> SoCachedFile::cache;					//static class member





SoCachedFile::SoCachedFile(SoNode* ret, const char* fn): file_name(fn)	//Ctor: cache file_name internally and
{									//	add real contents to this
   addChild(ret);
}


SoNode* SoCachedFile::read(const char* fname)				//Main routine: returns cached file-object,
{									//constructs (& caches) new one, or returns 0 on failure
   std::map<STRING,SoNode*>::iterator i = cache.find(fname);
   if (i!=cache.end()) return (*i).second;				//1. Is obj for this filename cached? If so, return it.
   
   SoInput inp;								//2. Try to read contents of given file.
   if (!inp.openFile(fname)) return 0;
   SoNode* ret = SoDB::readAll(&inp);
   inp.closeFile();
   if (!ret) return 0;
   
   SoCachedFile* grp = new SoCachedFile(ret,fname);			//3. Stuff read contents & filename in a new SoCachedFile group
									//   Insert group in global 'cache' looked up by filename
   std::map<STRING,SoNode*>::value_type v(fname,grp);			
   cache.insert(v);   
   
   return grp;
}


SoCachedFile::~SoCachedFile()						//Dtor: since this goes away it means ALL its instances in the scenegraph are	
{									//gone. So we can safely remove its entry from the cache.
   cache.erase(cache.find(file_name));
}


void SoCachedFile::flush(const char* fname)				//Simply removes entry for given filename
{									//from cache - or flushes whole cache if no
   if (!fname) cache.clear();						//filename given
   else cache.erase(cache.find(fname));
}