#---------------------------------------------------------------------------
#  GUI for visualising/editing attrs of a node/edge
#---------------------------------------------------------------------------


# Create dialog window
#-------------------------------------------------------------------------
proc pr_gui:attrset {wid} {
   if {[info commands $wid]!={}} { raise $wid; return }

   toplevel $wid
   pr_list:build $wid {Name Value} { {Ok "pr_del_observer %W; destroy %W"} \
                  {Refresh "pr_gui:attrset_refresh %W"} } ""

   entry       $wid.entry 
   pack        $wid.entry -fill x

   frame $wid.f1; pack $wid.f1 -fill x
   frame $wid.f2; pack $wid.f2 -fill x

   frame $wid.f1.f1; pack $wid.f1.f1 -side left
   frame $wid.f1.f2; pack $wid.f1.f2 -side right
   frame $wid.f2.f1; pack $wid.f2.f1 -side left
   frame $wid.f2.f2; pack $wid.f2.f2 -side right

   label $wid.f1.f1.l1 -text "Id:";   pack $wid.f1.f1.l1  -anchor w
   label $wid.f1.f2.id;		        pack $wid.f1.f2.id  -anchor w
   label $wid.f2.f1.l1 -text "Type:"; pack $wid.f2.f1.l1  -anchor w
   label $wid.f2.f2.typ;	        pack $wid.f2.f2.typ -anchor w
   
   bind $wid <Destroy> { pr_del_observer %W; destroy %W }
   pr_add_observer $wid [concat pr_gui:attrset_refresh $wid]
   pr_gui:attrset_refresh $wid
}


#Refresh the GUI to show details of a node given by 'id'
#-------------------------------------------------------------------------
proc pr_gui:attrset_refresh_id {wid nid} {

    #First check the widget exists indeed
    if {[info commands $wid]=={}} { return }

    $wid.entry  delete 0   [string length [$wid.entry get]]
    $wid.entry  insert 0   $nid
    pr_gui:attrset_refresh $wid
}



#--------------------------------------------------------------------------
# Private (implementation) part 
#--------------------------------------------------------------------------



#Refresh the GUI to show details of a node/edge given by 'name'
#--------------------------------------------------------------------------
proc pr_gui:attrset_refresh {wid} {

    #First check this widget exists indeed
    if {[info commands $wid]=={}} { return }

    set hl [$wid.top.a subwidget hlist]
    set as [$wid.entry get]

    if {$as==""} return

    #Then try to get node by id/name. If failed, try to get edge by id/name.
    #If both fail, this id is somehow not in use, so clear this and return.
    if [string is integer $as] {set opt "id"} else {set opt "attr name"}
    set at [pr_get_node All $opt $as]
    if {$at!=0} {set pr_attr pr_node} else {
       set pr_attr pr_edge
       set at [pr_get_edge All $opt $as]
    }
        
# Must check whether the returned node $at is null or not, since this refresh
# might be called after a cleanup of the system, for example.
#    
    if {$at!=0} {
      $wid.f1.f2.id   configure -text [$pr_attr $at id]
      $wid.f2.f2.typ  configure -text [$pr_attr $at type]
      set l [$pr_attr $at attrs]
      pr_list:browse $wid $l
    } else {
      $wid.f1.f2.id   configure -text ""
      $wid.f2.f2.typ  configure -text ""
      pr_list:browse $wid {}
    }
}

