
# colormap{wid func}: Main entry point to the colormap visualization GUI.
#	     	      Constructs a colormap GUI for the function 'func'
#		      'func' receives a scalar in [0..1] and returns a list {r,g,b}
#		      of color scalars in [0..1]
#--------------------------------------------------------------------------

proc pr_gui:colormap {wid func} {
  toplevel .$wid

  pr_gui:colormap:make .$wid.cmap $func
  pack .$wid.cmap -expand yes -side top -fill both

  tixSelect .$wid.s
  pack .$wid.s -expand yes -side bottom -fill x

  .$wid.s add ok -text "Ok" -command "destroy .$wid"
  .$wid.s add up -text "XX" -command "destroy .$wid.cmap; pr_gui:colormap:make .$wid.cmap $func"

  return ""
}


# make{wid func}: Constructs the body of the colormap GUI
#	     
#--------------------------------------------------------------------------
proc pr_gui:colormap:make {wid func} {

  canvas $wid -height 20 -width 200
  pack $wid -fill x

  set wd [$wid cget -width]
  set ht [$wid cget -height]
  set dx [expr $wd/100.0]
  set dy $ht

  set i 0
  while {$i<100} {

    pr_gui:colormap:rect $wid [expr $i * $dx] 0 $dx $dy \
                              [pr_gui:colormap:list2rgb [$func [expr double($i)/100]]]
    incr i
  }
}



# rect{wid x y dx dy col}: Makes a rectangle graphical item in a canvas
#	     
#--------------------------------------------------------------------------
proc pr_gui:colormap:rect {wid x y dx dy col} {
  $wid create rectangle $x $y [expr $x+$dx] [expr $y+$dy] -fill $col -outline ""
}


# list2rgb{l}: Returns a Tcl-compliant color string "#rrggbb" made from 
#	       a list l of 3 floats {r,g,b}	
#--------------------------------------------------------------------------
proc pr_gui:colormap:list2rgb {l} {

  set r [expr int(255*[lindex $l 0])]
  if {$r>255} {set r 255}
  if {$r<0}   {set r 0}
  set g [expr int(255*[lindex $l 1])]
  if {$g>255} {set g 255}
  if {$g<0}   {set g 0}  
  set b [expr int(255*[lindex $l 2])]
  if {$b>255} {set b 255}
  if {$b<0}   {set b 0}

  set c [format %06x [expr ($r<<16) + ($g<<8) + ($b)]]
  return [format %s%s  # $c]
}


proc fff {val} {

  set r $val
  set g $r
  set b $r
  return [list $r $g $b]
}


