#
# This file contains the Tk GUIs for the domain-models
#
#--------------------------------------------------------------------



proc pr_gui:dmodels {wid} {
   if {[info commands .$wid]!={}} { raise .$wid; return }
   toplevel .$wid

   tixButtonBox .$wid.box -orientation horizontal
   .$wid.box add 0 -text "Refresh" -width 5 -command "pr_gui:dmodels_refresh .$wid"
   .$wid.box add 1 -text "Reset"   -width 5 -command "pr_domain reset"
   .$wid.box add 2 -text "Load"    -width 5 -command "pr_load_domain"
   .$wid.box add 3 -text "Dismiss" -width 5 -command "pr_del_observer %W; destroy .$wid"
   pack .$wid.box -side bottom -anchor s -fill x 

   set opts { label.width 15 label.anchor w }
   tixLabelEntry .$wid.dom -label "Domain model" -options $opts 
   pack .$wid.dom -fill x -side bottom -anchor n

   frame .$wid.f
   frame .$wid.f.f1
   pack .$wid.f.f1 -expand yes -side left -fill both
   pr_list:build .$wid.f.f1 {"Node type"} { } "pr_gui:dmodels:sel_ntype .$wid"
   frame .$wid.f.f2
   pack .$wid.f.f2 -expand yes -side left -after .$wid.f.f1 -fill both 
   pr_list:build .$wid.f.f2 {"Edge type"} { } "pr_gui:dmodels:sel_etype .$wid"
   frame .$wid.f.f3
   pack .$wid.f.f3 -expand yes -side left -after .$wid.f.f2 -fill both
   pr_list:build .$wid.f.f3 {"Attribute name" "Type"} { } ""
 
   pack .$wid.f -fill both -expand yes -anchor n -side top
   
   bind .$wid <Destroy> { pr_del_observer %W; destroy %W }
   pr_add_observer .$wid [concat pr_gui:dmodels_refresh .$wid]
   pr_gui:dmodels_refresh .$wid
}

#  Private part
#------------------------------------------------------------------------------------

# pr_gui:dmodels_refresh: Refresh the two leftmost (node & edge attr-set-type panels) from kernel.
#
proc pr_gui:dmodels_refresh {w} {
   set dom [pr_domain name]

   #Translate "" empty domain name to something more readable
   if {$dom==""} { set dom "<default domain model>" }

   set v [$w.dom subwidget entry]
   $v  delete 0 [string length [$v get]]
   $v  insert 0 $dom

   set n  [pr_domain node_types]; set i 0; set l {}
   while {$i<$n} {
     set nt_name [pr_domain node_type i $i]
     lappend l $nt_name
     incr i
   }
   pr_list:browse $w.f.f1 $l

   set n  [pr_domain edge_types]; set i 0; set l {}
   while {$i<$n} {
     set et_name [pr_domain edge_type i $i]
     lappend l $et_name
     incr i
   }
   pr_list:browse $w.f.f2 $l

   #Clear all in the 'Type & Value' panel
   #
   pr_gui:dmodels:refresh_type $w {}
}

proc pr_gui:dmodels:sel_ntype {w sel} {
  set nt [pr_domain node_type idx $sel]
  pr_gui:dmodels:refresh_type $w $nt
}

proc pr_gui:dmodels:sel_etype {w sel} {
  set et [pr_domain edge_type idx $sel]
  pr_gui:dmodels:refresh_type $w $et

}

proc pr_gui:dmodels:refresh_type {w l} {
     pr_list:browse $w.f.f3 $l
}


