#  Procedures for editing Tcl scripts (procs/code fragments) from
#  within the system.
#
#--------------------------------------------------------------------


#  pr_gui:edit_proc {wid}: main entry point. Opens a dialog to edit or delete an existing
#			   Tcl proc, or to write a new one, with given name+args.
#---------------------------------------------------------------------------------------
proc pr_gui:edit_proc {wid proc args} {
   if {[info commands .$wid]!={}} { 
     raise .$wid
     return
   }

   #Must provide a non-empty procedure name and arglist!
   #
   if {[string is space $proc]} return
   if {[string is space $args]} return

   toplevel .$wid

   tixScrolledText .$wid.ed 
   pack .$wid.ed -expand yes -fill both

   tixSelect .$wid.box 
   .$wid.box add ok  -text "Apply"   -command "pr_gui:edit_proc:apply  .$wid"
   .$wid.box add del -text "Delete"  -command "pr_gui:edit_proc:delete .$wid $proc"
   .$wid.box add cc  -text "Dismiss" -command "destroy .$wid"

   tixBalloon .$wid.box.ok_b -initwait 10
   .$wid.box.ok_b bind [.$wid.box subwidget ok] -msg [concat "Set edited code to Tcl proc called " $proc]
   tixBalloon .$wid.box.del_b -initwait 10
   .$wid.box.del_b bind [.$wid.box subwidget del] -msg [concat "Delete Tcl proc called " $proc]


   pack .$wid.box -side bottom

   bind .$wid <Destroy> { destroy %W }

   pr_gui:edit_proc:refresh .$wid $proc $args
}








#--------------------------------------------------------------------------
# Private (implementation) part 
#-------------------------------------------------------------------------------



# Refreshes the GUI completely from the core data, via the Tcl API
#
#-------------------------------------------------------------------------------
proc pr_gui:edit_proc:refresh {wid proc args} {
   set ed $wid.ed 
   if {[info procs $proc]!=""} {
     #There is a Tcl proc with that name, edit it using its actual arg-names
     #
     [$ed subwidget text] insert end "proc "
     [$ed subwidget text] insert end $proc
     [$ed subwidget text] insert end " {"
     [$ed subwidget text] insert end [info args $proc]
     [$ed subwidget text] insert end "} {"
     [$ed subwidget text] insert end [info body $proc]
     [$ed subwidget text] insert end "}"
   } else {
     #No Tcl proc with that name, edit new onw with given name+args
     #
     [$ed subwidget text] insert end "proc "
     [$ed subwidget text] insert end $proc
     [$ed subwidget text] insert end " {"
     [$ed subwidget text] insert end $args
     [$ed subwidget text] insert end "} {\n"
     [$ed subwidget text] insert end "\n}\n"
   }
}



# Sends the contents of the editor to Tcl to create/modify a new procedure
#
#-------------------------------------------------------------------------
proc pr_gui:edit_proc:apply {wid} {
  set ed $wid.ed

  set txt [[$ed subwidget text] get 0.0 end]
  
  eval $txt
}



# Deletes the Tcl procedure currently edited, if any is there, and
# closes the GUI dialog.
#------------------------------------------------------------------
proc pr_gui:edit_proc:delete {wid proc} {

  if {[info procs $proc]!=""} {
     rename $proc ""
  }

  destroy $wid
}

