#-----------------------------------------------------------------------------------
# System-wide initialization script - to be loaded first
# 
#-----------------------------------------------------------------------------------


#
# Load the extra Tix widgets - Tix 8.0 or higher should be available
#
package require Tix 8.0

#
# Configure stuff for the main window "."
#
wm title . "Main"
#wm iconbitmap . [image create bitmap -file [pr_qualify RSCS/filenew.gif]]      
bind . <Destroy> { pr_exit }
frame .top
pack .top -fill both -expand yes

#
# Choose a smaller font than the default one
# Warning: different Tix installations might allow different smallest fontsizes
#
tix resetoptions TK 12Point

#
# Initialize various components that depend on machine-specific paths
#
#pr_gem_init "/home/cream/NOKIA/GEM"
 pr_gem_init "d:/mydocu~1/nokia/gem"
#pr_gem_init "c:/gem"

#pr_gv_init "/usr/local/gv1.5/bin"
# Don't need pr_gv_init usually, dot/neato are on the path...
pr_glyphs init

#
# Load additional application components written in Tcl
#
pr_read_tcl TCL/list.tcl
pr_read_tcl TCL/view.tcl
pr_read_tcl TCL/attrset.tcl
pr_read_tcl TCL/load.tcl
pr_read_tcl TCL/save.tcl
pr_read_tcl TCL/glyphs.tcl
pr_read_tcl TCL/layouts.tcl
pr_read_tcl TCL/colormap.tcl
pr_read_tcl TCL/editproc.tcl
pr_read_tcl TCL/dialogs.tcl
pr_read_tcl TCL/dmodels.tcl
pr_read_tcl TCL/settings.tcl
pr_compo_lib load TCL/misc.tcl
pr_read_tcl TCL/reginit.tcl


#--------------------------------------------------------------------------------------
#  Start building the toplevel GUI
#
#--------------------------------------------------------------------------------------


tixSelect .top.bbox -pady 1
.top.bbox add b1  -text New      -image [image create photo -file [pr_qualify RSCS/filenew.gif]]      -command pr_reset
.top.bbox add b2  -text Load     -image [image create photo -file [pr_qualify RSCS/fileopen.gif]]     -command pr_load_dlg
.top.bbox add b20 -text Load_DM  -image [image create photo -file [pr_qualify RSCS/domopen.gif]]      -command { pr_gui:dmodels domains }
.top.bbox add b21 -text Save     -image [image create photo -file [pr_qualify RSCS/filesave.gif]]     -command { pr_gui:save dialog }
.top.bbox add b3  -text View     -image [image create photo -file [pr_qualify RSCS/viewer.gif]]       -command { pr_gui:view viewers }
.top.bbox add b4  -text Sels     -image [image create photo -file [pr_qualify RSCS/objects.gif]]      -command { pr_gui:sel selections }
.top.bbox add b5  -text Nodes    -image [image create photo -file [pr_qualify RSCS/nodes.gif]]        -command { pr_gui:attrset .nodes }
.top.bbox add b6  -text Layouts  -image [image create photo -file [pr_qualify RSCS/tree.gif]]  	      -command { pr_gui:layouts layouts }
.top.bbox add b7  -text Glyphs   -image [image create photo -file [pr_qualify RSCS/cube.gif]] 	      -command { pr_gui:glyphs glyphs }
.top.bbox add b8  -text Settings -image [image create photo -file [pr_qualify RSCS/settings.gif]]     -command { pr_gui:settings settings }
.top.bbox add b9  -text Help     -image [image create photo -file [pr_qualify RSCS/help.gif]] 
.top.bbox add b10 -text Quit     -image [image create photo -file [pr_qualify RSCS/finish.gif]]       -command pr_exit 
pack .top.bbox -side top -anchor w



tixBalloon .top.bbox.b1_b -initwait 10
.top.bbox.b1_b bind [.top.bbox subwidget b1] -msg "Clear all data"
tixBalloon .top.bbox.b2_b -initwait 10
.top.bbox.b2_b bind [.top.bbox subwidget b2] -msg "Load data"
tixBalloon .top.bbox.b20_b -initwait 10
.top.bbox.b20_b bind [.top.bbox subwidget b20] -msg "Load domain model"
tixBalloon .top.bbox.b21_b -initwait 10
.top.bbox.b21_b bind [.top.bbox subwidget b21] -msg "Save data"
tixBalloon .top.bbox.b3_b -initwait 10
.top.bbox.b3_b bind [.top.bbox subwidget b3] -msg "Viewer panel"
tixBalloon .top.bbox.b4_b -initwait 10
.top.bbox.b4_b bind [.top.bbox subwidget b4] -msg "Selections panel"
tixBalloon .top.bbox.b5_b -initwait 10
.top.bbox.b5_b bind [.top.bbox subwidget b5] -msg "Node details panel"
tixBalloon .top.bbox.b6_b -initwait 10
.top.bbox.b6_b bind [.top.bbox subwidget b6] -msg "Layout engines panel"
tixBalloon .top.bbox.b7_b -initwait 10
.top.bbox.b7_b bind [.top.bbox subwidget b7] -msg "Glyph engines panel"
tixBalloon .top.bbox.b8_b -initwait 10
.top.bbox.b8_b bind [.top.bbox subwidget b8] -msg "Settings"
tixBalloon .top.bbox.b9_b -initwait 10
.top.bbox.b9_b bind [.top.bbox subwidget b9] -msg "Provide help"
tixBalloon .top.bbox.b10_b -initwait 10
.top.bbox.b10_b bind [.top.bbox subwidget b10] -msg "Quit"

#
# Build the text input (command area) widget
#
tixScrolledText .top.tcw  -height 300 
pack .top.tcw -side bottom -fill both -expand yes
set tcww [.top.tcw subwidget text]



#Script associated with <Return> in the cmd-widget. Does something similar to
#what the tclsh/wish shells do.
#--------------------------------------------------------------------
bind $tcww <KeyPress-Return> { 
       set res [eval [$tcww get {end -1 lines} {end -1 chars}]]
       if {$res!=""} {
	  $tcww insert end "\n"
          $tcww insert end $res
       }
}

#--------------------------------------------------------------------------
#  End of init.tcl
#--------------------------------------------------------------------------


