#
# This file contains the Tk GUIs for setting global parameters of the
# supported layout-engines
#
#--------------------------------------------------------------------



proc pr_gui:layouts {wid} {
   if {[info commands .$wid]!={}} { raise .$wid; return }
   
   toplevel .$wid
   tixNoteBook .$wid.nb
   .$wid.nb add dot    -label "dot"     -raisecmd [concat pr_gui:layouts:refresh_dot    .$wid.nb]
   .$wid.nb add neato  -label "neato"   -raisecmd [concat pr_gui:layouts:refresh_neato  .$wid.nb]
   .$wid.nb add gem    -label "gem"     -raisecmd [concat pr_gui:layouts:refresh_gem    .$wid.nb]
   .$wid.nb add grid   -label "grid"    -raisecmd [concat pr_gui:layouts:refresh_grid   .$wid.nb]
   .$wid.nb add nested -label "nested"  -raisecmd [concat pr_gui:layouts:refresh_nested .$wid.nb]
   .$wid.nb add level  -label "level"   -raisecmd [concat pr_gui:layouts:refresh_level  .$wid.nb]
   .$wid.nb add genrl  -label "general" -raisecmd [concat pr_gui:layouts:refresh_genrl  .$wid.nb]
   pr_gui:layouts:make_genrl  [.$wid.nb subwidget genrl]
   pr_gui:layouts:make_dot    [.$wid.nb subwidget dot]
   pr_gui:layouts:make_neato  [.$wid.nb subwidget neato]
   pr_gui:layouts:make_gem    [.$wid.nb subwidget gem]
   pr_gui:layouts:make_grid   [.$wid.nb subwidget grid]
   pr_gui:layouts:make_level  [.$wid.nb subwidget level]
   pr_gui:layouts:make_nested [.$wid.nb subwidget nested]
   pack .$wid.nb -side top -expand yes -fill both

   tixButtonBox .$wid.box -orientation horizontal
   .$wid.box add ok -text "Apply"   -command "pr_gui:layouts:ok .$wid"
   .$wid.box add cc -text "Dismiss" -command "destroy .$wid"
   pack .$wid.box -side bottom -fill x

   bind .$wid <Destroy> { destroy %W }
}





#--------------------------------------------------------------------------
# Private (implementation) part 
#-------------------------------------------------------------------------------


# pr_gui:layouts:make_genrl {parent}: constructs the page for the DOT layout-engine
#--------------------------------------------------------------------------------
proc pr_gui:layouts:make_genrl {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 35 label.anchor w }

   label         $top.l      -text  "General Layout Parameters" -justify center; pack $top.l
   tixSelect     $top.keepsz -label "Keep node sizes" -options $opts\
		 -allowzero true; pack $top.keepsz -fill x
   tixLabelEntry $top.nodesz -label "Default node size" -options $opts; pack $top.nodesz -fill x
   tixSelect     $top.usepos -label "Pass position to layout servers" -options $opts\
		  -allowzero true; pack $top.usepos -fill x
   tixSelect     $top.usedim -label "Pass sizes to layout servers" -options $opts\
		  -allowzero true; pack $top.usedim -fill x

   $top.keepsz add btn
   [$top.keepsz subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
   $top.usepos add btn
   [$top.usepos subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
   $top.usedim add btn
   [$top.usedim subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
}

# Refreshes the genrl-layout page
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_genrl {wid} {
    set top [$wid subwidget genrl] 
    append top ".top"

    if {[pr_lay -get keep_dims]==1} { $top.keepsz config -value {btn} } else { $top.keepsz config -value {} }

    set v [$top.nodesz subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay -get node_size]   

    set v [$top.usepos subwidget btn]
    if {[pr_lay -get use_crt_pos]==1} { $top.usepos config -value {btn} } else { $top.usepos config -value {} }

    set v [$top.usedim subwidget btn]
    if {[pr_lay -get use_crt_dim]==1} { $top.usedim config -value {btn} } else { $top.usedim config -value {} }

}


# Copies params from the 'genrl' page to the kernel via the Tcl API.
#------------------------------------------------------------------------------
proc pr_gui:layouts:ok_genrl {wid} {
    set top [$wid subwidget genrl] 
    append top ".top"

    set kp [expr [string compare [$top.keepsz cget -value] btn] + 1]
    set sz [[$top.nodesz subwidget entry] get]
    set up [expr [string compare [$top.usepos cget -value] btn] + 1]
    set ud [expr [string compare [$top.usedim cget -value] btn] + 1]

    pr_lay -set keep_dims $kp 
    pr_lay -set node_size $sz
    pr_lay -set use_crt_pos $up
    pr_lay -set use_crt_dim $ud
}




# pr_gui:layouts:make_dot {parent}: constructs the page for the DOT layout-engine
#--------------------------------------------------------------------------------
proc pr_gui:layouts:make_dot {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 15 label.anchor w }

   label         $top.l      -text  "DOT Layout Parameters" -justify center; pack $top.l
   tixLabelEntry $top.rnksep -label "Rank distance" -options $opts; pack $top.rnksep -fill x
   tixLabelEntry $top.ndesep -label "Node distance" -options $opts; pack $top.ndesep -fill x
   tixSelect     $top.fliped -label "Flip edges"    -options $opts -allowzero true; pack $top.fliped -fill x
   $top.fliped   add btn
   [$top.fliped subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
   tixLabelEntry $top.rank   -label "Rank attribute" -options $opts; pack $top.rank -fill x	
   tixSelect     $top.rnkdir -label "Direction"     -options $opts \
		     -allowzero false -radio true; pack $top.rnkdir -fill x
   $top.rnkdir add btn1 -image [image create photo -file [pr_qualify RSCS/rightarrow.gif]]
   $top.rnkdir add btn2 -image [image create photo -file [pr_qualify RSCS/leftarrow.gif]]
   $top.rnkdir add btn3 -image [image create photo -file [pr_qualify RSCS/downarrow.gif]]
   $top.rnkdir add btn4 -image [image create photo -file [pr_qualify RSCS/uparrow.gif]]
}

# Refreshes the dot-layout page
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_dot {wid} {
    set top [$wid subwidget dot] 
    append top ".top"

    set v [$top.rnksep subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_dot -get ranksep]

    set v [$top.ndesep subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_dot -get nodesep]
  
    set v [$top.rank subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_dot -get rank_attr]

    set v [$top.fliped subwidget btn]
    if {[pr_lay_dot -get flip_edges]==1} { $top.fliped config -value {btn} } else { $top.fliped config -value {} }    

    set v [pr_lay_dot -get rankdir]
    if {$v==0} { $top.rnkdir configure -value {btn1} }
    if {$v==1} { $top.rnkdir configure -value {btn2} }
    if {$v==2} { $top.rnkdir configure -value {btn3} }
    if {$v==3} { $top.rnkdir configure -value {btn4} }
}


# Copies params from the 'dot' page to the kernel via the Tcl API.
#------------------------------------------------------------------------------
proc pr_gui:layouts:ok_dot {wid} {
    set top [$wid subwidget dot] 
    append top ".top"

    set rs [[$top.rnksep subwidget entry] get]
    set ns [[$top.ndesep subwidget entry] get]
    set fe [expr [string  compare [$top.fliped cget -value] btn] + 1]
    set ra [[$top.rank subwidget entry] get]
    set v  [$top.rnkdir cget -value]
    if [string equal $v btn1] { set rd 0 }
    if [string equal $v btn2] { set rd 1 } 
    if [string equal $v btn3] { set rd 2 }
    if [string equal $v btn4] { set rd 3 }

    pr_lay_dot -set ranksep $rs nodesep $ns rankdir $rd flip_edges $fe rank_attr $ra
}




# pr_gui:layouts:make_neato {parent}: constructs the page for the DOT layout-engine
#--------------------------------------------------------------------------------
proc pr_gui:layouts:make_neato {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 15 label.anchor w }

   label         $top.l       -text  "NEATO Layout Parameters" -justify center; pack $top.l
   tixLabelEntry $top.spring  -label "Spring strength"  -options $opts; pack $top.spring  -fill x
   tixLabelEntry $top.edgelen -label "Edge length"      -options $opts; pack $top.edgelen -fill x
   tixLabelEntry $top.energy  -label "Stop energy"      -options $opts; pack $top.energy  -fill x
   tixLabelEntry $top.maxitr  -label "Max iterations"   -options $opts; pack $top.maxitr  -fill x
   tixSelect     $top.random  -label "Randomize"        -options $opts\
		     -allowzero true; pack $top.random -fill x
   $top.random add btn
   [$top.random subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
   tixSelect     $top.overlap -label "Allow overlap"    -options $opts\
		     -allowzero true; pack $top.overlap -fill x
   $top.overlap add btn
   [$top.overlap subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
}


# Refreshes the neato-layout page
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_neato {wid} {
    set top [$wid subwidget neato] 
    append top ".top"

    set v [$top.spring subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_neato -get weight]

    set v [$top.edgelen subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_neato -get edgelen]

    set v [$top.energy subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_neato -get eps]

    set v [$top.maxitr subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_neato -get maxiter]

    set v [$top.random subwidget btn]
    if {[pr_lay_neato -get random]==1} { $top.random config -value {btn} } else { $top.random config -value {} }
    set v [$top.overlap subwidget btn]
    if {[pr_lay_neato -get overlap]==1} { $top.overlap config -value {btn} } else { $top.overlap config -value {} }
}


# Copies params from the 'neato' page to the kernel via the Tcl API.
#------------------------------------------------------------------------------
proc pr_gui:layouts:ok_neato {wid} {
    set top [$wid subwidget neato] 
    append top ".top"

    set sp [[$top.spring  subwidget entry] get]
    set en [[$top.energy  subwidget entry] get]
    set el [[$top.edgelen subwidget entry] get]
    set it [[$top.maxitr  subwidget entry] get]
    set rn [expr [string  compare [$top.random cget -value] btn] + 1]
    set ov [expr [string  compare [$top.overlap cget -value] btn] + 1]

    pr_lay_neato -set weight $sp eps $en maxiter $it random $rn overlap $ov edgelen $el
}




# pr_gui:layouts:make_gem {parent}: constructs the page for the DOT layout-engine
#--------------------------------------------------------------------------------
proc pr_gui:layouts:make_gem {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 15 label.anchor w }

   label         $top.l  -text  "GEM Layout Parameters" -justify center; pack $top.l
   tixLabelEntry $top.f1 -label "Distance" -options $opts; pack $top.f1 -fill x
   tixLabelEntry $top.f2 -label "Distance" -options $opts; pack $top.f2 -fill x
}


# Refreshes the gem-layout page
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_gem {wid} {
    set top [$wid subwidget gem] 
    append top ".top"
}


# pr_gui:layouts:make_grid {parent}: constructs the page for the DOT layout-engine
#--------------------------------------------------------------------------------
proc pr_gui:layouts:make_grid {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 15 label.anchor w }

   label         $top.l       -text  "Grid Layout Parameters" -justify center; pack $top.l
   tixLabelEntry $top.spacing -label "Spacing"     -options $opts; pack $top.spacing -fill x
   tixSelect     $top.axis    -label "Major axis"  -options $opts \
		 -allowzero false -radio true; pack $top.axis -fill x
   $top.axis add btn1 -text "X"
   $top.axis add btn2 -text "Y"
}


# Refreshes the grid-layout page
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_grid {wid} {
    set top [$wid subwidget grid] 
    append top ".top"

    set sp [$top.spacing subwidget entry]
    $sp  delete 0 [string length [$sp get]]
    $sp  insert 0 [pr_lay_grid -get grid]

    set ax [pr_lay_grid -get axis]
    if {$ax=="x"} { $top.axis configure -value {btn1} }
    if {$ax=="y"} { $top.axis configure -value {btn2} }
}

# Copies params from the 'grid' page to the kernel via the Tcl API
#------------------------------------------------------------------------------
proc pr_gui:layouts:ok_grid {wid} {
    set top [$wid subwidget grid] 
    append top ".top"

    set sp [[$top.spacing subwidget entry] get]
    set v  [$top.axis cget -value]
    if [string equal $v btn1] { set ax x }
    if [string equal $v btn2] { set ax y } 

    pr_lay_grid -set grid $sp axis $ax
}



# pr_gui:layouts:make_nested {parent}: constructs the page for the nested layout-engine
#----------------------------------------------------------------------------------------
proc pr_gui:layouts:make_nested {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 15 label.anchor w }
   label         $top.l       -text  "Nested Layout Parameters" -justify center; pack $top.l
   tixLabelEntry $top.layout  -label "Basic layout"        -options $opts; pack $top.layout  -fill x
   tixLabelEntry $top.xybord  -label "Nesting border (xy)" -options $opts; pack $top.xybord  -fill x

   tixSelect     $top.zlayout -label "Set Z coordinate"    -options $opts -allowzero true; pack $top.zlayout -fill x
   $top.zlayout add btn
   [$top.zlayout subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
}


# Refreshes the nested-layout page
#
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_nested {wid} {
    set top [$wid subwidget nested] 
    append top ".top"

    set xy [$top.xybord subwidget entry]
    $xy  delete 0 [string length [$xy get]]
    $xy  insert 0 [pr_lay_nested -get xyborder]

    set v [$top.zlayout subwidget btn]
    if {[pr_lay_nested -get zlayout]==1} { $top.zlayout config -value {btn} } else { $top.zlayout config -value {} }

    #!!! Add the other fields of the nested layout too
}


# Copies params from the 'nested' page to the kernel via the Tcl API.
#
#------------------------------------------------------------------------------
proc pr_gui:layouts:ok_nested {wid} {
    set top [$wid subwidget nested] 
    append top ".top"

    set la [[$top.layout  subwidget entry] get]
    set xy [[$top.xybord  subwidget entry] get]
    set zl [expr [string  compare [$top.zlayout cget -value] btn] + 1]

    pr_lay_nested -set xyborder $xy zlayout $zl
    
    #!!! pr_lay_nested --- add the other options please 
}



# pr_gui:layouts:make_level {parent}: constructs the page for the level-layout-engine
#--------------------------------------------------------------------------------
proc pr_gui:layouts:make_level {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x
   set opts { label.width 15 label.anchor w }

   label         $top.l       -text  "Level Layout Parameters" -justify center; pack $top.l
   tixLabelEntry $top.level   -label "Level attribute"  -options $opts; pack $top.level   -fill x
   tixLabelEntry $top.spacing -label "Level spacing"    -options $opts; pack $top.spacing -fill x
   tixSelect     $top.method  -label "Spacing method"   -options $opts \
		 -allowzero false -radio true; pack $top.method -fill x
   $top.method add btn1 -text "Absolute"
   $top.method add btn2 -text "Normalized"
}


# Refreshes the level-layout page
#-------------------------------------------------------------------------------
proc pr_gui:layouts:refresh_level {wid} {
    set top [$wid subwidget level] 
    append top ".top"

    set v [$top.level subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_level -get level]

    set v [$top.spacing subwidget entry]
    $v  delete 0 [string length [$v get]]
    $v  insert 0 [pr_lay_level -get spacing]

    set v [pr_lay_level -get spacing_method]
    if {$v==0} { $top.method configure -value {btn1} }
    if {$v==1} { $top.method configure -value {btn2} }
}


# Copies params from the 'level' page to the kernel via the Tcl API.
#------------------------------------------------------------------------------
proc pr_gui:layouts:ok_level {wid} {
    set top [$wid subwidget level] 
    append top ".top"

    set lv [[$top.level   subwidget entry] get]
    set sp [[$top.spacing subwidget entry] get]
    set v  [$top.method cget -value]
    if [string equal $v btn1] { set mt 0 }
    if [string equal $v btn2] { set mt 1 } 
   
    pr_lay_level -set level $lv spacing $sp spacing_method $mt
}



# pr_gui:layouts:ok{parent}: sets all values from the UI's fields to the system via the Tcl API
#---------------------------------------------------------------
proc pr_gui:layouts:ok {wid} {

# get current notebook-panel and apply it...

  set page [$wid.nb raised]
  if {$page=="grid"}   { pr_gui:layouts:ok_grid   $wid.nb }
  if {$page=="neato"}  { pr_gui:layouts:ok_neato  $wid.nb }
  if {$page=="dot"}    { pr_gui:layouts:ok_dot    $wid.nb }
  if {$page=="nested"} { pr_gui:layouts:ok_nested $wid.nb }
  if {$page=="level"}  { pr_gui:layouts:ok_level  $wid.nb }
  if {$page=="genrl"}  { pr_gui:layouts:ok_genrl  $wid.nb }
}

