#---------------------------------------------------------------------------------
# GUI for visualising the selection objects
#
#---------------------------------------------------------------------------------



# pr_gui:sel:  Opens the selection GUI.
#
#

proc pr_gui:sel {wid} {
   if {[info commands .$wid]!={}} {
      raise .$wid
      return
   }

   toplevel .$wid
   pr_list:build .$wid {Name Nodes Edges} { {Ok "pr_del_observer %W; destroy %W"} \
                       {Refresh "pr_gui:sel_refresh %W"} } "pr_gui:browsecmd .$wid"
   bind .$wid <Destroy> { pr_del_observer %W; destroy %W }

   frame .$wid.ctrls -relief raised -bd 1
   frame .$wid.ctrls.f1
   tixControl .$wid.ctrls.f1.niter -label "Nodes" -integer true -min 0 -option { entry.width 5 } -validatecmd "pr_gui:iter:validatecmd .$wid nnodes"
   tixControl .$wid.ctrls.f1.eiter -label "Edges" -integer true -min 0 -option { entry.width 5 } -validatecmd "pr_gui:iter:validatecmd .$wid nedges"
   pack .$wid.ctrls.f1.niter -side left 
   pack .$wid.ctrls.f1.eiter -after .$wid.ctrls.f1.niter -side left -anchor w
   frame .$wid.ctrls.f2
   button .$wid.ctrls.f2.reset  -text "Clear"  -command "pr_gui:sel_reset  .$wid"
   button .$wid.ctrls.f2.delete -text "Delete" -command "pr_gui:sel_delete .$wid"
   button .$wid.ctrls.f2.rename -text "Rename" -command "pr_gui:sel_rename .$wid"
   button .$wid.ctrls.f2.new    -text "New"
   pack .$wid.ctrls.f2.reset .$wid.ctrls.f2.delete .$wid.ctrls.f2.rename .$wid.ctrls.f2.new -side left -anchor w 
   pack .$wid.ctrls.f1
   pack .$wid.ctrls.f2
   pack .$wid.ctrls

  

   pr_add_observer .$wid [concat pr_gui:sel_refresh .$wid]
   pr_gui:sel_refresh .$wid
}




##------ PRIVATE -------------------------------------------------

proc pr_gui:sel_reset {w} {
  set sidx [[$w.top.a subwidget hlist] info selection]
  set sel  [pr_get_selection idx $sidx]
  pr_selection $sel reset
}

proc pr_gui:sel_delete {w} {
  set sidx [[$w.top.a subwidget hlist] info selection]
  set sel  [pr_get_selection idx $sidx]
  pr_selection $sel delete
}

proc pr_gui:sel_rename {w} {
  set sidx [[$w.top.a subwidget hlist] info selection]
  set sel  [pr_get_selection idx $sidx]
  set cmd  "pr_gui:sel_rename_cb $sel .dialog; destroy %W"
 

  pr_dialog:text_input .dialog "Rename selection to" validatecmd \
                     [list [list Ok $cmd] [list Cancel "destroy %W"]]
  #pr_selection $sel delete
}

proc pr_gui:sel_rename_cb {sel dialog} {
   set newname [pr_dialog:text_get $dialog]
   pr_selection $sel rename $newname
}

proc pr_gui:sel_refresh {wid} {

   set i 0
   set n [pr_num_selections]
   set l {}
   
   while {$i<$n} { 

      set sel    [pr_get_selection idx $i]
      set sname  [pr_selection $sel name]
      if [expr !([string equal $sname ""])] {
         lappend l  [list $sname [pr_selection $sel nnodes] [pr_selection $sel nedges] 3 ]
      }
      incr i
   }
   
   pr_list:browse $wid $l

   #Reset iterators to 0
   $wid.ctrls.f1.niter configure -value 0
   $wid.ctrls.f1.eiter configure -value 0
}


#
# pr_gui:browsecmd: Browse callback, called when someone changes the selected entry in the list
#
proc pr_gui:browsecmd {w idx} {

  #Reset iterators to 0
  $w.ctrls.f1.niter configure -value 0
  $w.ctrls.f1.eiter configure -value 0
}

#
# pr_gui:iter:validatecmd: Validates the node and edge iterators to index only existing selection-elements
#
proc pr_gui:iter:validatecmd {w which_iter value} {
  set sidx [[$w.top.a subwidget hlist] info selection]
  set sel  [pr_get_selection idx $sidx]
  set num  [pr_selection $sel $which_iter] 

  # Enable or disable iterator depending on the max #elems in it being 0 or >0
  if [string equal $which_iter nnodes] {
     if {$num==0} {
        $w.ctrls.f1.niter configure -state disabled
     } else {
        $w.ctrls.f1.niter configure -state normal
     }
  } else {
    if {$num==0} {
        $w.ctrls.f1.eiter configure -state disabled
     } else {
        $w.ctrls.f1.eiter configure -state normal
     }
  }

  #Clamp user input value to [0..max #elems in selection]
  if [expr $num==0 || $value<0] { return 0 }
  if {$value>=$num} { return [expr $num-1] } 
  return $value
}


proc pr_list:build {w hdrs btns browsecmd} {

    frame $w.top -relief raised -bd 1

    set sz [llength $hdrs]

    tixScrolledHList $w.top.a -options [list hlist.columns [expr $sz+1] hlist.header true \
                      hlist.selectBackground black hlist.selectForeground white]

    #Connect user browse-callback to widget - the cb will be called with an extra 'index' argument
    [$w.top.a subwidget hlist] config -browsecmd $browsecmd
                     
    pack $w.top.a -expand yes -fill both -side left
 
    set hlist [$w.top.a subwidget hlist]

    # Create the title for the HList widget. We have set the hlist.header subwidget option to true
    #      so that the header is displayed

    # Styles for the headers
    set sty_hdr [tixDisplayStyle text  -refwindow $hlist \
	-fg black -anchor c -padx 8 -pady 2\
	-font [tix option get bold_font ]]

    # Create the headers as the 2nd arg specifies
    set idx 0 
    foreach item $hdrs  {
        $hlist header create $idx -itemtype text -text $item -style $sty_hdr
        incr idx
    }

    # We use sz columns in the hlist widget. This way when the user
    # expands the windows wide, the right side of the header doesn't look
    # chopped off. The following line ensures that the sz-th column header is
    # not shown unless the hlist window is wider than its contents.
    $hlist column width $sz 0

    if {$btns=={}} {return}

    #Build the buttons of the window
    #
    tixButtonBox $w.box -orientation horizontal -padx 1
    set idx 0
    foreach btn $btns {
        set name [lindex $btn 0]
        set cmd  [lindex $btn 1]
        while {1} {
           set pos  [string first "%W" $cmd]
	   if {$pos!=-1}  {set cmd [string replace $cmd $pos [expr $pos+1] $w]} else break
        }	
        $w.box add $idx -text $name -command $cmd -width 7
        incr idx
    }

    pack $w.box -side bottom -fill x
    pack $w.top -side top -fill both -expand yes
}



proc pr_list:browse {w data} {

    set hlist [$w.top.a subwidget hlist]

    $hlist delete all

    set idx 0
    foreach line $data {
	$hlist add $idx -itemtype text -text [lindex $line 0]
        set idx2 1
        set sz   [llength $line]
        while {$idx2 < $sz}  {
	   $hlist item create $idx $idx2 -itemtype text -text [lindex $line $idx2] 
           incr idx2
        }
	incr idx
    }

    #Set the selected entry to be the first entry in the list, if the list is not empty
    if {$idx>0} { $hlist selection set 0 }
}


