#  This file contains the initialization of the registry database with various system-wide
#  information. Add here whatever (key,value) pairs you want to make available to the system.
#
#-----------------------------------------------------------------------------------------------


#  KEY 'generic_layout'
#-----------------------------------------------------------------------------------------------
#  Add the several existing layouts in the system to the registry under the
#  name 'layouts'. For every layout, we add a text label and the Tcl command doing the layout.
#  This layout database will then be available to anyone in the system needing to know which
#  are the generic layouts available.
#
#  WARNING:
#  It is the responsibility of the one constructing the registry to add the correct information.
#  For the 'generic_layout' key, ONLY layouts applicable to ANY selection are allowed.
#-----------------------------------------------------------------------------------------------
pr_register add generic_layout "Grid layout"             pr_lay_grid
pr_register add generic_layout "Tree layout (DOT)"       pr_lay_dot
pr_register add generic_layout "Spring embedder (NEATO)" pr_lay_neato
pr_register add generic_layout "Spring embedder  (GEM)"  pr_lay_gem



#  KEYS 'node_glyph_factory' and 'edge_glyph_factory'
#-----------------------------------------------------------------------------------------------
#  Add the several existing node/edge glyph-factory types (not instances ) in the system.
#  This info is used to automatically construct the glyph-GUIs (see glyphs.tcl).
#  For every glyph-factory type, 4 entries are needed:
#  - label: short text used as a tab-panel label
#  - title: long text used as a tab-panel title
#  - type:  name of the C++ type of the glyph-factory
#  - id:    unique id used as suffix for GUI procedure names. Should be a valid Tk-window id name. 
#           So far we need 3 procedures: make_id,select_id,ok_id. See glyphs.tcl.
#
#  WARNING:
#  For every entry, several GUI procedures MUST be written in glyphs.tcl. See glyphs.tcl.
#-----------------------------------------------------------------------------------------------


pr_register add node_glyph_factory "Tcl"	 "Tcl node glyph engine"      IVTclNodeGlyphFactory		ntcl	
pr_register add node_glyph_factory "Basic" "Basic node glyph engine"	IVBasicNodeGlyphFactory		nbasic	
pr_register add edge_glyph_factory "Tcl"	 "Tcl edge glyph engine"	IVTclEdgeGlyphFactory		etcl
pr_register add edge_glyph_factory "Basic" "Basic edge glyph engine"	IVBasicEdgeGlyphFactory		ebasic

#  KEY 'save_format'
#-----------------------------------------------------------------------------------------------
#  Add the several available save-formats here to get them into the 'Save' GUI (see save.tcl)
#  For every format, 3 entries are expected:
#  - name:  Textual description of that format as it will appear in the GUI
#  - ext:   File extension for that format (e.g. ".rsf" or ".dat")
#  - proc:  Tcl proc to be called to save the data in the given format. This proc should be callable as:
#           "proc_name sel file" where 'sel' is a selection-name to be saved and 'file' the file-name 
#           to save to.
#

pr_register add save_format "Own format"       ".dat"   pr_write_own_format
pr_register add save_format "GraphViz (.dot)"  ".dot"   pr_write_gv
pr_register add save_format "GraphEd (.g)"     ".g"     pr_write_graphed
pr_register add save_format "Rigi (.rsf)"      ".rsf"   pr_write_rsf
pr_register add save_format "Chaco (.graph)"   ".graph" pr_write_chaco


#  KEYS 'viewer_cmd', 'viewer_activate', 'viewer_sweep_cb'
#-----------------------------------------------------------------------------------------------
#  Add the custom viewer-GUI buttons and Tcl-commands to the 'viewer_cmd' key. This allows customizing the 
#  apperarance/functionality of the Inventor viewers. Every button takes 3 entries:
#  - the name of a bitmap-file to be used as button icon. The name should be supplied without extension,
#    the latter being automatically added depending on the platform. For MS-Windows, a '.bmp' file is
#    sought. 
#  - a Tcl command to be executed for that button. The Tcl command gets as extra parameter
#    the name of the viewer the invoking button belongs to.
#  - a short tooltip (balloon) message. This item is optional
#  The key 'viewer_activate' takes one parameter, the name of a Tcl proc to invoke when an
#  Inventor viewer is click-activated (i.e. receives focus). This proc is currently used to refresh
#  the Viewers GUI to show the parameters of the active viewer. 
#
#  The key 'viewer_sweep_cb' takes one parameter, the name of a Tcl proc to invoke as default sweep-callback.
#  Viewers can be customized each with separate sweep-callbacks, but this is the one set by default.


pr_register add viewer_cmd  "RSCS/vw_small"	"pr_viewer_cmd:show_params"  	"Viewers panel"
pr_register add viewer_cmd  "RSCS/lt_small"	"pr_viewer_cmd:pan_left"	"Pan view left"	
pr_register add viewer_cmd  "RSCS/rt_small"	"pr_viewer_cmd:pan_right"	"Pan view right"
pr_register add viewer_cmd  "RSCS/up_small"	"pr_viewer_cmd:pan_up"		"Pan view up"
pr_register add viewer_cmd  "RSCS/dn_small"	"pr_viewer_cmd:pan_down"	"Pan view down"
pr_register add viewer_cmd  "RSCS/del_small"	"pr_viewer_cmd:delete"		"Destroy highlighted items"
pr_register add viewer_cmd  "RSCS/rm_small"	"pr_viewer_cmd:remove"		"Remove highlighted items from view"
pr_register add viewer_cmd  "RSCS/node_small"	"pr_viewer_cmd:node_details"	"Show details of highlighted item"
pr_register add viewer_cmd  "RSCS/sweep_small"	"pr_viewer_cmd:sweep"		"Show details under mouse"   check
pr_register add viewer_cmd  "RSCS/logo_small"	"wm deiconify .; raise ."	"Main window"
pr_register add viewer_activate			"pr_viewer_cmd:refresh_params" 
pr_register add viewer_sweep_cb pr_viewer_cmd:sweep_cb

#  Since some of the button commands are actually very short Tcl-procs, we declare them right here.
#  This is not exactly correct, as this file is supposed to contain only 'pr_register' calls. However,
#  so far we don't have a better place (file) for these commands to go, so we put them here.

proc pr_viewer_cmd:pan_left  {vw} { pr_view $vw pan left }
proc pr_viewer_cmd:pan_right {vw} { pr_view $vw pan right }
proc pr_viewer_cmd:pan_up    {vw} { pr_view $vw pan up }
proc pr_viewer_cmd:pan_down  {vw} { pr_view $vw pan down }

proc pr_viewer_cmd:delete    {vw} {
  set hl_sel [pr_view $vw highlight]
  if {$hl_sel==""} { return }
  pr_selection [pr_get_selection name $hl_sel] flush
}

proc pr_viewer_cmd:remove    {vw} {
  set hl_sel [pr_view $vw highlight]
  if {$hl_sel==""} { return }
  set sel [pr_view $vw selection]
  set temp [pr_selection [pr_sel_new] name]
  pr_selection_op difference $sel $hl_sel $temp
  pr_selection [pr_get_selection name $temp] rename $sel
}

proc pr_viewer_cmd:node_details {vw} {
  set hl_sel [pr_view $vw highlight]
  if {$hl_sel==""} { return }
  set hls [pr_get_selection name $hl_sel]
  if {[pr_selection $hls nnodes]!=1} { return }
  set i [pr_selection $hls iter_nodes]
  set n [pr_iterator $i next_node]  
  pr_gui:attrset .nodes
  pr_gui:attrset_refresh_id .nodes [pr_node $n id] 
}

proc pr_viewer_cmd:sweep {vw} {
   #Enable/disable the sweep-cb of viewer $vw
   #
   if {[pr_view $vw enable_sweep_cb]=="1"} { 
     pr_view $vw set enable_sweep_cb 0 
   } else {
      pr_view $vw set enable_sweep_cb 1
   }
}

proc pr_viewer_cmd:show_params    {vw} { pr_gui:viewers:refresh_view viewers $vw true }
proc pr_viewer_cmd:refresh_params {vw} { pr_gui:viewers:refresh_view viewers $vw false }


