#
# This file contains the Tk GUIs for saving the graph data to an external file.
#--------------------------------------------------------------------------------


proc pr_gui:save {wid} {
   if {[info commands .$wid]!={}} {
      raise .$wid
      return
   }
   toplevel .$wid

   frame .$wid.top; set top .$wid.top; pack $top -fill x
   set opts { label.width 20 label.anchor w }
   label         $top.lbl -text "Save data to file" -justify center; pack $top.lbl -fill x 

   tixLabelEntry $top.sel       -label "Selection" -options $opts; pack $top.sel -fill x
   frame         $top.fl
   tixLabelEntry $top.fl.entry  -label "File name" -options $opts; pack $top.fl.entry -fill x -side left -expand yes
   button        $top.fl.dlg    -text  "Edit" -image [image create photo -file [pr_qualify RSCS/openmin.gif]] \
		     -command "pr_gui:save:dialog .$wid"; pack $top.fl.dlg -side right
   pack          $top.fl -fill x	
   tixComboBox   $top.formats   -label "File format" -options $opts -dropdown true -editable false\
                 -command "pr_gui:save:sel_format .$wid"; pack $top.formats -fill x
   tixSelect     $top.ndattrs   -label "Save node attributes" -options $opts -allowzero true; \
		     pack $top.ndattrs -fill x
   $top.ndattrs add btn
   [$top.ndattrs subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
   tixSelect     $top.edattrs   -label "Save edge attributes" -options $opts -allowzero true; \
		     pack $top.edattrs -fill x
   $top.edattrs add btn
   [$top.edattrs subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]

   # Initialize the state of the dialog:
   # Add a few formats that we currently support
   #
   set l [pr_register get save_format]; set i 0; set n [llength $l]
   while {$i<$n} {
      set name   [lindex [lindex $l $i] 0]
      $top.formats insert end $name
      incr i
   }
   $top.formats pick 0
   
   # Set the selection to 'All' (by default, we save everything)
   #
   [$top.sel subwidget entry] insert 0 "All"

   tixButtonBox .$wid.box -orientation horizontal
   .$wid.box add ss  -text "Save session" -command "pr_gui:save:session .$wid"
   .$wid.box add ok  -text "Save data"    -command "pr_gui:save:ok .$wid"
   .$wid.box add cc  -text "Cancel"       -command "destroy .$wid"
   pack .$wid.box -side bottom -fill x
   bind .$wid <Destroy> { destroy %W }
}


#-------------------------------------------------------------------------------
# Private (implementation) part 
#-------------------------------------------------------------------------------

# pr_gui:save:dialog: cb for the browsing for file-names button. Simply selects a filename.
#
proc pr_gui:save:dialog {w} {

   set types { 
		   {{All files}          *  }  
               {{Own format files} {.dat}} 
             }
   set v  [$w.top.fl.entry subwidget entry]
   set sel [tk_getSaveFile -parent $w -filetypes $types -initialfile [$v get]]

   if [llength $sel] {
       set v [$w.top.fl.entry subwidget entry]
       $v  delete 0 [string length [$v get]]
       $v  insert 0 $sel
   }
}

#
#
proc pr_gui:save:sel_format {w sel} {
   #ops to be done on format selection  
}   

# pr_gui:save:ok{w}	Saves graph data given the params (format, file name) in the dialog-box $w
#
proc pr_gui:save:ok {w} {
   set fname    [[$w.top.fl.entry subwidget entry] get]
   set fmt_name [[$w.top.formats subwidget listbox] get active]

   #If no filename given, simply do nothing (but also don't exit the dialog)
   if {$fname=={}} {return}

   #If no selection name or name of inexistent selection given, nothing to do
   set sel_name [[$w.top.sel subwidget entry] get]
   if {$sel_name=={}} {return}
   set sel [pr_get_selection name $sel_name]
   if {$sel==[pr_get_selection name None]} {return}
   
   #Find the Tcl-proc to call for the currently selected format
   set l [pr_register get save_format]; set i 0; set n [llength $l]
   while {$i<$n} {
      set e      [lindex $l $i]
      set name   [lindex $e 0]
      set ext    [lindex $e 1]
      set cb     [lindex $e 2]
      if {$fmt_name==$name} {break}
      incr i
   }
   
   #Save the file & exit dialog
   $cb $sel_name $fname
   destroy $w
}


# pr_gui:save:session{w}  Saves session given the params (format, file name) in the dialog-box $w
#
proc pr_gui:save:session {w} {
   set fname    [[$w.top.fl.entry subwidget entry] get]
   
   #If no filename given, simply do nothing (but also don't exit the dialog)
   if {$fname=={}} {return}

   #Save the file & exit dialog
   pr_write_session $fname
   destroy $w
}

