#  Procedures for managing the Viewers GUI component
# 
#
#--------------------------------------------------------------------


#  pr_gui:view {wid}: main entry point
#
#
proc pr_gui:view {wid} {
   if {[info commands .$wid]!={}} { raise .$wid; return }
   toplevel .$wid

   # Create a global variable used to block/unblock the menu-callbacks. This is needed for setting the radio-entries
   # without calling their associated callbacks. A value of 1 means callbacks are blocked, 0 means callbacks are unblocked.
   #
   global pr_gui_view_cb_block_flag
   set pr_gui_view_cb_block_flag 0

   pr_gui:viewers:make_menu .$wid
   pr_gui:viewers:make .$wid

   tixSelect .$wid.box 
   set box .$wid.box
   $box add ok  -text "Apply"   -command "pr_gui:viewers:apply .$wid"
   $box add new -text "New"     -command "pr_view_new \"new_viewer\" None"
   $box add del -text "Delete"  -command "pr_gui:viewers:del .$wid"
   $box add upd -text "Update"  -command "pr_gui:viewers:update .$wid"
   $box add cc  -text "Dismiss" -command "pr_del_observer .$wid; destroy .$wid"
   tixBalloon $box.ok_b -initwait 10
   $box.ok_b bind [$box subwidget ok] -msg "Apply current params to current viewer"
   tixBalloon $box.new_b -initwait 10
   $box.new_b bind [$box subwidget new] -msg "Create new viewer"
   tixBalloon $box.del_b -initwait 10
   $box.del_b bind [$box subwidget del] -msg "Delete current viewer"
   tixBalloon $box.upd_b -initwait 10
   $box.upd_b bind [$box subwidget upd] -msg "Update image in current viewer"

   pack $box -side bottom
   bind .$wid <Destroy> { pr_del_observer %W; destroy %W }

   pr_add_observer .$wid [concat pr_gui:viewers:refresh .$wid]
   pr_gui:viewers:refresh .$wid
}




#  pr_gui:viewers:refresh_view : refreshes the GUI to show the details of the viewer 'vw'.
# 					   If last argument is true, the GUI is also popped up,
#					   otherwise it is refreshed only if it is already popped up.
#------------------------------------------------------------------------------------------

proc pr_gui:viewers:refresh_view {wid vw popup} {
   if {$popup=="true"} { pr_gui:view $wid }
   if {[info commands .$wid]=={}} { return }

   set i 0; set n [pr_num_views]
   while {$i<$n} {
      if {[pr_get_view $i]==$vw} break
      incr i
   }   
   if {$i==$n} return
   .$wid.top.fr.vw pick $i
   pr_gui:viewers:refresh2 .$wid $vw
}

#--------------------------------------------------------------------------
# Private (implementation) part 
#-------------------------------------------------------------------------------


# pr_gui:viewers:make_menu: Construct the GUI's menu
#----------------------------------------------------------------------------

proc pr_gui:viewers:make_menu {wid} {
   menu $wid.menu -tearoff 0
   set m   $wid.menu.layout;  set m2  $wid.menu.highlight
   set m21 $m2.dragstyle;     set m3  $wid.menu.mapper
   set m31 $m3.node_mapper;   set m32 $m3.edge_mapper
   set m33 $m3.edge_drawing;  set m34 $m3.bgnd_mappers 
   set m4  $wid.menu.options
   menu $m   -tearoff 0; menu $m2  -tearoff 0; menu $m21 -tearoff 0
   menu $m3  -tearoff 0; menu $m31 -tearoff 0; menu $m32 -tearoff 0
   menu $m33 -tearoff 0; menu $m34 -tearoff 0; menu $m4  -tearoff 0

   # The menu 'Layouts' adds all layouts registered under the 'generic_layout' key
   # so one can apply them on the current (viewer,selection,pos-attr) combination.
   #
   $wid.menu add cascade -label "Layouts" -menu $m -underline 0
   set lays [pr_register get generic_layout]
   set i 0
   set n [llength $lays]
   while {$i<$n} {
      set layout [lindex $lays $i]
      set label  [lindex $layout 0]
      set cmd    [lindex $layout 1] 
      $m add command -label $label -command [list pr_gui:viewers:menu_layout_cb $wid $cmd]
      incr i
   }
   
   # The menu 'Highlight' provides options controlling the manipulation of the highlight-selection
   # in the viewer.
   #
   $wid.menu add cascade -label "Highlight" -menu $m2 -underline 0
   $m2 add cascade -label "Dragging style"  -menu $m21 -underline 0
   $m21 add radio -label "Low resolution (fast)"  -underline 0 -command [list pr_gui:viewers:menu_style_cb $wid low]
   $m21 add radio -label "Medium resolution"	  -underline 0 -command [list pr_gui:viewers:menu_style_cb $wid mid] 
   $m21 add radio -label "High resolution (slow)" -underline 0 -command [list pr_gui:viewers:menu_style_cb $wid high]
   $m2 add command -label "Color..." -command [list pr_gui:viewers:menu_color_cb $wid]
   
   # The menu 'Mapper' provides the options of the current IVMapper of the current viewer
   # When new mappers are made available to the viewer, this menu will have to be extended...
   #
   $wid.menu add cascade -label "Mapper"	    -menu $m3 -underline 0
   $m3 add cascade -label "Node mapper"		    -menu $m31 -underline 0
   $m3 add cascade -label "Edge mapper"		    -menu $m32 -underline 0
   $m3 add cascade -label "Edge drawing"            -menu $m33 -underline 5
   $m3 add cascade -label "Background node mappers" -menu $m34 -underline 0
   $m31 add radio -label "Basic (boxes)" -underline 0 -command [list pr_gui:viewers:menu_node_mapper_cb $wid basic]
   $m31 add radio -label "Tcl Glyphs"    -underline 0 -command [list pr_gui:viewers:menu_node_mapper_cb $wid factory]
   $m31 add radio -label "Splat"         -underline 0 -command [list pr_gui:viewers:menu_node_mapper_cb $wid splat] 
   $m32 add radio -label "Basic (lines)" -underline 0 -command [list pr_gui:viewers:menu_edge_mapper_cb $wid basic]
   $m32 add radio -label "Factory"       -underline 0 -command [list pr_gui:viewers:menu_edge_mapper_cb $wid factory]
   $m33 add radio -label "Draw line between node centers" -command [list pr_gui:viewers:menu_edge_drawing_cb $wid node_centers]
   $m33 add radio -label "Draw line between nodes"        -command [list pr_gui:viewers:menu_edge_drawing_cb $wid straight]
   $m33 add radio -label "Draw polyline (default)"        -command [list pr_gui:viewers:menu_edge_drawing_cb $wid default]
   $m33 add radio -label "Draw smooth curve"              -command [list pr_gui:viewers:menu_edge_drawing_cb $wid smooth]
   $m34 add checkbutton -label "Basic (boxes)" -command [list pr_gui:viewers:menu_bgnd_node_cb $wid basic]
   $m34 add checkbutton -label "Tcl Glyphs"    -command [list pr_gui:viewers:menu_bgnd_node_cb $wid factory]
   $m34 add checkbutton -label "Splat"         -command [list pr_gui:viewers:menu_bgnd_node_cb $wid splat]

   # The menu 'Options' provides various other options that dont' fit into the other menus
   #
   $wid.menu add cascade -label "Options" -menu $m4 -underline 0
   $m4 add command -label "Background color..." -command [list pr_gui:viewers:menu_bkcol_cb $wid]
   $m4 add checkbutton   -label "2D rendering"  -command [list pr_gui:viewers:menu_2d_cb $wid]
			
   $wid configure -menu $wid.menu 
}

# Callback for the menu Highlight
#
proc pr_gui:viewers:menu_style_cb {wid style} {
  # Check whether callbacks are blocked. If so, don't do anything.
  #
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return

  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active]    

  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { pr_view $vw set move_style $style }
}

#  Callback for the menu Highlight.Color
#
proc pr_gui:viewers:menu_color_cb {wid} {
  set viewers [$wid.top.fr.vw subwidget listbox]  
  
  #Get the selected viewer
  set vw [$viewers get active] 

  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { 
     set col [tk_chooseColor -parent $wid -initialcolor [pr_view $vw highlight_col] ]		
     if {$col!=""} { pr_view $vw set highlight_col $col }
  }
}




#  Callback for the menu Options.Background Color
#
proc pr_gui:viewers:menu_bkcol_cb {wid} {
  #Get the selected viewer
  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active]  
  
  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { 
     set col [tk_chooseColor -parent $wid -initialcolor [pr_view $vw background_col] ]		
     if {$col!=""} { pr_view $vw set background_col $col }
  }
}

proc pr_gui:viewers:menu_2d_cb {wid} {
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return

  #Get the selected viewer
  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active]  
  
  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { 
     if {[pr_view $vw render_2d]=="1"} { 
       pr_view $vw set render_2d 0
     } else { pr_view $vw set render_2d 1}
  }
}


# Callback for the menu Mapper.Node mapper
#
proc pr_gui:viewers:menu_node_mapper_cb {wid type} {
  # Check whether callbacks are blocked. If so, don't do anything.
  #
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return

  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active] 

  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { pr_view $vw set node_mapper $type }
}

# Callback for the menu Mapper.Edge mapper
#
proc pr_gui:viewers:menu_edge_mapper_cb {wid type} {
  # Check whether callbacks are blocked. If so, don't do anything.
  #
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return

  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active] 

  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { pr_view $vw set edge_mapper $type }
}

# Callback for the menu Mapper.Edge Drawing
#
proc pr_gui:viewers:menu_edge_drawing_cb {wid type} {
  # Check whether callbacks are blocked. If so, don't do anything.
  #
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return

  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active] 

  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { pr_view $vw set edge_res $type }
}

#Callback for the menus Mapper.Background node mappers.*
#
proc pr_gui:viewers:menu_bgnd_node_cb {wid map_type} {
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return

  #Get the selected viewer
  set viewers [$wid.top.fr.vw subwidget listbox]; set vw [$viewers get active]  
  
  #Check we have any viewer selected, otherwise nothing to do
  if {$vw!={}} { 
     if {[pr_view $vw aux_node_mapper $map_type]=="1"} { 
       pr_view $vw set aux_node_mapper $map_type 0
     } else { pr_view $vw set aux_node_mapper $map_type 1}
  }
}




# Callback for the menu Layout
#
proc pr_gui:viewers:menu_layout_cb {wid cmd} {
  set viewers [$wid.top.fr.vw subwidget listbox]  
  
  #Get the selected viewer
  set vw [$viewers get active] 
  
  #Check we have any viewer selected, otherwise nothing to do
  #
  if {$vw!={}} {
    set sel [pr_view $vw selection]
    set pos [pr_view $vw position]
    #Translate empty sel-name to None
    #
    if {$sel=={}} {set sel None}	  
    #Execute the layout command $cmd with arguments $sel and $pos
    #  
    if {$pos!={}} {$cmd $sel $pos}      
  }
}


# Constructs the main part of the GUI
#
#--------------------------------------------------------------------------------
proc pr_gui:viewers:make {wid} {

   frame $wid.top
   set top $wid.top
   pack $top -fill x -expand yes -fill both
   set opts { label.width 20 label.anchor w }

   label         $top.l      -text  "Viewing Parameters" -justify center; pack $top.l
   frame	     $top.fr     -border 1 -relief raised;		      pack $top.fr     -fill x
   tixComboBox   $top.fr.vw  -label "Viewers"           -options $opts -dropdown true -editable false\
                             -command "pr_gui:viewers:refresh2 $wid"; pack $top.fr.vw     -fill x
   tixLabelEntry $top.sel    -label "Selection"         -options $opts; pack $top.sel    -fill x
   tixLabelEntry $top.hlt    -label "Highlight"         -options $opts; pack $top.hlt    -fill x
   tixLabelEntry $top.pos    -label "Position"          -options $opts; pack $top.pos    -fill x
   tixSelect     $top.out    -label "Output only"       -options $opts -allowzero true; pack $top.out -fill x
                             $top.out add btn; [$top.out subwidget btn] configure -image [image create photo -file [pr_qualify RSCS/tick.gif]]
   pr_gui:cbox_edit  $top.ngly "Node glyph engine" $opts [pr_qualify RSCS/glyph_small.gif] "pr_gui:viewers:go_ngly" \
   			             "pr_gui:viewers:set_glyph_engine $wid nodes" "Examine node glyph engines"
   pr_gui:cbox_edit  $top.egly "Edge glyph engine" $opts [pr_qualify RSCS/glyph_small.gif] "pr_gui:viewers:go_egly" \
   					 "pr_gui:viewers:set_glyph_engine $wid edges" "Examine edge glyph engines"
   pr_gui:combo_edit $top.ucb  "User Tcl action"   $opts [pr_qualify RSCS/edit.gif] "pr_gui:viewers:edit_ucb"  "Edit user action Tcl script"
   pr_gui:combo_edit $top.swcb "Sweep Tcl action"  $opts [pr_qualify RSCS/edit.gif] "pr_gui:viewers:edit_swcb" "Edit sweep action Tcl script"
   scale         $top.scale  -label "Node scale"        -from 0 -to 1.0 -resolution 0.001    \
            	           -orient horizontal -command "pr_gui:viewers:scale $wid.top"; pack $top.scale -fill x;
}



# Refreshes the GUI completely from the core data, via the Tcl API.
# The selected-viewer will be the same as before, if that viewer exists, else the first one in the viewers' list.
#-----------------------------------------------------------------------------------------------------------------------
proc pr_gui:viewers:refresh {wid} {
   set top $wid.top 
   set active_vw [[$wid.top.fr.vw subwidget listbox] get active]    
  
   [$top.fr.vw subwidget listbox] delete 0 end
   set i 0; set j 0; set n [pr_num_views]
   while {$i<$n} { 
      $top.fr.vw insert end [pr_get_view $i]
      if {[pr_get_view $i]==$active_vw} {set j $i} 
      incr i 
   }

   if {[pr_view $active_vw]!=0} { 
      $top.fr.vw pick $j
      pr_gui:viewers:refresh2 $wid $active_vw
   } else { 
     $top.fr.vw pick 0
   }
}



#  Refreshes the GUI entries to show data of the viewer currently selected in the combo-box
#-------------------------------------------------------------------------------------------
proc pr_gui:viewers:refresh2 {wid sel} {
   global pr_gui_view_cb_block_flag
   set top $wid.top 

   if {$sel!={}} {
   # Block the execution of the menu-callbacks since we want to invoke the menu here just to set
   # the right radio-entry and not to exec its callback. After doing this, unblock the callback.
   #   
   set pr_gui_view_cb_block_flag 1
   set vw [lindex $sel 0]
   
   set v [$top.sel subwidget entry]
   $v  delete 0 [string length [$v get]]; $v  insert 0 [pr_view $vw selection]
   set v [$top.hlt subwidget entry]
   $v  delete 0 [string length [$v get]]; $v  insert 0 [pr_view $vw highlight]
   set v [$top.pos subwidget entry]
   $v  delete 0 [string length [$v get]]; $v  insert 0 [pr_view $vw position]
   set v $top.scale; $v  set [pr_view $vw node_scale]

   #Make a list of all node-glyph-engines availabe in the system and show them
   #
   set lst {}; set i 0; set n [pr_glyphs get node_glyph number]
   while {$i<$n} {
     set ng [pr_glyph_factory [pr_glyphs get node_glyph idx $i] get name] 
     set lst [linsert $lst end $ng]; incr i
   }
   pr_gui:cbox_edit_set  $top.ngly $lst
   pr_gui:cbox_edit_pick $top.ngly [pr_view $vw node_glyph]

   #Make a list of all edge-glyph-engines availabe in the system and show them
   #
   set lst {}; set i 0; set n [pr_glyphs get edge_glyph number]
   while {$i<$n} {
     set eg [pr_glyph_factory [pr_glyphs get edge_glyph idx $i] get name] 
     set lst [linsert $lst end $eg]; incr i
   }
   pr_gui:cbox_edit_set  $top.egly $lst
   pr_gui:cbox_edit_pick $top.egly [pr_view $vw edge_glyph]

   pr_gui:combo_edit_set $top.ucb  [pr_view $vw user_cb]
   pr_gui:combo_edit_set $top.swcb [pr_view $vw sweep_cb]

   if {[pr_view $vw output_only]==1} { $top.out config -value {btn} } else { $top.out config -value {} }

   $wid.menu.highlight.dragstyle invoke [pr_view $vw move_style]
   $wid.menu.mapper.node_mapper invoke [pr_view $vw node_mapper]
   $wid.menu.mapper.edge_mapper invoke [pr_view $vw edge_mapper]
   $wid.menu.mapper.edge_drawing invoke [pr_view $vw edge_res]
   #!!!
   #Set the state of the checkbox to the state of pr_view $vw render_2d
   #Set the state of the 3 checkboxes in wid.menu.mapper.bgnd_mappers to 
   #the state of pr_view $vw aux_node_mapper basic|factory|splat
   #
   set pr_gui_view_cb_block_flag 0

   pr_view $vw show  
  } else {

   set v [$top.sel subwidget entry]; $v  delete 0 [string length [$v get]]
   set v [$top.hlt subwidget entry]; $v  delete 0 [string length [$v get]]
   set v [$top.pos subwidget entry]; $v  delete 0 [string length [$v get]]
   $top.out config -value {} 
   pr_gui:cbox_edit_set  $top.ngly {}
   pr_gui:cbox_edit_set  $top.egly {}
   pr_gui:combo_edit_set $top.ucb  ""
   pr_gui:combo_edit_set $top.swcb ""
   $top.scale set 0
  }
}


#Callback for the node size slider
#------------------------------------------------------------------
proc pr_gui:viewers:scale {wid sc} {
  set vw [$wid.fr.vw cget -value]
  if {$vw!=""} { pr_view $vw set node_scale $sc }
}

#Callback for the Apply button
#------------------------------------------------------------------
proc pr_gui:viewers:apply {wid} {
  set top $wid.top
  set vw [$top.fr.vw cget -value]
  if {$vw==""} return
  set hlt [[$top.hlt subwidget entry] get]; if {$hlt==""} {set hlt "None"}
  pr_view $vw set highlight   $hlt
  pr_view $vw set selection   [[$top.sel subwidget entry] get]
  pr_view $vw set position    [[$top.pos subwidget entry] get]
  pr_view $vw set output_only [expr [string compare [$top.out cget -value] btn] + 1]
  pr_view $vw set node_glyph  [pr_gui:cbox_edit_get  $top.ngly]
  pr_view $vw set edge_glyph  [pr_gui:cbox_edit_get  $top.egly]
  pr_view $vw set user_cb     [pr_gui:combo_edit_get $top.ucb]
  pr_view $vw set sweep_cb    [pr_gui:combo_edit_get $top.swcb]
  pr_view $vw touch
}

#Callback for selecting a node/edge glyph-engine. Selects the engine in the current viewer.
#------------------------------------------------------------------------------------------
proc pr_gui:viewers:set_glyph_engine {wid type value} {
  global pr_gui_view_cb_block_flag
  if {$pr_gui_view_cb_block_flag==1} return
  set vw [$wid.top.fr.vw cget -value]
  if {$vw==""}  return 

  if {$type=="nodes"} {
    pr_view $vw set node_glyph $value
  } else {
    pr_view $vw set edge_glyph $value
  }
}

# Deletes the viewer currently selected in the combo-box
#-------------------------------------------------------------------
proc pr_gui:viewers:del {wid}  {
  set vw [$wid.top.fr.vw cget -value]

  if {$vw!=""} { pr_view $vw delete; pr_gui:viewers:refresh $wid }
}

# Updates the viewer currently selected in the combo-box
#-------------------------------------------------------------------
proc pr_gui:viewers:update {wid}  {
  set vw [$wid.top.fr.vw cget -value]
  if {$vw!=""} pr_view $vw touch
}

# pr_gui:viewers:edit_ucb: Edits the user-callback for a given viewer.
#--------------------------------------------------------------------

proc pr_gui:viewers:edit_ucb {wid} {
    set ucb [pr_gui:combo_edit_get $wid]
    pr_gui:edit_proc edit $ucb {opt val}
}


# pr_gui:viewers:edit_swcb: Edits the sweep-callback for a given viewer.
#--------------------------------------------------------------------

proc pr_gui:viewers:edit_swcb {wid} {
    set swcb [pr_gui:combo_edit_get $wid]
    pr_gui:edit_proc edit $swcb {opt val}
}

proc pr_gui:viewers:go_ngly {wid} {
  pr_gui:glyphs:show_node_engine glyphs [pr_gui:cbox_edit_get $wid]
}

proc pr_gui:viewers:go_egly {wid} {
  pr_gui:glyphs:show_edge_engine glyphs [pr_gui:cbox_edit_get $wid]
}
