#include <iostream.h>
#include "Ops.h"
#include "GraphManager.h"
#include "PathQualifier.h"
#include "SelList.h"
#include "Node.h"
#include "Edge.h"
#include "Attr.h"
#include "AttrType.h"
#include "Filter.h"
#include "Geometry.h"
#include "CompoLibrary.h"
#include "RSFDomainModel.h"
#include "GlyphFactoryList.h"
#include "EventHandler.h"
#include "ObsList.h"
#include "Registry.h"
#include "GUIManager.h"
#include "GUIObserver.h"
#include "TclApi.h"
#include "IVViewer.h"
#include "AttrSetType.h"
#include "IVMapper.h"
#include "IVMapperImp.h"
#include "IVGlyphs.h"
#include "IVGlyphFactory.h"
#include "IVUtils.h"
#include "utils.h"
#include <tcl.h>
#include <tk.h>



//Option-processing functions
static void option_layout_bbox(Tcl_Interp*,int&,Tcl_Obj* CONST * &,LayoutOp&);


//---  MISCELLANEOUS METHODS  -------------------------------------------------

int pr_config(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING opt = Tcl_GetString(obj[1]); int i; Tcl_Obj* res = 0;
  if (opt=="set")                                       //Set params:
  {
     opt = Tcl_GetString(obj[2]);
     if (opt=="clear_at_load")                          //Set clear-at-load flag of LoadOp
     {  
        if (Tcl_GetIntFromObj(in,obj[3],&i)!=TCL_OK) return TCL_ERROR;
        LoadOp::clearAtLoad((i)? TRUE:FALSE);
     }
  }
  else if (opt=="get")                                  //Get params:
  {
     opt = Tcl_GetString(obj[2]);
     if (opt=="clear_at_load")                          //Get clear-at-load flag of LoadOp
        res = Tcl_NewIntObj((LoadOp::isClearAtLoad())?1:0);
  }

  if (res) Tcl_SetObjResult(in,res);
  return TCL_OK;
}



int pr_qualify(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   char* p = Tcl_GetString(obj[1]);

   STRING crtp = PathQualifier::setCurrentPath("");     //!!!needed since we don't want qualification w. crt-path

   STRING qp = PathQualifier::qualify(p);

   PathQualifier::setCurrentPath(crtp+"/");             //needed since we changed crt-path, we must set it back

   Tcl_SetObjResult(in,Tcl_NewStringObj(qp,-1));

   return TCL_OK;
}





//---  LOAD METHODS  ------------------------------------------------------------------

int pr_domain_load_rsf(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{							//Load RSF domain and make it the current one
  if (n!=2) return TCL_ERROR;				//REMARK: this doesn't clean up the graph data or other data
							//	  in the system. It is the responsibility of the caller
  char riginode[200],rigiarc[200],rigiattr[200];	//	  to clear up whatever he ants when changing the domain-
                  		                        //	  model. This lets us keep as much as we want from the data
  STRING p = PathQualifier::qualify(Tcl_GetString(obj[1]));  //	  when changing the domain-model.
  sprintf(riginode,"%s/Riginode",(const char*)p);	
  sprintf(rigiarc, "%s/Rigiarc" ,(const char*)p);
  sprintf(rigiattr,"%s/Rigiattr",(const char*)p);

  RSFDomainModel* dom = new RSFDomainModel("Rigi domain model");
  if (!dom->constr(riginode,rigiarc,rigiattr))          //If we couldn't construct the domain-model from files, delete it
  {                                                     
     delete dom;
     return TCL_ERROR;
  }
  dom->setDomainFile(p);                                //Remember file from which we loaded this domain-model
                                                        //(needed for persistency, when this domain-model should reload itself)

  GraphManager::getInstance()->setDomainModel(dom);
  GraphManager::getInstance()->getObsList()->update();	//!!!needed since there might be domain-model observers				

  return TCL_OK;
}


int pr_domain(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING opt = Tcl_GetString(obj[1]);
  GraphManager* gm = GraphManager::getInstance();
  DomainModel*  dm = gm->getDomainModel();
  Tcl_Obj* res = 0; int idx; const AttrSetType* type = 0;
  
  if (opt=="name")
  {  res = Tcl_NewStringObj(dm->getName(),-1); }
  else if (opt=="node_types")
  {  res = Tcl_NewIntObj(dm->numNodeTypes());  }
  else if (opt=="edge_types")
  {  res = Tcl_NewIntObj(dm->numArcTypes());  }
  else if (opt=="node_type")
  {
     opt = Tcl_GetString(obj[2]);
     if (opt=="idx" || opt=="i")
     {  
	if (Tcl_GetIntFromObj(in,obj[3],&idx)!=TCL_OK) return TCL_ERROR;
	type = dm->getNodeType(idx);
	if (opt=="i") res = Tcl_NewStringObj(type->getName(),-1);
     }
     else if (opt=="name")
     {  opt = Tcl_GetString(obj[3]); type = dm->getNodeType(opt);  }
  }
  else if (opt=="edge_type")
  {
     opt = Tcl_GetString(obj[2]);
     if (opt=="idx" || opt=="i")
     {  
	if (Tcl_GetIntFromObj(in,obj[3],&idx)!=TCL_OK) return TCL_ERROR;
	type = dm->getArcType(idx);
	if (opt=="i") res = Tcl_NewStringObj(type->getName(),-1);
     }
     else if (opt=="name")
     {  opt = Tcl_GetString(obj[3]); type = dm->getArcType(opt);  }
  }
  else if (opt=="reset")
  {  gm->setDomainModel(0); gm->getObsList()->update(); }


  if (type && !res)
  {
     res = Tcl_NewListObj(0,0);
     for(int i=0;i<type->numAttrTypes();i++)
     {
        const AttrType* a = type->getAttrType(i);
	Attr::TYPE      t = a->getType();
	const char*    tn = (t==Attr::INT)?       "int" : (t==Attr::FLOAT)? "float" :
			    (t==Attr::STRING)? "string" : (t==Attr::PTR)?     "ptr" : "unset";
        
	Tcl_Obj* CONST arg[] = { Tcl_NewStringObj(a->getName(),-1),Tcl_NewStringObj(tn,-1) };
	Tcl_Obj* itm = Tcl_NewListObj(2,arg);
        Tcl_ListObjAppendElement(in,res,itm);
     }
  }  

  if (res) Tcl_SetObjResult(in,res);
  return TCL_OK;
}



int pr_read_tcl(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;  
  const char* f = Tcl_GetString(obj[1]); 

  STRING qf = PathQualifier::qualify(f);
 
  STRING crt_path = PathQualifier::setCurrentPath(qf);          //since 'f' may recursively load other files,
                                                                //set the current-path for qualification to f's path,
  STRING s; s = "source "; s += addQuotes(qf);                  //so that files referred by 'f' can be searched here too

  GUIManager::exec(s);                                          //this allows 'f' to refer to files relative to its path

  PathQualifier::setCurrentPath(crt_path+"/");                  //restore the crt-path to whatever we had when entered this

  return TCL_OK;
}


int pr_read_sel_file(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;  
  const char* f = Tcl_GetString(obj[1]); 
  STRING qf = PathQualifier::qualify(f);
  GraphManager::getInstance()->getSelList()->read(qf);
  return TCL_OK;
}

int pr_read_own_format(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<2) return TCL_ERROR;  
  Selection* ctx=0;
  if (n>2)                                                      //get the selection-ctx where to read the data,
  {      						        //if any was supplied. If not, read in ctx of                 
     long ptr; Tcl_GetLongFromObj(in,obj[2],&ptr);              //the full-selection
     ctx = (Selection*)ptr;
  }
  const char* f = Tcl_GetString(obj[1]); 
  STRING qf = PathQualifier::qualify(f);
  LoadOp::readOwnFormat(qf,ctx);
  return TCL_OK;
}

int pr_read_graph_ed(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;
  const char* f = Tcl_GetString(obj[1]); 
  STRING qf = PathQualifier::qualify(f);
  LoadOp::readGraphEd(qf);
  return TCL_OK;
}

int pr_read_rsf(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;
  const char* f = Tcl_GetString(obj[1]); 
  STRING qf = PathQualifier::qualify(f);
  LoadOp::readRSF(qf);
  return TCL_OK;
}


int pr_write_session(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   const char* f = Tcl_GetString(obj[1]); 
   GraphManager::getInstance()->save(f); 
   return TCL_OK;
}

int pr_write_own_format(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   if (n!=3) return TCL_ERROR;
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(Tcl_GetString(obj[1]));
   const char*  f = Tcl_GetString(obj[2]); 
   SaveOp::writeOwnFormat(sel,f);
   return TCL_OK;
}

int pr_write_chaco(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   if (n!=3) return TCL_ERROR;
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(Tcl_GetString(obj[1]));
   const char*  f = Tcl_GetString(obj[2]); 
   SaveOp::writeChaco(sel,f);
   return TCL_OK;
}

int pr_write_gv(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   if (n!=3) return TCL_ERROR;
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(Tcl_GetString(obj[1]));
   const char*  f = Tcl_GetString(obj[2]); 
   //REMARK: A more advanced API would be to give to this func the pos,wr_pos,wr_dim params of writeGV() too.
   //	     For the time being we don't use these.
   SaveOp::writeGV(sel,f);
   return TCL_OK;
}

int pr_write_graphed(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   if (n!=3) return TCL_ERROR;
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(Tcl_GetString(obj[1]));
   const char*  f = Tcl_GetString(obj[2]); 
   //REMARK: A more advanced API would be to give to this func the pos,wr_pos params of writeGraphEd() too.
   //	     For the time being we don't use these.
   SaveOp::writeGraphEd(sel,f);						//Do not write node-ids in this file
									//since we want to make it strict to GraphEd-syntax
   return TCL_OK;
}


int pr_write_rsf(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   if (n!=3) return TCL_ERROR;
   Selection* sel = GraphManager::getInstance()->getSelList()->getSelection(Tcl_GetString(obj[1]));
   const char*  f = Tcl_GetString(obj[2]); 
   SaveOp::writeRSF(sel,f);						
   return TCL_OK;
}


//---  COMPONENT LIB MANAGEMENT  -------------------------------------------

int pr_compo_lib(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  ComponentLibrarySet* cl = GraphManager::getInstance()->getCompoLibSet();
  STRING o1 = Tcl_GetString(obj[1]), o2;
  if (n>2) o2 = Tcl_GetString(obj[2]);
  Tcl_Obj* ret = 0;

  if (o1=="load")                                                       //Load compo-lib
     cl->loadLib(o2);      
  else if (o1=="add_proc")                                              //Add Tcl-proc to compo-lib currently undergoing loading
  {
     const char* descr = (n==4)? Tcl_GetString(obj[3]):0;   
     cl->addGlyphProc(o2,descr);
  }
  else if (o1=="num_libs")                                              //Get # compo-libs
     ret = Tcl_NewIntObj(cl->numLibs());
  else if (o1=="lib")                                                   //Get info from ith-lib
  {
     int lib; Tcl_GetIntFromObj(in,obj[2],&lib);
     ComponentLibrary* clib = cl->getLib(lib);                        

     o2 = Tcl_GetString(obj[3]);
     if (o2=="file")                                                    //Get Tcl-lib's filename
        ret = Tcl_NewStringObj(clib->getTclFile(),-1);
     else if (o2=="num_compos")                                         //Get # compos in the lib
        ret = Tcl_NewIntObj(clib->numGlyphProcs());
     else if (o2=="proc")                                               //Get info about Tcl-proc in the lib
     {
        int p; Tcl_GetIntFromObj(in,obj[4],&p);        
        GlyphProc* proc = clib->getGlyphProc(p);
        o2 = Tcl_GetString(obj[5]);                     
        if (o2=="name")                                                 //Get Tcl-proc name
           ret = Tcl_NewStringObj(proc->getTclName(),-1);
        else if (o2=="info")                                            //Get Tcl-proc textual description
           ret = Tcl_NewStringObj(proc->getDescription(),-1);
     }
  }   


  if (ret) Tcl_SetObjResult(in,ret);                                    //Return if there's anything to
  return TCL_OK;
}




//---  SELECTION METHODS  --------------------------------------------------

int pr_sel_new(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  Selection* sel = 0;
  SelectGenericOp::select(sel);						//make new Selection
  if (n==2)								//if name for new selection given:
  {
     sel->ref(Tcl_GetString(obj[1]));					//register it under given name
     sel = GraphManager::getInstance()->getSelList()->getSelection(Tcl_GetString(obj[1]));
									//get the new Selection object 
  }									//('sel' might change, see Selection::ref())
  else sel->ref();							//if no given name, register it under its own name
									//(we must register this new selection anyhow...)

  Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)sel);		//return new Selection* as
  Tcl_SetObjResult(in,res);						//unsigned long...
  return TCL_OK;
}



//---  LAYOUT METHODS  -----------------------------------------------------

int pr_lay(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING o1 = Tcl_GetString(obj[1]), o2 = Tcl_GetString(obj[2]);

  if (o1=="-get")						//we're called only to get some
  {								//of the default-options of class GridLayoutOp
     Tcl_Obj* ret = 0;
     if (o2=="keep_dims")
        ret = Tcl_NewIntObj(LayoutOp::getDefaultKeepDimensions());
     else if (o2=="node_size")
        ret = Tcl_NewDoubleObj(LayoutOp::getDefaultNodeSize());
     else if (o2=="use_crt_pos")
	ret = Tcl_NewIntObj((int)LayoutOp::getDefaultExtLayoutUseCrtPos());
     else if (o2=="use_crt_dim")
	ret = Tcl_NewIntObj((int)LayoutOp::getDefaultExtLayoutUseCrtDim());
     if (ret) Tcl_SetObjResult(in,ret);
     return TCL_OK;
  }
  else if (o1=="-set")
  {
     if (o2=="keep_dims")
     {
	int opt;
        Tcl_GetIntFromObj(in,obj[3],&opt);
        LayoutOp::setDefaultKeepDimensions((BOOL)opt);
        return TCL_OK;
     }
     else if (o2=="node_size")
     {
	double opt;
        Tcl_GetDoubleFromObj(in,obj[3],&opt);
        LayoutOp::setDefaultNodeSize((float)opt);
        return TCL_OK;
     }
     else if (o2=="use_crt_pos")
     {
	int opt;
        Tcl_GetIntFromObj(in,obj[3],&opt);
        LayoutOp::setDefaultExtLayoutUseCrtPos((BOOL)opt);
        return TCL_OK;
     }
     else if (o2=="use_crt_dim")
     {
	int opt;
        Tcl_GetIntFromObj(in,obj[3],&opt);
        LayoutOp::setDefaultExtLayoutUseCrtDim((BOOL)opt);
        return TCL_OK;
     }
  }

  return TCL_ERROR;
}




int pr_lay_grid(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING o1 = Tcl_GetString(obj[1]);
  STRING o2 = Tcl_GetString(obj[2]);
  if (o1=="-get")						//we're called only to get some
  {								//of the default-options of class GridLayoutOp
     Tcl_Obj* ret = 0;
     if (o2=="grid")
        ret = Tcl_NewIntObj(GridLayoutOp::getDefaultGrid());
     else if (o2=="axis")
        ret = Tcl_NewStringObj((GridLayoutOp::getDefaultAxis()==0)? "x" : "y",-1);
     if (ret) Tcl_SetObjResult(in,ret);
     return TCL_OK;
  }
		
  int nelems=-1,axis=-1;
  GridLayoutOp l;

  if (o1!="-set") {  n -= 3; obj += 3; }
  else 		  {  n -= 2; obj += 2; }

  while(n>0)							//process optional args
  {
     STRING opt = Tcl_GetString(obj[0]); obj++; n--;
     if (opt=="bbox") 
        option_layout_bbox(in,n,obj,l);
     else 
     if (opt=="grid")
     { Tcl_GetIntFromObj(in,obj[0],&nelems); obj++;n--; }
     else
     if (opt=="axis")
     {
        char* ax = Tcl_GetString(obj[0]); obj++;n--;
        if (ax[0]=='x' || ax[0]=='X') axis=0;
        if (ax[0]=='y' || ax[0]=='Y') axis=1;
     }
     else { n--; obj++; }
  }

  if (o1!="-set")						//non-static options given
  {
    if (nelems!=-1) l.setGrid(nelems);
    if (axis!=-1)   l.setAxis(axis);
    l.LayoutOp::layout(o1,o2);					//apply the layout
    GraphManager::getInstance()->getObsList()->update();	//!!!needed since viewers observe selection-attrs
  }
  else								//static "-set" options given
  {
    if (nelems!=-1) GridLayoutOp::setDefaultGrid(nelems); 
    if (axis!=-1)   GridLayoutOp::setDefaultAxis(axis);	
  }

  return TCL_OK;
}


int pr_lay_dot(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING o1 = Tcl_GetString(obj[1]);
  STRING o2 = Tcl_GetString(obj[2]);

  if (o1=="-get")						//we're called only to get some
  {								//of the default-options of class NeatoLayoutOp
     Tcl_Obj* ret = 0;
     if (o2=="ranksep")
        ret = Tcl_NewDoubleObj(DotLayoutOp::getDefaultRankSep());
     else if (o2=="nodesep")
        ret = Tcl_NewDoubleObj(DotLayoutOp::getDefaultNodeSep());
     else if (o2=="rankdir")
        ret  = Tcl_NewIntObj(DotLayoutOp::getDefaultRankDir());
     else if (o2=="flip_edges")
	ret = Tcl_NewIntObj((int)DotLayoutOp::getDefaultEdgeFlipping());
     else if (o2=="rank_attr")
	ret = Tcl_NewStringObj(DotLayoutOp::getDefaultLevelAttr(),-1);
     if (ret) Tcl_SetObjResult(in,ret);
     return TCL_OK;
  }

  if (o1!="-set") {  n -= 3; obj += 3; }
  else 		  {  n -= 2; obj += 2; }
  DotLayoutOp l; double d; int i;

  while(n>0)							//process optional args
  {
     STRING opt = Tcl_GetString(obj[0]); obj++; n--;
     if (opt=="bbox") 
        option_layout_bbox(in,n,obj,l);
     else if (opt=="ranksep" && o1=="-set")
     { 
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	DotLayoutOp::setDefaultRankSep((float)d); 
     }
     else if (opt=="nodesep" && o1=="-set")
     {
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	DotLayoutOp::setDefaultNodeSep((float)d); 
     }
     else if (opt=="rankdir" && o1=="-set")
     {
 	Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--; 
	DotLayoutOp::setDefaultRankDir(i); 
     }
     else if (opt=="flip_edges" && o1=="-set")
     {
        Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--;
	DotLayoutOp::setDefaultEdgeFlipping((i)?TRUE:FALSE);
     }
     else if (opt=="rank_attr" && o1=="-set")
     {
        const char* attr = Tcl_GetString(obj[0]); obj++;n--;
	DotLayoutOp::setDefaultLevelAttr(attr);
     }
     else { n--; obj++; }
  }

  if (o1=="-set") return TCL_OK;
  l.LayoutOp::layout(o1,o2);
  GraphManager::getInstance()->getObsList()->update();		//!!!needed since viewers observe selection-attrs
  return TCL_OK;
}


int pr_lay_neato(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING o1 = Tcl_GetString(obj[1]);
  STRING o2 = Tcl_GetString(obj[2]);

  if (o1=="-get")						//we're called only to get some
  {								//of the default-options of class NeatoLayoutOp
     Tcl_Obj* ret = 0;
     if (o2=="weight")
        ret = Tcl_NewDoubleObj(NeatoLayoutOp::getDefaultWeight());
     else if (o2=="eps")
        ret = Tcl_NewDoubleObj(NeatoLayoutOp::getDefaultEpsilon());
     else if (o2=="random")
        ret  = Tcl_NewIntObj(NeatoLayoutOp::getDefaultRandom());
     else if (o2=="maxiter")
        ret  = Tcl_NewIntObj(NeatoLayoutOp::getDefaultMaxIter());
     else if (o2=="overlap")
        ret  = Tcl_NewIntObj(NeatoLayoutOp::getDefaultOverlap());
     else if (o2=="edgelen")
        ret  = Tcl_NewDoubleObj(NeatoLayoutOp::getDefaultEdgeLength());
     if (ret) Tcl_SetObjResult(in,ret);
     return TCL_OK;
  }

  if (o1!="-set") {  n -= 3; obj += 3; }
  else 		  {  n -= 2; obj += 2; }
  NeatoLayoutOp l; double d; int i;

  while(n>0)							//process optional args
  {
     STRING opt = Tcl_GetString(obj[0]); obj++; n--;
     if (opt=="bbox") 
        option_layout_bbox(in,n,obj,l);
     else if (opt=="weight" && o1=="-set")
     { 
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	NeatoLayoutOp::setDefaultWeight((float)d); 
     }
     else if (opt=="eps" && o1=="-set")
     {
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	NeatoLayoutOp::setDefaultEpsilon((float)d); 
     }
     else if (opt=="random" && o1=="-set")
     {
 	Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--; 
	NeatoLayoutOp::setDefaultRandom((BOOL)i); 
     }
     else if (opt=="edgelen" && o1=="-set")
     {
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	NeatoLayoutOp::setDefaultEdgeLength((float)d); 
     }
     else if (opt=="maxiter" && o1=="-set")
     {
 	Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--; 
	NeatoLayoutOp::setDefaultMaxIter(i); 
     }
     else if (opt=="overlap" && o1=="-set")
     {
 	Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--; 
	NeatoLayoutOp::setDefaultOverlap((BOOL)i);
     }
     else { n--; obj++; }
  }
  
  if (o1=="-set") return TCL_OK;
  l.LayoutOp::layout(o1,o2);
  GraphManager::getInstance()->getObsList()->update();		//!!!needed since viewers observe selection attrs	

  return TCL_OK;
}


int pr_lay_level(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING o1 = Tcl_GetString(obj[1]);
  STRING o2 = Tcl_GetString(obj[2]);

  if (o1=="-get")						//we're called only to get some
  {								//of the default-options of class NeatoLayoutOp
     Tcl_Obj* ret = 0;
     if (o2=="level")
        ret = Tcl_NewStringObj(LevelLayoutOp::getDefaultLevelAttr(),-1);
     else if (o2=="spacing")
        ret = Tcl_NewDoubleObj(LevelLayoutOp::getDefaultSpacing());
     else if (o2=="spacing_method")
        ret  = Tcl_NewIntObj(LevelLayoutOp::getDefaultSpacingMethod());
     if (ret) Tcl_SetObjResult(in,ret);
     return TCL_OK;
  }

  if (o1!="-set") {  n -= 3; obj += 3; }
  else 		  {  n -= 2; obj += 2; }
  LevelLayoutOp l; double d; int i;

  while(n>0)							//process optional args
  {
     STRING opt = Tcl_GetString(obj[0]); obj++; n--;
     if (opt=="bbox") 
        option_layout_bbox(in,n,obj,l);
     else if (opt=="level" && o1=="-set")
     { 
 	char* lev = Tcl_GetString(obj[0]); obj++;n--; 
	LevelLayoutOp::setDefaultLevelAttr(lev); 
     }
     else if (opt=="spacing" && o1=="-set")
     {
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	LevelLayoutOp::setDefaultSpacing((float)d); 
     }
     else if (opt=="spacing_method" && o1=="-set")
     {
 	Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--; 
	LevelLayoutOp::setDefaultSpacingMethod((LevelLayoutOp::SPACING_METHOD)i); 
     }
     else { n--; obj++; }
  }

  if (o1=="-set") return TCL_OK;
  l.LayoutOp::layout(o1,o2);
  GraphManager::getInstance()->getObsList()->update();		//!!!needed since viewers observe selection attrs

  return TCL_OK;
}


int pr_lay_nested(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING o1 = Tcl_GetString(obj[1]);
  STRING o2 = Tcl_GetString(obj[2]);

  if (o1=="-get")						//we're called only to get some
  {								//of the default-options of class LevelLayoutOp
     Tcl_Obj* ret = 0;
     if (o2=="zborder")
        ret = Tcl_NewDoubleObj(NestedLayoutOp::getZBorder());
     else if (o2=="xyborder")
        ret = Tcl_NewDoubleObj(NestedLayoutOp::getXYBorder());
     else if (o2=="zlayout")
        ret = Tcl_NewIntObj((int)NestedLayoutOp::getDoZLayout());
     if (ret) Tcl_SetObjResult(in,ret);
     return TCL_OK;
  }

  if (o1!="-set") {  n -= 3; obj += 3; }
  else 		  {  n -= 2; obj += 2; }
  NestedLayoutOp l; double d; int i;

  while(n>0)							//process optional args
  {
     STRING opt = Tcl_GetString(obj[0]); obj++; n--;
     if (opt=="bbox") 
        option_layout_bbox(in,n,obj,l);
     else if (opt=="zborder" && o1=="-set")
     {
 	Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	NestedLayoutOp::setZBorder((float)d); 
     }
     else if (opt=="xyborder" && o1=="-set")
     {
        Tcl_GetDoubleFromObj(in,obj[0],&d); obj++;n--; 
	NestedLayoutOp::setXYBorder((float)d); 
     } 
     else if (opt=="zlayout" && o1=="-set")
     {
        Tcl_GetIntFromObj(in,obj[0],&i); obj++;n--; 
	NestedLayoutOp::setDoZLayout((BOOL)(i!=0)); 
     } 
     else if (opt=="traverse")					//specify how to determine the containment relation
     {
	const char* name = Tcl_GetString(obj[0]); obj++;n--;     
	const char* val  = Tcl_GetString(obj[0]); obj++;n--;    
	AttrFilter f(name,Attr(val),Node::OUT);
	NestedLayoutOp::setFilter(&f);
     }
     else { n--; obj++; }
  }

  if (o1=="-set") return TCL_OK;
  
  l.LayoutOp::layout(o1,o2);				
  GraphManager::getInstance()->getObsList()->update();		//!!!needed since viewers observe selection attrs	
  return TCL_OK;
}



int pr_lay_gem(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  char* sel = Tcl_GetString(obj[1]);
  char* pos = Tcl_GetString(obj[2]);

  GEMLayoutOp l;
  n -= 3; obj+=3;
  while(n>0)							//process optional args
  {
     STRING opt = Tcl_GetString(obj[0]); obj++; n--;
     if (opt=="bbox") 
        option_layout_bbox(in,n,obj,l);
     else { n--; obj++; }
  }

  l.LayoutOp::layout(sel,pos);
  GraphManager::getInstance()->getObsList()->update();		//!!!needed since viewers observe selection attrs
  return TCL_OK;
}



int pr_gem_init(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;

  GEMLayoutOp::init(Tcl_GetString(obj[1]));

  return TCL_OK;
}


int pr_gv_init(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;

  GVLayoutOp::init(Tcl_GetString(obj[1]));

  return TCL_OK;
}

//-----------  Glyph and glyph-factory related  ----------------------------------

int pr_glyphs(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  STRING val; Tcl_Obj* res=0; int i;
  STRING opt = Tcl_GetString(obj[1]);
  IVGlyphFactoryList* gl = GraphManager::getInstance()->getGlyphFactoryList();

  if      (opt=="init")  IVGlyphs::initialize();				//initialize glyph-factory subsystem
  else if (opt=="scale")							//set global scale
  { 
     double s; Tcl_GetDoubleFromObj(in,obj[2],&s);
     IVGlyphs::setScale(s);		
  }
  else if (opt=="new")								//make new named glyph-factory
  {                                                                             //or new glyph-proc
     opt = Tcl_GetString(obj[2]);	
     val = Tcl_GetString(obj[3]);
     if      (opt=="IVTclNodeGlyphFactory")   gl->addFactory(new IVTclNodeGlyphFactory(val));
     else if (opt=="IVTclEdgeGlyphFactory")   gl->addFactory(new IVTclEdgeGlyphFactory(val));
     else if (opt=="IVBasicNodeGlyphFactory") gl->addFactory(new IVBasicNodeGlyphFactory(val));
     else if (opt=="IVBasicEdgeGlyphFactory") gl->addFactory(new IVBasicEdgeGlyphFactory(val));
     else return TCL_ERROR;
  }
  else if (opt=="get")								//get glyph-factory data:
  {
     opt = Tcl_GetString(obj[2]);						//what to get
     if (opt=="scale")			
         res = Tcl_NewDoubleObj(IVGlyphs::getScale());				//get global glyph scale:
     else if (opt=="node_glyph")						//get node-glyph-factory:
     {	
        opt = Tcl_GetString(obj[3]); IVNodeGlyphFactory* ngf = 0;
	if (opt=="name")							//by name
           ngf = gl->getNGFactory(Tcl_GetString(obj[4]));
	else if (opt=="idx")							//by idx
        {
	    if (Tcl_GetIntFromObj(in,obj[4],&i)!=TCL_OK) return TCL_ERROR; 
	    ngf = gl->getNGFactory(i);
        }
        else if (opt=="number")							//# node-glyph-factories
           res = Tcl_NewIntObj(gl->numNGFactories());
	else return TCL_ERROR;
	if (ngf) res = Tcl_NewLongObj((long)(unsigned long)(IVGlyphFactory*)ngf);
     }
     else if (opt=="edge_glyph")						//get edge-glyph-factory:
     {	
        opt = Tcl_GetString(obj[3]); IVEdgeGlyphFactory* egf = 0;
	if (opt=="name")							//by name
           egf = gl->getEGFactory(Tcl_GetString(obj[4]));
	else if (opt=="idx")							//by idx
        {
	    if (Tcl_GetIntFromObj(in,obj[4],&i)!=TCL_OK) return TCL_ERROR; 
	    egf = gl->getEGFactory(i);
        }
        else if (opt=="number")							//# edge-glyph-factories
           res = Tcl_NewIntObj(gl->numEGFactories());
	else return TCL_ERROR;
	if (egf) res = Tcl_NewLongObj((long)(unsigned long)(IVGlyphFactory*)egf);
     }
     else return TCL_ERROR;
  }
  else return TCL_ERROR;							//wrong option

  if (res) Tcl_SetObjResult(in,res);
  return TCL_OK;
}


int pr_glyph_factory(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  long ptr; 									//get IVGlyphFactory this-pointer
  Tcl_GetLongFromObj(in,obj[1],&ptr);						//from Tcl back...
  IVGlyphFactory* gf = (IVGlyphFactory*)ptr;
  Tcl_Obj* res = 0; STRING opt = Tcl_GetString(obj[2]), val;	

  if (opt=="set")								//set glyph-factory params:
  {
     ObserverList* obsl = GraphManager::getInstance()->getObsList();
     opt = Tcl_GetString(obj[3]);						//what to set
     val = Tcl_GetString(obj[4]);						//to what to set
     if (gf->isOfType("IVTclNodeGlyphFactory"))					//for IVTclNodeGlyphFactory:
     {
	IVTclNodeGlyphFactory* ngf = (IVTclNodeGlyphFactory*)gf;
	if (opt=="cb")								//set Tcl-cb
	{  ngf->setTclCallback(val); obsl->update();  }
	else return TCL_ERROR;  
     }
     else if (gf->isOfType("IVTclEdgeGlyphFactory"))				//for IVTclEdgeGlyphFactory:
     {
	IVTclEdgeGlyphFactory* egf = (IVTclEdgeGlyphFactory*)gf;
	if (opt=="cb")								//set Tcl-cb
	{  egf->setTclCallback(val); obsl->update();  }
	else return TCL_ERROR;  
     }
     else return TCL_ERROR;
  }
  else if (opt=="get")								//get glyph-factory param:
  {
     opt = Tcl_GetString(obj[3]);						//what to get
     if (opt=="name")
        res = Tcl_NewStringObj(gf->getName(),-1);				//get name
     else if (opt=="type")
        res = Tcl_NewStringObj(gf->getType(),-1);				//get type-name
     else if (gf->isOfType("IVTclNodeGlyphFactory"))				//for IVTclNodeGlyphFactory
     {
	IVTclNodeGlyphFactory* ngf = (IVTclNodeGlyphFactory*)gf;
	if (opt=="cb")								//get Tcl-cb
	   res = Tcl_NewStringObj(ngf->getTclCallback(),-1);
	else return TCL_ERROR;  
     }
     else if (gf->isOfType("IVTclEdgeGlyphFactory"))				//for IVTclEdgeGlyphFactory
     {
	IVTclEdgeGlyphFactory* egf = (IVTclEdgeGlyphFactory*)gf;
	if (opt=="cb")								//get Tcl-cb
	   res = Tcl_NewStringObj(egf->getTclCallback(),-1);
	else return TCL_ERROR;  
     }
     else return TCL_ERROR;
  }
  else return TCL_ERROR;							//it's set or get...

  if (res) Tcl_SetObjResult(in,res);	
  return TCL_OK;
}



int pr_num_selections(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n>1) return TCL_ERROR;						//check arg count

  Tcl_Obj* res = Tcl_NewIntObj(GraphManager::getInstance()->getSelList()->numSelections());

  Tcl_SetObjResult(in,res);
  return TCL_OK;
}





int pr_get_selection(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=3) return TCL_ERROR;						//check arg count

  SelectionList* sl = GraphManager::getInstance()->getSelList();
  STRING       arg1 = Tcl_GetString(obj[1]);
  Selection*    sel = 0;

  if (arg1=="name")							//get selection by name:
     sel = sl->getSelection(Tcl_GetString(obj[2]));
  else if (arg1=="idx")							//get selection by idx:
  {
     int i;
     if (Tcl_GetIntFromObj(in,obj[2],&i)!=TCL_OK) return TCL_ERROR;  	//check we can read arg2 as int
     if (i>=sl->numSelections()) return TCL_ERROR;			//check i-th selection exists indeed
     sel = sl->getSelection(i);
  }
  else return TCL_ERROR;						//wrong option

  Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)sel);		//return Selection* as
  Tcl_SetObjResult(in,res);						//unsigned long...
  return TCL_OK;
}




int pr_selection(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<2) return TCL_ERROR;						//check arg count

  long ptr; 								//get Selection* this-pointer
  Tcl_GetLongFromObj(in,obj[1],&ptr);					//from Tcl back...
  Selection* sel = (Selection*)ptr;

  Tcl_Obj* res = 0; 						
  STRING opt = Tcl_GetString(obj[2]);		

  if (opt=="nnodes")							//get # nodes in selection:
     res = (sel)? Tcl_NewIntObj(sel->numNodes()) : 0;
  else if (opt=="nedges")						//get # edges in selection:
     res = (sel)? Tcl_NewIntObj(sel->numEdges()) : 0;			
  else if (opt=="delete")						//delete selection:
  {
     if (sel)
     {	     
        GraphManager::getInstance()->getSelList()->delSelection(sel->getName());
	//!!!GraphManager::getInstance()->getObsList()->update();		//!!!needed since selections have changed
     }
     return TCL_OK;
  }
  else if (opt=="contains")
  { 
     opt = Tcl_GetString(obj[3]);
     if (opt=="node")
     {
        Tcl_GetLongFromObj(in,obj[4],&ptr); Node* n = (Node*)ptr;
	res = Tcl_NewIntObj(sel->hasNode(n));
     }
     else if (opt=="edge")
     {
        Tcl_GetLongFromObj(in,obj[4],&ptr); Edge* e = (Edge*)ptr;
	res = Tcl_NewIntObj(sel->hasEdge(e));
     }
     else return TCL_ERROR;
  }
  else if (opt=="name")							//get selection name:
     res = Tcl_NewStringObj((sel)? sel->getName(): Selection::NONE,-1);
  else if (opt=="iter_nodes")						//get node-iterator:
  {
     Selection::NodeIterator* ni = (sel)? sel->iterNodes() : 0;
     res = Tcl_NewLongObj((long)(unsigned long)ni);
  }
  else if (opt=="iter_edges")						//get edge-iterator:
  {
     Selection::EdgeIterator* ei = (sel)? sel->iterEdges() : 0;
     res = Tcl_NewLongObj((long)(unsigned long)ei);
  }
  else if (opt=="add_node")						//add node to selection:
  {
     Tcl_GetLongFromObj(in,obj[3],&ptr);				//get Node* to add 
     sel->addNode((Node*)ptr);
  }
  else if (opt=="add_edge")
  {
      Tcl_GetLongFromObj(in,obj[3],&ptr);				//get Edge* to add
      sel->addEdge((Edge*)ptr);
  }
  else if (opt=="reset")						//reset selection
  {
      sel->reset();
      GraphManager::getInstance()->getObsList()->update();		//!!!needed since selection modified
      return TCL_OK;
  }
  else if (opt=="flush")
  {
      sel->flush();
      GraphManager::getInstance()->getObsList()->update();		//!!!needed since selection modified
      return TCL_OK;
  }
  else if (opt=="rename")						//rename selection
  {
      const char* new_name = Tcl_GetString(obj[3]);
      EditOp::renameSelection(sel,new_name);
      return TCL_OK;
  }
  else									//wrong option  
     return TCL_ERROR;	

  if (res) Tcl_SetObjResult(in,res);
  return TCL_OK;
}


int pr_iterator(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=3) return TCL_ERROR;						//check arg count
  long ptr; 								//get Node/EdgeIterator*
  Tcl_GetLongFromObj(in,obj[1],&ptr);					//from Tcl back...
  if (!ptr) return TCL_OK;						//silently ignore NULL-pointers
  Tcl_Obj* res = 0; 						
  STRING opt = Tcl_GetString(obj[2]);		

  if (opt=="next_node")							//get next-node:
  {
     Selection::NodeIterator* ni = (Selection::NodeIterator*)ptr;
     Node* nd = ni->next();
     res = Tcl_NewLongObj((long)(unsigned long)nd);
  }
  else if (opt=="next_edge")						//get next-edge:
  {
     Selection::EdgeIterator* ei = (Selection::EdgeIterator*)ptr;
     Edge* ed = ei->next();
     res = Tcl_NewLongObj((long)(unsigned long)ed);
  }
  else if (opt=="delete")						//delete iterator - to be used ONLY
  {									//if we're sure the iterator is not at its end, 
     Selection::Iterator* i = (Selection::Iterator*)ptr;		//since then it deletes itself automatically
     delete i;								//This function is needed to dispose of not-yet-finished iterators
  }	  
  else return TCL_ERROR;

  Tcl_SetObjResult(in,res);
  return TCL_OK;
}



int pr_get_node(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<4) return TCL_ERROR;						//check arg count

  const char* arg0 = Tcl_GetString(obj[1]);                             //get name of selection to search into
  STRING      arg1 = Tcl_GetString(obj[2]);                             //get what to search on
  Selection*     g = GraphManager::getInstance()->getSelList()->getSelection(arg0);
  Node*         nd = 0;    

  if (arg1=="attr")							//get node by attr-value
  {
     const char* nm  = Tcl_GetString(obj[3]);
     const char* val = Tcl_GetString(obj[4]);
     nd = g->getNodeByAttr(nm,val);  
  }   
  else if (arg1=="id")							//get node by id
  {
     int id;
     if (Tcl_GetIntFromObj(in,obj[3],&id)!=TCL_OK) return TCL_ERROR;
     nd = g->getNodeById(id);
  }
  else return TCL_ERROR;						//wrong option
   
  Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)nd);		//return Node* as
  Tcl_SetObjResult(in,res);						//unsigned long...
  return TCL_OK;
}


int pr_get_edge(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<4) return TCL_ERROR;						//check arg count

  const char* arg0 = Tcl_GetString(obj[1]);                             //get name of selection to search into
  STRING      arg1 = Tcl_GetString(obj[2]);                             //get what to search on
  Selection*     g = GraphManager::getInstance()->getSelList()->getSelection(arg0);
  Edge*         ed = 0;    

  if (arg1=="attr")							//get edge by attr-value
  {
     const char* nm  = Tcl_GetString(obj[3]);
     const char* val = Tcl_GetString(obj[4]);
     ed = g->getEdgeByAttr(nm,val);  
  }   
  else if (arg1=="id")							//get edge by id
  {
     int id;
     if (Tcl_GetIntFromObj(in,obj[3],&id)!=TCL_OK) return TCL_ERROR;
     ed = g->getEdgeById(id);
  }
  else return TCL_ERROR;						//wrong option
   
  Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)ed);		//return Edge* as
  Tcl_SetObjResult(in,res);						//unsigned long...
  return TCL_OK;
}


int pr_node(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<2) return TCL_ERROR;

  long ptr; 								//get Node* this-pointer
  Tcl_GetLongFromObj(in,obj[1],&ptr);					//from Tcl back...
  Node* nd = (Node*)ptr;
  ObserverList* obsl = GraphManager::getInstance()->getObsList();

  Tcl_Obj* res = 0;
  STRING opt = Tcl_GetString(obj[2]);

  if (opt=="id") 							//get id of node:
     res = Tcl_NewIntObj((nd)? nd->getId() : 0);
  else if (opt=="delete")
  {
     if (nd) {  nd->destroy(); obsl->update(); }
     return TCL_OK;  
  }   
  else if (opt=="type")
  {  res = Tcl_NewStringObj(nd->getType()->getName(),-1);  }
  else if (opt=="attrs")						//get attrs of node:
  {
     res = Tcl_NewListObj(0,0);
     if (nd)
       for(int i=0;i<nd->numAttrs();i++)
       {  
         const char* val = nd->getAttrVal(i)->getString();
         const char* nm  = nd->getAttrName(i);
         Tcl_Obj* CONST arg[] = { Tcl_NewStringObj(nm,-1),Tcl_NewStringObj(val,-1) };
	 Tcl_Obj*    itm = Tcl_NewListObj(2,arg);
         Tcl_ListObjAppendElement(in,res,itm);
       }
  }
  else if (opt=="attr")							//get a named attr of node
  {
     const char* nm  = Tcl_GetString(obj[3]);				//get attr-name 
     const char* val = (nd)? nd->getString(nm) : "";			//get attr-val
     res = Tcl_NewStringObj(val,-1);
  }
  else if (opt=="set")							//set a named attr of node
  {
     if (!nd) return TCL_OK;
     STRING nm  = Tcl_GetString(obj[3]);				//get name of attr
     STRING typ = Tcl_GetString(obj[5]);				//get type of attr
     if (typ=="int")							//set int-attrs 
     { 
        int val; 
        if (Tcl_GetIntFromObj(in,obj[4],&val)==TCL_OK)
	   nd->set(nm,val); else return TCL_ERROR;
     }
     else if (typ=="float")						//set float-attrs
     {
	double val;						
        if (Tcl_GetDoubleFromObj(in,obj[4],&val)==TCL_OK)
	   nd->set(nm,(float)val); else return TCL_ERROR;
     }
     else if (typ=="string")						//set string-attrs
        nd->set(nm,Tcl_GetString(obj[4]));
     else return TCL_ERROR;						//only "int","float","string" attr-types allowed!
     return TCL_OK;	
  }
  else if (opt=="unset")						//delete a named attr of node
  {									//(or all attrs if no name is given)
     if (!nd) return TCL_OK;
     nd->del((n>3)? Tcl_GetString(obj[3]) : 0);
     return TCL_OK;
  }
  else if (opt=="edges")						//get edges of this node into a selection-object
  {
     Selection* ctx;    
     if (n==4)                                                          //get Selection in ctx of which we look for edges
     {                                                                  //if none given, use the full-selection ctx
        Tcl_GetLongFromObj(in,obj[3],&ptr); ctx = (Selection*)ptr;
     }
     else ctx = GraphManager::getInstance()->getSelList()->getFullSelection();
     Selection* e   = new Selection; e->ref();                          //make new Selection and put node's edges in it.
     EdgeSet&   es  = nd->getEdges(ctx);                                //don't forget to ref() the selection since we
     for(int i=0;i<es.Count();i++)                                      //do NOT delete it ourselves but return it to caller...
        e->addEdge(es[i]);      
     res = Tcl_NewLongObj((long)(unsigned long)e);
  }

  if (!res) return TCL_ERROR;						//we couldn't create return-val, abort
  Tcl_SetObjResult(in,res);
  return TCL_OK;
}



int pr_edge(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<2) return TCL_ERROR;

  long ptr; 								//get Edge* this-pointer
  Tcl_GetLongFromObj(in,obj[1],&ptr);					//from Tcl back...
  Edge* ed = (Edge*)ptr;

  Tcl_Obj* res = 0;
  STRING opt = Tcl_GetString(obj[2]);

  if (opt=="id") 							//get id of edge:
     res = Tcl_NewIntObj((ed)? ed->getId() : 0);
  else if (opt=="delete")
  {  if (ed) ed->destroy(); return TCL_OK;  }   
  else if (opt=="type")							//get type-name of edge-type
     res = Tcl_NewStringObj(ed->getType()->getName(),-1);
  else if (opt=="attrs")						//get attrs of edge:
  {
     res = Tcl_NewListObj(0,0);
     if (ed)
       for(int i=0;i<ed->numAttrs();i++)
       {  
         const char* val = ed->getAttrVal(i)->getString();
         const char* nm  = ed->getAttrName(i);
         Tcl_Obj* CONST arg[] = { Tcl_NewStringObj(nm,-1),Tcl_NewStringObj(val,-1) };
	 Tcl_Obj*    itm = Tcl_NewListObj(2,arg);
         Tcl_ListObjAppendElement(in,res,itm);
       }
  }
  else if (opt=="node") 						//get the 2 nodes connected by edge
  {
     STRING nd  = Tcl_GetString(obj[3]);
     Node*   n  = (nd=="from")? ed->fromNode() : (nd=="to")? ed->toNode() : 0;
     res        = Tcl_NewLongObj((long)(unsigned long)n);
  }
  else if (opt=="attr")							//get a named attr of edge
  {
     const char* nm  = Tcl_GetString(obj[3]);				//get attr-name 
     const char* val = (ed)? ed->getString(nm) : "";			//get attr-val and return it as string
     res = Tcl_NewStringObj(val,-1);
  }
  else if (opt=="set")							//set a named attr of edge
  {
     if (!ed) return TCL_OK;
     STRING nm  = Tcl_GetString(obj[3]);				//get name of attr
     STRING typ = Tcl_GetString(obj[5]);				//get type of attr
     if (typ=="int")							//set int-attrs 
     { 
        int val; 
        if (Tcl_GetIntFromObj(in,obj[4],&val)==TCL_OK)
	   ed->set(nm,val); else return TCL_ERROR;
     }
     else if (typ=="float")						//set float-attrs
     {
	double val;						
        if (Tcl_GetDoubleFromObj(in,obj[4],&val)==TCL_OK)
	   ed->set(nm,(float)val); else return TCL_ERROR;
     }
     else if (typ=="string")						//set string-attrs
        ed->set(nm,Tcl_GetString(obj[4]));
     else return TCL_ERROR;						//only "int","float","string" attr-types allowed!
     return TCL_OK;	
  }
  else if (opt=="unset")						//delete a named attr
  {									//(or all attrs if no name is given)
     if (!ed) return TCL_OK;
     ed->del((n>3)? Tcl_GetString(obj[3]) : 0);
     return TCL_OK;
  }

  if (!res) return TCL_ERROR;						//we couldn't create return-val, abort
  Tcl_SetObjResult(in,res);
  return TCL_OK;
}
 


int pr_register(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[]) //Set or query the registry database
{
  if (n<3) return TCL_ERROR;

  STRING opt = Tcl_GetString(obj[1]);
  STRING key = Tcl_GetString(obj[2]);
  Registry* reg = GraphManager::getInstance()->getRegistry();
  
  if (opt=="add")							//register new entry
  {
     DARRAY<STRING> val;
     for(int i=3;i<n;i++)
        val.Add(Tcl_GetString(obj[i]));
     reg->add(key,val);	
     return TCL_OK;
  }
  else if (opt=="get")							//get entries for given key
  {
     Tcl_Obj* res = Tcl_NewListObj(0,0);
     Registry::Iterator it(*reg,key);					//iterate over all entries for 'key'
     for(int i=0;i<it.num();i++)
     {
       Tcl_Obj* lst = Tcl_NewListObj(0,0);
       DARRAY<STRING>& val = it.get(i);
       for(int j=0;j<val.Count();j++)
       {  
         Tcl_Obj* itm = Tcl_NewStringObj(val[j],-1);
	 Tcl_ListObjAppendElement(in,lst,itm);
       }
       Tcl_ListObjAppendElement(in,res,lst);
     }
     Tcl_SetObjResult(in,res);
     return TCL_OK;
  }
  
  return TCL_ERROR;
}

//---   Viewer-related   ---------------------------------------------------------------------------------


int pr_view_new(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n<2 || n>5) return TCL_ERROR;				//check arg count

  const char* name     = Tcl_GetString(obj[1]);			//get viewer name
  const char* selname  = (n>2)? Tcl_GetString(obj[2]):Selection::NONE;	
								//get selection name if any given
  const char* posname  = (n>3)? Tcl_GetString(obj[3]):"";	//get pos-attr-prefix name if any given
  const char* hlname   = (n>4)? Tcl_GetString(obj[4]):Selection::NONE;	
								//get highlight sel name if any given
 
  IVViewer* vw = new IVViewer(name,selname,posname,hlname);	//build new IVViewer - it get automatically into 
								//the ObsList, since it is an Observer

  GraphManager::getInstance()->getObsList()->update();		//needed since viewer-GUIs observe viewers

  return TCL_OK;
}



int pr_num_views(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n>1) return TCL_ERROR;					//check arg count
  Tcl_Obj* res = Tcl_NewIntObj(IVViewer::numViewers());
  Tcl_SetObjResult(in,res);
  return TCL_OK;
}


int pr_get_view(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n>2) return TCL_ERROR;					//check arg count

  int i;
  if (Tcl_GetIntFromObj(in,obj[1],&i)!=TCL_OK) return TCL_ERROR;  //check we can read arg1 as int
  
  if (i>=IVViewer::numViewers()) return TCL_ERROR;		  //check i-th viewer exists indeed
  Tcl_Obj* res = Tcl_NewStringObj(IVViewer::getViewer(i)->getName(),-1);
  Tcl_SetObjResult(in,res);
  return TCL_OK;
}



int pr_view(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  const char* vn = Tcl_GetString(obj[1]);						//get viewer name

  ObserverList* obsl = GraphManager::getInstance()->getObsList();
  IVViewer* 	  vw = (IVViewer*)obsl->getObserver(vn);
  if (!vw || n==2) { Tcl_SetObjResult(in,Tcl_NewLongObj((long)(unsigned long)vw)); return TCL_OK; }

  STRING opt = Tcl_GetString(obj[2]), val;						//get option arg

  Tcl_Obj* res =0; 									//GET ATTRIBUTES:
  if (opt=="node_scale")    res = Tcl_NewDoubleObj(vw->getMapper()->getNodeScale());	//return vw's node scale
  else
  if (opt=="edge_res")      res = Tcl_NewIntObj(vw->getMapper()->getEdgeResolution());	//return vw's edge-resolution-hint
  else
  if (opt=="selection")	    res = Tcl_NewStringObj(vw->getSelection(),-1);		//return vw's sel-name
  else
  if (opt=="position")	    res = Tcl_NewStringObj(vw->getPosAttr(),-1);		//return vw's pos-attr-name
  else 
  if (opt=="highlight")	    res = Tcl_NewStringObj(vw->getHighlightSelection(),-1);	//return vw's highlight-sel-name
  else  
  if (opt=="highlight_col")								//return vw's highlight-color
  {  float r,g,b; vw->getHighlightColor(r,g,b);	 res = Tcl_NewStringObj(rgb2tkcolor(r,g,b),-1); }
  else											//return vw's background-color
  if (opt=="background_col")
  {  float r,g,b; vw->getBackgroundColor(r,g,b); res = Tcl_NewStringObj(rgb2tkcolor(r,g,b),-1); } 
  else
  if (opt=="move_style")  res = Tcl_NewIntObj(vw->getMapper()->getMoveStyle());		//return vw's move style
  else
  if (opt=="output_only") res = Tcl_NewIntObj(vw->getOutputOnly());			//return if this is output-only
  else										
  if (opt=="render_2d")   res = Tcl_NewIntObj(vw->get2DRendering());			//return if this uses 2D render/shading look
  else								
  if (opt=="pan_step")    res = Tcl_NewDoubleObj(UIViewerBase::getPanStep());		//return global pan-step
  else
  if (opt=="user_cb")									//get user-callback data, i.e. Tcl-proc
  {  res = Tcl_NewStringObj(vw->getUserCallback(),-1); }
  else
  if (opt=="sweep_cb")									//get sweep-callback data, i.e. Tcl-proc 
  {  res = Tcl_NewStringObj(vw->getSweepCallback(),-1); }	  
  else    
  if (opt=="enable_sweep_cb")
  {
     res = Tcl_NewIntObj((vw->getEventHandler()->sweepCallbackEnabled())?1:0);
  }
  else
  if (opt=="touch")      { vw->touch();return TCL_OK; }
  else
  if (opt=="show")	 { vw->show(); return TCL_OK; }
  else 
  if (opt=="pan")									//pan the view in the X/Y directions	 
  {   
     val = Tcl_GetString(obj[3]);
     if (val=="left")		vw->pan(UIViewerBase::INCR_X);
     else if (val=="right")	vw->pan(UIViewerBase::DECR_X);
     else if (val=="up")	vw->pan(UIViewerBase::DECR_Y);
     else if (val=="down")	vw->pan(UIViewerBase::INCR_Y);
     return TCL_OK;
  }
  else if (opt=="add_button")								//add user-button
  {											//TO_DO: Expose last 2 optional params
     vw->addButton(Tcl_GetString(obj[3]),Tcl_GetString(obj[4]));
     return TCL_OK;
  }
  else if (opt=="delete")
  {
    obsl->delObserver(vn);								//delete Observer with that name - will destroy the IVViewer
    obsl->update();									//needed since viewer-GUIs observe viewers
    return TCL_OK;
  }
  else if (opt=="node_glyph")
  {
     const char* f = vw->getIVFactoryNodeMapper()->getNodeGlyphFactory();  
     res = Tcl_NewStringObj(f,-1);
  }
  else  if (opt=="edge_glyph")
  {
     const char* f = vw->getIVFactoryEdgeMapper()->getEdgeGlyphFactory();  
     res = Tcl_NewStringObj(f,-1);
  }
  else
  if (opt=="node_mapper")
  {  res = Tcl_NewIntObj(vw->getNodeMapper());  }
  else
  if (opt=="edge_mapper")
  {  res = Tcl_NewIntObj(vw->getEdgeMapper());  }
  else
  if (opt=="aux_node_mapper")
  {  
     val = Tcl_GetString(obj[3]);
     int nm = (val=="basic"   || val=="0")? IVViewer::BASIC :
              (val=="factory" || val=="1")? IVViewer::FACTORY : 
              (val=="splat"   || val=="2")? IVViewer::SPLAT : -1;
     if (nm==-1) return TCL_ERROR;    
     res = Tcl_NewIntObj(vw->getAuxNodeMapper((IVViewer::MAPPER_TYPE)nm));  
  }
  else
  if (opt=="set")						//SET ATTRIBUTES:--------------------------
  {
     opt = Tcl_GetString(obj[3]);				//get name of attr to set
     val = Tcl_GetString(obj[4]);				//get val to set to
     if (opt=="position")        vw->setPosAttr(val);
     else if (opt=="highlight")  vw->setHighlightSelection(val);
     else if (opt=="highlight_col") 
     {  float r,g,b; if (tkcolor2rgb(val,r,g,b)) vw->setHighlightColor(r,g,b);  }
     else if (opt=="background_col")
     {  float r,g,b; if (tkcolor2rgb(val,r,g,b)) vw->setBackgroundColor(r,g,b); }
     else if (opt=="selection")  vw->setSelection(val);   
     else if (opt=="output_only") vw->setOutputOnly(val=="1" || val=="true");
     else if (opt=="render_2d") vw->set2DRendering(val=="1" || val=="true");
     else if (opt=="move_style") 
     {
	int style = (val=="low"  || val=="0")? IVMapperBase::LOW_RES :
	            (val=="mid"  || val=="1")? IVMapperBase::MID_RES :
		    (val=="high" || val=="2")? IVMapperBase::HIGH_RES : -1;	
	if (style==-1) return TCL_ERROR;
	vw->getMapper()->setMoveStyle((IVMapperBase::MOVE_STYLE)style);
     }
     else if (opt=="node_mapper")
     {
	int nm = (val=="basic"   || val=="0")? IVViewer::BASIC :
	         (val=="factory" || val=="1")? IVViewer::FACTORY : 
                 (val=="splat"   || val=="2")? IVViewer::SPLAT : -1;
        if (nm==-1) return TCL_ERROR;
	vw->setNodeMapper((IVViewer::MAPPER_TYPE)nm);
     }
     else if (opt=="aux_node_mapper")
     {
	int nm = (val=="basic"   || val=="0")? IVViewer::BASIC :
	         (val=="factory" || val=="1")? IVViewer::FACTORY : 
                 (val=="splat"   || val=="2")? IVViewer::SPLAT : -1;
        if (nm==-1) return TCL_ERROR;
        val = Tcl_GetString(obj[5]);
	vw->setAuxNodeMapper((IVViewer::MAPPER_TYPE)nm,(val=="true" || val=="1"));
     }
     else if (opt=="edge_mapper")
     {
	int em = (val=="basic"   || val=="0")? IVViewer::BASIC :
	         (val=="factory" || val=="1")? IVViewer::FACTORY : -1;
        if (em==-1) return TCL_ERROR;
	vw->setEdgeMapper((IVViewer::MAPPER_TYPE)em);
     }    
     else if (opt=="node_scale")
     {
        double scale;
        if (Tcl_GetDoubleFromObj(in,obj[4],&scale)!=TCL_OK) return TCL_ERROR;
        vw->getMapper()->setNodeScale(float(scale));
     }
     else if (opt=="edge_res")
     {
	int er = (val=="node_centers" || val=="0")? IVMapperBase::NODE_CENTERS :
	         (val=="straight"     || val=="1")? IVMapperBase::STRAIGHT :
		 (val=="default"      || val=="2")? IVMapperBase::DEFAULT_RES : 
		 (val=="smooth"       || val=="3")? IVMapperBase::SMOOTH : -1;
	if (er==-1) return TCL_ERROR;
	vw->getMapper()->setEdgeResolution((IVMapperBase::EDGE_RESOLUTION)er);
     }		
     else if (opt=="node_glyph")
     {
	vw->getIVFactoryNodeMapper()->setNodeGlyphFactory(val);
        vw->touch();                                                    //REMARK: needed since viewers observe factories
     }
     else if (opt=="edge_glyph")
     {
	vw->getIVFactoryEdgeMapper()->setEdgeGlyphFactory(val);
        vw->touch();                                                    //REMARK: needed since viewers observe factories
     }
     else if (opt=="user_cb")						//add mouse-selection-callback to viewer, 	
     {									//passing the Tcl-code to be executed
        vw->setUserCallback(val);
     }
     else if (opt=="sweep_cb")						//add mouse sweep (move) callback
     {									//passing the Tcl-code to be executed
        vw->setSweepCallback(val);
     }
     else if (opt=="enable_sweep_cb")
     {
        vw->getEventHandler()->enableSweepCallback((BOOL)(val=="1" || val=="true"));
     }
     else if (opt=="pan_step")
     {  
	double pan_step;
	if (Tcl_GetDoubleFromObj(in,obj[4],&pan_step)!=TCL_OK) return TCL_ERROR;
	UIViewerBase::setPanStep(float(pan_step));
     }	
     return TCL_OK;
  }
  else return TCL_ERROR;					//unknown option	

  Tcl_SetObjResult(in,res);
  return TCL_OK;
}






int pr_add_observer(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=3) return TCL_ERROR;					//check arg count

  const char* name = Tcl_GetString(obj[1]);
  const char* code = Tcl_GetString(obj[2]);

  new GUIObserver(name,code);  

  return TCL_OK;
}


int pr_del_observer(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=2) return TCL_ERROR;					//check arg count

  const char* name = Tcl_GetString(obj[1]);
   
  GraphManager::getInstance()->getObsList()->delObserver(name);	//delete Observer with that name 
								//- will destroy the GUIObserver
  return TCL_OK;
}


//---  MISCELLANEOUS METHODS  ---------------------------------------------


int pr_reset(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=1) return TCL_ERROR;
  EditOp::reset();
  return TCL_OK;
}

int pr_exit(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
  if (n!=1) return TCL_ERROR;

  GUIManager::shutdown();				//This simply calls shutdown which forces
							//an exit() and never returns.
  return TCL_OK;
}

int pr_compute_level(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[]) 
{
  long ptr;						//get Selection* this-pointer
  Tcl_GetLongFromObj(in,obj[1],&ptr);			//from Tcl back...
  Selection* sel = (Selection*)ptr; 
  const char* lev_attr = Tcl_GetString(obj[2]);		//get level-attr-name
  const char* name     = Tcl_GetString(obj[3]);		//get name & value of attrs to follow to
  const char* value    = Tcl_GetString(obj[4]);		//traverse graph from 'sel'
  STRING      dir      = (n==6)?			//get direction: "out" for roots->leaves, "in" for leaves->roots 
			 Tcl_GetString(obj[5]):"out";	//(default arg val: "out")

  AttrFilter f(name,Attr(value),			//make a filter to encode the traversal via (name,value)
	       (dir=="in")? Node::IN:Node::OUT);	//in direction 'dir'
  AttrOp::computeLevel(sel,&f,lev_attr);
  return TCL_OK;
}


int pr_traverse_tree(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[]) 
{
  long ptr; 
  Tcl_GetLongFromObj(in,obj[1],&ptr);
  Selection*  all = (Selection*)ptr;			//complete selection to traverse into
  Tcl_GetLongFromObj(in,obj[2],&ptr);
  Selection*  root = (Selection*)ptr;			//root (start) selection
  STRING      opt  = Tcl_GetString(obj[3]);		//traverse via attr-values or type-names?
  const char* name = Tcl_GetString(obj[4]);		//attr-name or type-name
  DirectionFilter* filt = 0;
  const char* res  = 0;

  if (opt=="attr")					//make a traversal-filter via attr-names & values
  {  
     const char* val = Tcl_GetString(obj[5]);
     filt = new AttrFilter(name,Attr(val),Node::OUT);
     res  = Tcl_GetString(obj[6]);
  }
  else if (opt=="type")					//make a traversal-filter via type-names
  {
     filt = new TypeFilter(name,Node::OUT);
     res  = Tcl_GetString(obj[5]);	  
  }
  else return TCL_ERROR;

  Selection* res_sel = 0;				//do tree or DAG selection
  SelectSetOp::tree(root,all,filt,0,res_sel);
  res_sel->ref(res);					//rename selection
  delete filt;						//done with filter
  return TCL_OK;
}




int pr_traverse_dag(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[]) 
{
  long ptr; Tcl_GetLongFromObj(in,obj[1],&ptr);
  Selection*  root = (Selection*)ptr;			//root (start) selection
  STRING      opt  = Tcl_GetString(obj[2]);		//traverse via attr-values or type-names?
  const char* name = Tcl_GetString(obj[3]);		//attr-name or type-name
  DirectionFilter* filt = 0;
  const char* res  = 0;

  if (opt=="attr")					//make a traversal-filter via attr-names & values
  {  
     const char* val = Tcl_GetString(obj[4]);
     filt = new AttrFilter(name,Attr(val),Node::OUT);
     res  = Tcl_GetString(obj[5]);
  }
  else if (opt=="type")					//make a traversal-filter via type-names
  {
     filt = new TypeFilter(name,Node::OUT);
     res  = Tcl_GetString(obj[4]);	  
  }
  else return TCL_ERROR;
  
  Selection* res_sel = 0;				//do tree or DAG selection
  Selection* all     = GraphManager::getInstance()->getSelList()->getFullSelection();	
							//!!! in the future, we'd like maybe to traverse
							//within a given (maybe smaller) selection, not in All
  SelectSetOp::dag(root,all,filt,res_sel);
  res_sel->ref(res);					//rename selection
  delete filt;						//done with filter
  return TCL_OK;
}



int pr_selection_roots(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[]) 
{
  long ptr; Tcl_GetLongFromObj(in,obj[1],&ptr);
  Selection*  start = (Selection*)ptr;			//root (start) selection
  STRING      opt   = Tcl_GetString(obj[2]);		//traverse via attr-values or type-names?
  const char* name  = Tcl_GetString(obj[3]);		//attr-name or type-name
  const char* res   = 0;
  DirectionFilter* filt = 0;
  
  if (opt=="attr")					//make a traversal-filter via attr-names & values
  {  
     const char* val = Tcl_GetString(obj[4]);
     filt = new AttrFilter(name,Attr(val),Node::IN);
     res  = Tcl_GetString(obj[5]);
  }
  else if (opt=="type")					//make a traversal-filter via type-names
  {							//The filter goes 'inside', i.e. we find the roots and
     filt = new TypeFilter(name,Node::IN);		//not the leaves
     res  = Tcl_GetString(obj[4]);	  
  }
  else return TCL_ERROR;

  Selection* res_sel = 0;
  SelectSetOp::roots(start,filt,res_sel);		//put the roots of 'start' in 'res_sel'
  res_sel->ref(res);					//rename selection to given name 'res'
  delete filt;						//done with filter
  return TCL_OK;
}


int pr_selection_op(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[])
{
   STRING         op  = Tcl_GetString(obj[1]);
   SelectionList* sl  = GraphManager::getInstance()->getSelList();
   Selection*     res = 0;
   if (op=="reunion" || op=="intersect" || op=="edges" || op=="difference")
   {
      Selection* s1  = sl->getSelection(Tcl_GetString(obj[2]));
      Selection* s2  = sl->getSelection(Tcl_GetString(obj[3]));
      
      if      (op=="reunion")    SelectSetOp::reunion(s1,s2,res);
      else if (op=="intersect")  SelectSetOp::intersect(s1,s2,res);
      else if (op=="difference") SelectSetOp::difference(s1,s2,res);
      else if (op=="edges")      SelectSetOp::edges(s1,s2,res);
      res->ref(Tcl_GetString(obj[4]));
   }
   else return TCL_ERROR;
   return TCL_OK;
}

int pr_cluster(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj[]) 
{
  const char* sel_name = Tcl_GetString(obj[1]);
  Selection*  sel      = GraphManager::getInstance()->getSelList()->getSelection(sel_name);
  
  Node* parent = 0;
  BOOL ok = EditOp::cluster(sel,parent);
  if (ok) Tcl_SetObjResult(in,Tcl_NewLongObj((long)(unsigned long)parent));
  return (ok)? TCL_OK : TCL_ERROR;
}


//---- Glyph construction procedures -----------------------------------

int pr_make_cube(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj [])
{
   char* args[20]; int i;					//get Tcl args	
   for(i=1;i<n;i++) args[i-1]=Tcl_GetString(obj[i]); 
   args[i-1]=0;
								//make IV cube glyph
   Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)IVGlyphs::cube(args));
   Tcl_SetObjResult(in,res);				
   return TCL_OK;
}

int pr_make_sphere(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj [])
{
   char* args[20]; int i;					//get Tcl args
   for(i=1;i<n;i++) args[i-1]=Tcl_GetString(obj[i]);
   args[i-1]=0;
								//make IV sphere glyph
   Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)IVGlyphs::sphere(args));
   Tcl_SetObjResult(in,res);				
   return TCL_OK;
}

int pr_make_line(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj [])
{
   char* args[20]; int i;					//get Tcl args
   for(i=1;i<n;i++) args[i-1]=Tcl_GetString(obj[i]);
   args[i-1]=0;
								//make IV line glyph
   Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)IVGlyphs::line(args));
   Tcl_SetObjResult(in,res);				
   return TCL_OK;
}

int pr_make_file(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj [])
{
   char* args[20]; int i;					//get Tcl args
   for(i=1;i<n;i++) args[i-1]=Tcl_GetString(obj[i]);
   args[i-1]=0;
								//make IV file glyph
   Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)IVGlyphs::file(args));
   Tcl_SetObjResult(in,res);				
   return TCL_OK;
}

int pr_make_bar(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj [])
{
   char* args[100]; int i;					//get Tcl args	
   for(i=1;i<n;i++) args[i-1]=Tcl_GetString(obj[i]); 
   args[i-1]=0;
								//make IV cube glyph
   Tcl_Obj* res = Tcl_NewLongObj((long)(unsigned long)IVGlyphs::bar(args));
   Tcl_SetObjResult(in,res);				
   return TCL_OK;
}


int pr_bbox(ClientData cd,Tcl_Interp* in,int n,Tcl_Obj* CONST obj [])
{
   if (n!=3) return TCL_ERROR;
   const float MIN_SIZE  = 1.0e-3f;
   const float MIN_RATIO = 0.2f;

   long ptr; float xm,xM,ym,yM; 
   Tcl_GetLongFromObj(in,obj[1],&ptr);	
   Selection* sel = (Selection*)ptr;
   const char* pos = Tcl_GetString(obj[2]);

   Geometry::bbox2D(sel,pos,xm,ym,xM,yM);
   if (xM-xm<MIN_SIZE) xM = xm + (yM-ym)*MIN_RATIO;
   else
   if (yM-ym<MIN_SIZE) yM = ym + (xM-xm)*MIN_RATIO;

   Tcl_Obj* res = Tcl_NewListObj(0,0);
   Tcl_ListObjAppendElement(in,res,Tcl_NewDoubleObj(xm));
   Tcl_ListObjAppendElement(in,res,Tcl_NewDoubleObj(ym));
   Tcl_ListObjAppendElement(in,res,Tcl_NewDoubleObj(xM-xm));
   Tcl_ListObjAppendElement(in,res,Tcl_NewDoubleObj(yM-ym));

   Tcl_SetObjResult(in,res);				
   return TCL_OK;
}

   








//----------------------------------------------------------------------------------------

void pr_initialize(Tcl_Interp* interp)
{
   Tcl_CreateObjCommand(interp,"pr_reset",pr_reset,0,0);
   Tcl_CreateObjCommand(interp,"pr_traverse_tree",pr_traverse_tree,0,0);
   Tcl_CreateObjCommand(interp,"pr_traverse_dag",pr_traverse_dag,0,0);
   Tcl_CreateObjCommand(interp,"pr_selection_op",pr_selection_op,0,0);
   Tcl_CreateObjCommand(interp,"pr_selection_roots",pr_selection_roots,0,0);
   Tcl_CreateObjCommand(interp,"pr_compute_level",pr_compute_level,0,0);
   Tcl_CreateObjCommand(interp,"pr_exit",pr_exit,0,0);
   Tcl_CreateObjCommand(interp,"pr_read_sel_file",pr_read_sel_file,0,0);
   Tcl_CreateObjCommand(interp,"pr_read_graph_ed",pr_read_graph_ed,0,0);
   Tcl_CreateObjCommand(interp,"pr_read_rsf",pr_read_rsf,0,0);
   Tcl_CreateObjCommand(interp,"pr_read_own_format",pr_read_own_format,0,0);
   Tcl_CreateObjCommand(interp,"pr_read_tcl",pr_read_tcl,0,0);
   Tcl_CreateObjCommand(interp,"pr_write_own_format",pr_write_own_format,0,0);
   Tcl_CreateObjCommand(interp,"pr_write_session",pr_write_session,0,0);
   Tcl_CreateObjCommand(interp,"pr_write_gv",pr_write_gv,0,0);
   Tcl_CreateObjCommand(interp,"pr_write_chaco",pr_write_chaco,0,0);
   Tcl_CreateObjCommand(interp,"pr_write_graphed",pr_write_graphed,0,0);
   Tcl_CreateObjCommand(interp,"pr_write_rsf",pr_write_rsf,0,0);
   Tcl_CreateObjCommand(interp,"pr_compo_lib",pr_compo_lib,0,0);
   Tcl_CreateObjCommand(interp,"pr_sel_new",pr_sel_new,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay",pr_lay,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay_grid",pr_lay_grid,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay_dot",pr_lay_dot,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay_neato",pr_lay_neato,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay_gem",pr_lay_gem,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay_level",pr_lay_level,0,0);
   Tcl_CreateObjCommand(interp,"pr_lay_nested",pr_lay_nested,0,0);
   Tcl_CreateObjCommand(interp,"pr_gem_init",pr_gem_init,0,0);
   Tcl_CreateObjCommand(interp,"pr_gv_init",pr_gv_init,0,0);
   Tcl_CreateObjCommand(interp,"pr_glyphs",pr_glyphs,0,0);
   Tcl_CreateObjCommand(interp,"pr_glyph_factory",pr_glyph_factory,0,0);	
   Tcl_CreateObjCommand(interp,"pr_view_new",pr_view_new,0,0);
   Tcl_CreateObjCommand(interp,"pr_num_views",pr_num_views,0,0);
   Tcl_CreateObjCommand(interp,"pr_get_view",pr_get_view,0,0);
   Tcl_CreateObjCommand(interp,"pr_view",pr_view,0,0);
   Tcl_CreateObjCommand(interp,"pr_domain_load_rsf",pr_domain_load_rsf,0,0);
   Tcl_CreateObjCommand(interp,"pr_domain",pr_domain,0,0);
   Tcl_CreateObjCommand(interp,"pr_num_selections",pr_num_selections,0,0);
   Tcl_CreateObjCommand(interp,"pr_get_selection",pr_get_selection,0,0);
   Tcl_CreateObjCommand(interp,"pr_selection",pr_selection,0,0);
   Tcl_CreateObjCommand(interp,"pr_iterator",pr_iterator,0,0);
   Tcl_CreateObjCommand(interp,"pr_get_node",pr_get_node,0,0);
   Tcl_CreateObjCommand(interp,"pr_node",pr_node,0,0);
   Tcl_CreateObjCommand(interp,"pr_get_edge",pr_get_edge,0,0);
   Tcl_CreateObjCommand(interp,"pr_edge",pr_edge,0,0);
   Tcl_CreateObjCommand(interp,"pr_register",pr_register,0,0);
   Tcl_CreateObjCommand(interp,"pr_cluster",pr_cluster,0,0);
   Tcl_CreateObjCommand(interp,"pr_config",pr_config,0,0);
   Tcl_CreateObjCommand(interp,"pr_qualify",pr_qualify,0,0);
   
   Tcl_CreateObjCommand(interp,"pr_add_observer",pr_add_observer,0,0);
   Tcl_CreateObjCommand(interp,"pr_del_observer",pr_del_observer,0,0);

   Tcl_CreateObjCommand(interp,"pr_make_cube",pr_make_cube,0,0);
   Tcl_CreateObjCommand(interp,"pr_make_sphere",pr_make_sphere,0,0);
   Tcl_CreateObjCommand(interp,"pr_make_line",pr_make_line,0,0);
   Tcl_CreateObjCommand(interp,"pr_make_file",pr_make_file,0,0);
   Tcl_CreateObjCommand(interp,"pr_make_bar",pr_make_bar,0,0);
   Tcl_CreateObjCommand(interp,"pr_bbox",pr_bbox,0,0);
}



//---------  Option-procesing functions  ---------------------------------

void option_layout_bbox(Tcl_Interp* in, int& n, Tcl_Obj* CONST * &obj, LayoutOp& l)
{
     double ox,oy,dx,dy;
     Tcl_GetDoubleFromObj(in,obj[0],&ox); obj++;n--;
     Tcl_GetDoubleFromObj(in,obj[0],&oy); obj++;n--;
     Tcl_GetDoubleFromObj(in,obj[0],&dx); obj++;n--;
     Tcl_GetDoubleFromObj(in,obj[0],&dy); obj++;n--;
     l.setBoundingBox((float)ox,(float)oy,(float)dx,(float)dy);
}



