#include "TreeTraversal.h"
#include "SelList.h"
#include "Node.h"
#include "Edge.h"



TreeTraversal::TreeTraversal(Selection* in,Selection* g,Filter* f)
	      :filter(f),all(g)
{  
   Selection::NodeIterator* i = in->iterNodes();
   for(Node* n;(n=i->next());)
      root.Add(n);
}


Node* TreeTraversal::NodeIterator::next()		//Node-iterator
{
   if (!q.Count()) { delete this; return 0; }		//no nodes in queue? Iterator is done.
   Node* n = q.Get();					//get 1st node in queue - this is to be returned

   EdgeSet es = n->getEdges(tt->all,tt->filter);
   for(int i=0;i<es.Count();i++)			//put 'children' of this in queue (if not there)
   {
      Node* child = es[i]->otherNode(n);
      if (q.Contains(child)==-1) q.Put(child);
   }

   return n;
}



Edge* TreeTraversal::EdgeIterator::next()		//Edge-iterator	
{
    for(;;)
    {
      if (!q.Count()) { delete this; return 0; }
      if (idx<edge_set.Count())				//Still some not-traversed edges of the 'current' node?
         return edge_set[idx++];			//If so, return the next one.

      Node* n  = q.Get();				//Get next node and its edge-set. We'll get edges from here now
      edge_set = n->getEdges(tt->all,tt->filter);
      idx = 0;

      for(int i=0;i<edge_set.Count();i++)		//Since we got a new node, put all its not yet discovered	
      {							//children in the queue
        Node* child = edge_set[i]->otherNode(n);
        if (q.Contains(child)==-1) q.Put(child);
      }
    }	
}


TreeTraversal::NodeIterator::NodeIterator(DARRAY<Node*>& n,TreeTraversal* tt_): tt(tt_)
{  for(int i=0;i<n.Count();i++) q.Put(n[i]);  }

TreeTraversal::EdgeIterator::EdgeIterator(DARRAY<Node*>& n,TreeTraversal* tt_): tt(tt_), idx(0)
{  for(int i=0;i<n.Count();i++) q.Put(n[i]);  }

