#ifndef PR_DOMAIN_MODEL    
#define PR_DOMAIN_MODEL

//   DomainModel:	A DomainModel is a collection of named node and arc AttrSetTypes. The client of DomainModel
//			can thus query the various 'types' in a DomainModel and reason about them, as needed.
//			A DomainModel always contains the AttrSetType named "", for the 'unknown' node/arc type.
//			This attr-set-type contains no attr-types and is always the first AttrSetType this holds.
//			A DomainModel is thus a kind of type-table.
//			Specific implementations of DomainModels take care of specific ways to 
//			construct DomainModels and, possibly, specific info that they have/offer.
//			The AttrSetTypes are similar to record-declarations of named & typed fields.


#include "Genrl/string.h"
#include <map>
#include <stdio.h>
using namespace std;


class   AttrSetType;

class	DomainModel                     //Abstract baseclass
	{
	public:
	
	virtual	       		       ~DomainModel();				//Dtor
	const STRING&			getName() const;			//Get this's name	
	const STRING&                   getDomainFile() const;                  //Get file this was loaded from (or "" if none was mentioned)
        const AttrSetType*		getNodeType(const char*) const;		//Get node attr-set-type by type name 
										//(else returns the unknown-type)
	const AttrSetType*		getNodeType(int) const;			//Iterator for node attr-set-types
	int				numNodeTypes()   const;			//Iterator for node attr-set-types
	const AttrSetType*		getArcType(const char*) const;		//Get arc attr-set-type	by type name
										//(else returns the unknown type)
	const AttrSetType*		getArcType(int) const;			//Iterator for arc attr-set-types
	int				numArcTypes() const;			//
	const AttrSetType*		getUnknownNodeType() const;		//Get unknown node type
	const AttrSetType*		getUnknownArcType() const;		//Get unknown arc type
	const AttrSetType*		getUnknownType() const;			//Get unknown type (no distinction between arc or node)
        virtual void                    save(FILE*);                            //Save cmd to load this to given Tcl-file. Def=nop.
        void                            setDomainFile(const char*);             //Specify which file this was loaded from
		
	protected:

					DomainModel(const char*);                //Ctor (name)
	void				addNodeType(AttrSetType*);		 //Add new node AttrSetType to this.
										 //If AttrSetType with same name exists, it is deleted.
	void				addArcType(AttrSetType*);		 //Add new node AttrSetType to this.
										 //If AttrSetType with same name exists, it is deleted.
 	
	
	map<STRING,AttrSetType*>	node_types,arc_types;			//All node/arc attr-set-types in this
	STRING				name;					//This's name
        STRING                          domain_file;                            //File this domain is loaded from (def="", i.e. none)
	AttrSetType		       *node_unknown,*arc_unknown;		//The unknown, node/arc ""-type 
										//(also stored in node/arc_types[])
	
	};




inline const STRING& DomainModel::getName() const
{  return name;  }

inline const AttrSetType* DomainModel::getUnknownNodeType() const
{  return node_unknown;  }

inline const AttrSetType* DomainModel::getUnknownArcType() const
{  return arc_unknown;  }

inline const AttrSetType* DomainModel::getUnknownType() const
{  return node_unknown;  }

inline const STRING& DomainModel::getDomainFile() const
{  return domain_file;  }

inline void DomainModel::setDomainFile(const char* f)
{  domain_file = f;  }

#endif

