#ifndef PR_EDGE_H
#define PR_EDGE_H

//  Edge:	Models edges in a Graph. An edge has always exactly two Nodes at its end. It is also directed.
//		The 2nd important property of an Edge is whether it is a containment-edge or an
//		association-edge. This is actually an attribute. However, we implement it like this,
//		and not as an attribute, for speed reasons and because many clients need to know they can
//		set/get this attribute via the same interface (name), and not via some string-name...
//
//


#include "Element.h"
#include "Genrl/types.h"
#include "Node.h"


//    Edge:     Models a relationship between two Nodes in a graph. The graph-context is given by a
//              Selection the Ege belongs to. Several Edges may thus exist between two given Nodes, in
//              the same selection-ctx of in different ones.
//

class Node;
class AttrSetType;

class Edge : public Element
		{
		public:
			void	 destroy();
			Node* 	 fromNode();			
			Node* 	 toNode();
			Node* 	 otherNode(Node*);
			Node*	 node(Node::DIRECTION);					
			BOOL  	 uses(Node*) const;
			void	 addToSelection(Selection*);				//enh inh
			void	 delFromSelection(Selection*);				//enh inh

		protected:
		
			      	Edge(Selection*,Node* from,Node* to,const AttrSetType*,int id=-1);	
											//For Node
			       ~Edge();	
			void	nodeRef();						//For Node
			void    nodeUnref();						//For Node

		   friend class Node;			
			     
		private:

			void    delFromNodes(Selection* s);				//Release this from BOTH its nodes in ctx 's'
			Node    *from,*to;
			BOOL    in_dtor;
			int     node_refcount;
		}; 	   	  	

inline Node* Edge::fromNode()
{  return from;  }

inline Node* Edge::toNode()
{  return to;  }

inline BOOL Edge::uses(Node* n) const
{  return (from==n || to==n);  }

inline Node* Edge::otherNode(Node* n)
{  return (n==to)? from : (n==from)? to : 0;  }

inline void Edge::nodeRef()
{  node_refcount++;  }

inline void Edge::nodeUnref()
{  node_refcount--;  }
 
#endif
