#ifndef PR_ELEMENT_H
#define PR_ELEMENT_H

//	Element:		Superclass for all 'elements' that can be members of a selection.
//				Basically these are Nodes and Edges.
//				This class is not intended to be used by the public directly
//
//

#include "AttrSet.h"

class Selection;



class   AttrSetType;

class	Element : public AttrSet        //was public GVObject, public Selection
	{
	public:

	virtual void		delFromSelection(Selection*);	//this marks in itself that it doesn't belong to arg any more
								//(subclasses may enhance it)
	
				Element(const AttrSetType*,int=-1);		
								//Ctor (type,optional id)
								//If no type give, unknown-type of current DomainModel is used
                               ~Element();                      //Dtor  
	virtual void		addToSelection(Selection*);	//this marks in itself that it belongs to arg
	int			numSelections() const;		//iterator over selections that this belongs to
	Selection*		getSelection(int);		//iterator over selections that this belongs to	
	int			getId() const;			//get unique numeric id
	BOOL			containedIn(Selection*) const;
								//tells if this is already in given Selection
        static int              getFreeId();                    //gets the 1st free-id this has. All ids >= this one are free.
                                                                //Useful for clients that set ids and need to allocate ids in advance
	friend class		Selection;			//for delFromSelection()	
	
	protected:
	
	DARRAY<Selection*,5>	sels;				//selections that contain this
	int			id;				//unique numeric id
	int			refcount;			//reference count
	
	private:
	
	int			makeId(int);			//generate unique numeric ids 
        static int              num_insts;                      //total #insts created, used for recycling ids
        static int              crt_id;                         //1st free id, all ids above this are unused
	};
	


inline Element::Element(const AttrSetType* ast,int id_): AttrSet(ast), id(makeId(id_)), refcount(0)
{  num_insts++; }

inline int Element::numSelections() const
{  return sels.Count();  }

inline Selection* Element::getSelection(int i)
{  return sels[i];  }

inline int Element::getId() const
{  return id;  }

inline BOOL Element::containedIn(Selection* s) const
{  return sels.Contains(s)!=-1; }

inline int Element::getFreeId()
{  return crt_id;  }

#endif

