/* ARRAY.H 	Defines and implements a very simple array-template class */
/*		Over a standard array, this one remembers the #elems stored within */ 


#ifndef ARRAY_H
#define ARRAY_H

#include <iostream.h>
#include <stdlib.h>

template <class T> class ARRAY
	{
	public:
	   inline 		ARRAY(int=0);		//CONSTR: alloc a maxsize array	
				ARRAY(const ARRAY<T>&); //CONSTR: copy ctor
	   inline 	       ~ARRAY();		//DESTR: releases (eventually) alloc'd memo 
	   inline void 		Init(int);  		//INIT: realloc an array
	   inline void 		InitC(int); 		//INIT: calls Init + SetCount
	   inline void 		Add(T);			//Add elem at array's end,incr count
	   inline T 		Delete(int);		//Delete ith element, decr count, shift array	
	   inline int 		Remove(T);		//Search and remove 1st occurence of T.
							//Return pos where T was found or -1 if not found
	   void 		Append(T ptr);		//Like Add(), if elem NOT in array
	   inline int  		Count()		 const;	//Return #elems in array
	   int  		Contains(T)      const; //Check if T is in array
	   inline int  		Size()           const;	//Returns the # allocated elems
	   inline T&   		operator[](int);    	//Return ref to array[i] (for R/W)
	   inline const T&	operator[](int)	 const; //Const version of above
	   ARRAY<T>&   		operator=(T);		//Set array to a value  
	   ARRAY<T>&   		operator=(const ARRAY<T>&);//Assignment of arrays
	   inline void 		Flush();		//Empty the array
	   inline void 		SetCount(int);		//Set array counter (NOT recommended!)
	   
	protected:
	   T* 			array;
	   int 			count,size;		
	};

template <class T> inline ARRAY<T>::ARRAY(int Maxsize):array(new T[Maxsize]),count(0),size(Maxsize)
{  }

template <class T> inline ARRAY<T>::ARRAY(const ARRAY<T>& a): array(new T[a.size]),count(a.count),size(a.size)
{
  const T* p = a.array;
  for(int i=0;i<size;i++) array[i] = p[i];
}

template<class T> inline ARRAY<T>::~ARRAY()
{  delete[] array;  }

template <class T> inline void ARRAY<T>::Init(int Maxsize)
{
  delete[] array;
  array = new T[size=Maxsize]; 
  count = 0;
}


template <class T> inline int ARRAY<T>::Size() const
{  return size;  }

template<class T> inline void  ARRAY<T>::Add(T ptr)
{  array[count++] = ptr;  }

template <class T> inline T ARRAY<T>::Delete(int idx)
{
  T ret = array[idx];
  count--;
  for (;idx<count;idx++)  array[idx] = array[idx+1];
  return ret;
}

template <class T> inline int ARRAY<T>::Remove(T x)
{
  int i,ret; 
  if (!count) return -1;		//empty array, remove fails

  for(i=0;i<count && array[i]!=x;i++);  //search for x
  if (i==count) return -1;		//x not found, remove fails
  ret = i; 				//x found at position i
  count--;				//one less element
  for(;i<count;i++) array[i] = array[i+1];
  					//SHL array from i to end	
  return ret;				//return pos of removed element
}

template<class T> inline int ARRAY<T>::Count() const
{  return count;  }

template<class T> inline void ARRAY<T>::SetCount(int c)
{  count = c;  }

template<class T> inline void ARRAY<T>::InitC(int maxsize)
{
  Init(maxsize);
  SetCount(maxsize);
}

template<class T> int ARRAY<T>::Contains(T t) const
{
  for(int i=0;i<count;i++)
    if (array[i]==t) return i; 
  return -1;
}

template<class T> void ARRAY<T>::Append(T ptr)
{
  for(int i=0;i<count;i++)
    if (array[i]==ptr) return;
  Add(ptr);
}  

template<class T> inline T& ARRAY<T>::operator[](int i)
{  return array[i];  }

template<class T> inline const T& ARRAY<T>::operator[](int i) const
{  return array[i];  }

template<class T> ARRAY<T>& ARRAY<T>::operator=(T val)
{
  T* end = array+size;
  for(T* i=array;i<end;) *i++ = val;
  return *this;
}

template<class T> ARRAY<T>& ARRAY<T>::operator=(const ARRAY<T>& a)
{
  if (&a == this) return *this;
  if (size!=a.size)
  {
    delete[] array; 
    array = new T[size=a.size];
  }

  T*  av = a.array;
  T* end = array+size; 
  for(T* i=array;i<end;) *i++ = *av++;

  count = a.count;
  return *this;
}


template<class T> inline void ARRAY<T>::Flush()
{
  count = 0;
}

#endif
