/* LIST.H:    Simple linked list template class and iterator. */

#ifndef LIST_H
#define LIST_H

#include <stdlib.h>

template <class T> class LISTITER;		// A list-iterator class (see below)

template <class T> class LIST
		{
		       public:
		       	      class       Iterator;			//List iterator
		       
			      int  	  count() const;      		//Returns #elems in list 
			                  LIST();	    		//Ctor
			      	         ~LIST();    	    		//Dtor
			      void        flush();    	    		//Flushes list-cells
			      void        insertEnd(const T&);  	//Inserts x at end
			      void        insertFront(const T&);	//Inserts x at start
			      int         contains(const T&);   	//Returns 1 if x in list, else 0
			      int         remove(const T&);		//Searches and removes elem x. Returns 1 if removed else 0
			      
		       private:	      
		       
			      struct CELL
			      {
		  	         T    	     x;
		  	         struct CELL* next;
				 CELL(const T& y,CELL* ptr=0): x(y),next(ptr) {}
			      };
			      CELL* head;


		       public:
		       	
			      class Iterator				//List iterator
			      {
			      public:
		  			Iterator(LIST<T>& l): crt(l.head) {}
		  		T 	next()	      		{ 
		  						   if (!crt) return 0;
		  						   CELL* prev=crt;
		  						   crt=crt->next;
		  						   return prev->x;
		  						}  
				T	value()			{  return (crt)? crt->x : 0; }
			      private:
		  
		  		CELL *crt;
			      };

		       private:
		       
			      CELL* 	   tail;
			      int   	   cnt;
			      friend class Iterator;
		 };


template <class T> inline LIST<T>::LIST():head(0),tail(0),cnt(0)
{  }

template <class T> LIST<T>::~LIST()
{
  while (head)
  {
    CELL* tmp = head;
    head      = head->next;
    delete tmp;
  }
}


template <class T> inline int LIST<T>::count() const
{
  return cnt;
}


template <class T> inline void LIST<T>::insertEnd(const T& x)
{
  CELL* tmp = new CELL(x);
  
  if (!cnt)
    tail = head = tmp;
  else
  {
    tail->next = tmp;
    tail       = tmp;
  }

  cnt++;
}

template <class T> inline void LIST<T>::insertFront(const T& x)
{
  CELL* tmp = new CELL(x); 
  
  if (!cnt)
    tail = head = tmp; 
  else
  {
    tmp->next = head;
    head      = tmp;
  }

  cnt++;
}

template <class T> void LIST<T>::flush()
{
  while(head)
  {
    CELL* tmp = head;
    head      = head->next;
    delete tmp;
  }
  cnt = 0; tail = 0;
}

template <class T> int LIST<T>::contains(const T& x)
{
  for(CELL* tmp=head;tmp;tmp=tmp->next)
    if (tmp->x==x) return 1;
  return 0;
}   

template <class T> int LIST<T>::remove(const T& x)
{
   CELL  *tmp,*p; 
   if (head)
   {
     if (head->x==x)
     {
       tmp = head; head = head->next;
       if (!head) tail = 0;
     }
     else
     {
       for(p=head,tmp=head->next;tmp && tmp->x!=x;tmp=tmp->next) p = tmp;
       if (!tmp) return 0;
       p->next = tmp->next;
       if (tail==tmp) tail=p;
     }

     cnt--;
     delete tmp;
     return 1;
   }    
   return 0;
}   

#endif


