#ifndef MATRIX_H
#define MATRIX_H

// MATRIX:		Ctor creates an empty matrix. User has the responsibility
//			to dimension it, by calling Init(int) to alloc the # rows and
//			then Init(int,int) to alloc each row separately.
//			The class maintains a correct Size(int) and Size(int,int), so the
//			user can check whether the respective row(s) are allocated or not. 

class MATRIX 	{
		public:
		 
		   	    MATRIX();			//constr empty matrix
			   ~MATRIX();			//destr matrix	
		   double&  operator()(int,int);	//reference to elem (i,j)
		   const
		   double&  operator()(int,int) const;	//const reference to elem (i,j)
		   double*  operator()(int);		//reference to row i's first elem
		   const
		   double*  operator()(int) const;	//const reference to row i's first elem
		   void     Init(int);			//alloc # rows
		   void     Init(int j,int sz);		//alloc row j of length sz
		   int      Size(int) const;		//size of a row 
		   int      Size() const;		//# rows	
		   const
		   MATRIX&  operator=(const MATRIX&);   //assignment operator
		protected:
		
		   double** 	m;			//the matrix
		   int* 	szv;			//sizes of all rows
		   int		sz;			//# rows
		};

/////////////////////////////////////////////////////////////////////


inline MATRIX::MATRIX() 
{
   m = NULL; szv = NULL; sz = 0;		//empty matrix
}

inline double& MATRIX::operator()(int i,int j) 
{ 
   return m[i][j]; 
}

inline const double& MATRIX::operator()(int i,int j) const
{ 
   return m[i][j]; 
}

inline double* MATRIX::operator()(int i)
{ 
   return m[i]; 
}

inline const double* MATRIX::operator()(int i) const
{ 
   return m[i]; 
}	

inline int MATRIX::Size(int i) const
{ 
   return szv[i]; 	
}

inline int MATRIX::Size() const		    
{ 
   return sz; 
}


inline void MATRIX::Init(int size)
{
   if (size==sz) return;			//nothing to do: same size
   for(int i=0;i<sz;i++) delete[] m[i];		//delete all rows 
   delete[] m; delete[] szv;			//delete matrix

   m   = new double*[sz=size];			//alloc matrix
   szv = new int[sz];				//alloc size-vec
   for(i=0;i<sz;i++) { szv[i] = 0; m[i] = NULL; }
}

inline MATRIX::~MATRIX()
{
   for(int i=0;i<sz;i++) delete[] m[i];		//delete rows
   delete[] m; delete[] szv;			//delete matrix and size-vec
}

inline void MATRIX::Init(int i,int size)
{
   if (szv[i]==size) return;			//nothing to do: same size
   delete[] m[i];				//delete row
   m[i] = new double[szv[i]=size];		//(re)alloc row
}

inline const MATRIX& operator=(const MATRIX& mat)
{
   Init(size);					//realloc this matrix, if necessary
   for(int i=0;i<sz;i++) Init(i,mat.szv[i]);
   
   for(int i=0;i<sz;i++)			//copy all the row-vectors into this
   {
     double* ap = m[i];
     double* as = m[i]+szv[i];
     double* av = mat.m[i];
     for(; ap < as; ) *ap++ = *av++;
   }	
   return *this;
}

#endif

