#ifndef PR_GRAPH_MANAGER_H
#define PR_GRAPH_MANAGER_H

//	GraphManager:	The central manager of the system's data structures. This class owns and manages
//                      all subsystems. These are the following:
//			- a set of named Selections (SelectionList)
//			- the DomainModel (optional custom + default one). Only one DomainModel can be
//			  active at a time.
//                      - the set of named Observers
//                      - the system-wide Registry
//                      - the ComponentLibrarySet for libs of Tcl-based components 
//                      - the GlyphFactoryList
//			
//			Various system-wide editing operations go via this class, to ensure 
//			that data is maintained consistent.
//
//
//


class Node;
class Edge;
class DomainModel;
class SelectionList;
class Selection;
class ObserverList;
class IVGlyphFactoryList;
class ComponentLibrarySet;
class Registry;

#include "Genrl/types.h"
#include "Genrl/darray.h"

class WriteOperation; 

class GraphManager	 
		  {
		  public:
		  
		  static void   	init();
		  static void		destr();
		  static GraphManager*  getInstance();		//QUERY METHODS
		  SelectionList*	getSelList();		//get global SelectionList
		  ObserverList*		getObsList();		//get global ObserverList
		  IVGlyphFactoryList*	getGlyphFactoryList();	//get global IVGlyphFactoryList
		  Registry*		getRegistry();		//get global Registry
                  ComponentLibrarySet*  getCompoLibSet();       //get global ComponentLibrarySet
		  void			setDomainModel(DomainModel*);
		  						//set global DomainModel
		  DomainModel*		getDomainModel();	//get global DomainModel
		  void			reset();		//clear/destroy all data structures managed by this
		  void			cleanup();              //
                  void                  save(const char*);      //save entire system state to given Tcl-file
				  	 			  
		  private:
		  		  	GraphManager();
		  		       ~GraphManager();	
		   
		  SelectionList*  	sellist;
		  ObserverList*		obslist;
		  IVGlyphFactoryList*	glyphlist;
		  DomainModel*		dmodel;
		  DomainModel*		def_dmodel;
		  Registry*		registry;
                  ComponentLibrarySet*  libs;
		  	
		  static GraphManager*  instance;
		  };


inline void GraphManager::init()
{  if (!instance) instance = new GraphManager();  }

inline GraphManager* GraphManager::getInstance()
{  return instance;  }      

inline SelectionList* GraphManager::getSelList()
{  return sellist;  }

inline ObserverList* GraphManager::getObsList()
{  return obslist;  }

inline IVGlyphFactoryList* GraphManager::getGlyphFactoryList()
{  return glyphlist;  }

inline DomainModel* GraphManager::getDomainModel()
{  return (dmodel)? dmodel : def_dmodel;  }

inline Registry* GraphManager::getRegistry()
{  return registry;  }

inline ComponentLibrarySet* GraphManager::getCompoLibSet()
{  return libs;  }

inline void GraphManager::destr()
{   if (instance) { delete instance; instance=0; } }
   

#endif
