#ifndef PR_IV_GLYPH_FACTORY_H
#define PR_IV_GLYPH_FACTORY_H

//	IVGlyphFactory:	Encodes a mapping of a single Node and Edge to an Inventor SoNode.
//			Subclasses implement concrete ways of constructing this SoNode.
//                      All IVGlyphFactories are managed by IVGlyphFactoryList.
//


#include "Attr.h"
#include "GVObject.h"
#include "Genrl/darray.h"
#include "Genrl/string.h"
#include "Genrl/types.h"
#include "Genrl/rtti.h"
#include <set>

class	Node;
class	Edge;
class   Selection;
class	SoSeparator;
class	SoNode;
class	IVGlyphFactoryList;



class	IVGlyphFactory : public GVObject			//Generic glyph-factory baseclass
	{							//(these are named objects)
	public:								
	
	virtual	       ~IVGlyphFactory();
        virtual void    save(FILE*);                            //Save settings of this to Tcl-file (enhanced by subclasses)
        virtual void    getTclDeps(std::set<STRING>&);          //Get all Tcl-procs this uses (calls) (enhanced by subclasses)
	
	protected:
			IVGlyphFactory(const char* n):GVObject(n) {}	//Ctor (name)
	friend class	IVGlyphFactoryList;			//for renaming this

	RT_TYPE_INFO("IVGlyphFactory")				//RTTI
	};



class	IVNodeGlyphFactory : public IVGlyphFactory		//Factory for node-glyphs made out of individual nodes
	{
	public:	
	virtual SoNode*	map(Node*) =0;				//Main method called by clients to map a Node to an SoNode
	protected:
			IVNodeGlyphFactory(const char* n):IVGlyphFactory(n){}
	RT_TYPE_INFO("IVNodeGlyphFactory")			//RTTI		
	};



class	IVEdgeGlyphFactory : public IVGlyphFactory		//Factory for edge-glyphs made out of individual edges	
	{
	public:
	virtual SoNode* map(Edge*,const char*,const char*,const char*) =0;
								//Make a glyph for given edge. This may use the position-attrs
								//named by the last 3 args.		
	protected:
			IVEdgeGlyphFactory(const char* n):IVGlyphFactory(n){}	
	RT_TYPE_INFO("IVEdgeGlyphFactory")			//RTTI	
	};



class	IVTclGlyphFactory					//Baseclass for all Tcl-callback based glyph factories
	{
	public:
	
			IVTclGlyphFactory();
	void		setTclCallback(const char*);
	const STRING&	getTclCallback() const;	
        
	protected:
        
        void            save(FILE*,const char*,const char*);            //Save settings of this to Tcl-file (only for subclasses)
        void            getTclDeps(std::set<STRING>&);                  //Get Tcl-procs this uses (calls)
        STRING		tcl_cb;		
	};



class	IVTclNodeGlyphFactory : public IVNodeGlyphFactory, public IVTclGlyphFactory
	{
	public:
			IVTclNodeGlyphFactory(const char* n):IVNodeGlyphFactory(n){}	
	SoNode*		map(Node*);				//impl inh using Tcl cb	
        void            save(FILE*);                            //enh inh
        void            getTclDeps(std::set<STRING>&);          //enh inh
	
	protected:
	RT_TYPE_INFO("IVTclNodeGlyphFactory")			//RTTI	
	};



class	IVTclEdgeGlyphFactory : public IVEdgeGlyphFactory, public IVTclGlyphFactory
	{
	public:
			IVTclEdgeGlyphFactory(const char* n):IVEdgeGlyphFactory(n){}
	SoNode*		map(Edge*,const char*,const char*,const char*);	//impl inh using Tcl cb	
        void            save(FILE*);                            //enh inh
        void            getTclDeps(std::set<STRING>&);          //enh inh
	
	protected:
	RT_TYPE_INFO("IVTclEdgeGlyphFactory")			//RTTI	
	};
	

class	IVBasicNodeGlyphFactory :  public IVNodeGlyphFactory
	{
	public:
			IVBasicNodeGlyphFactory(const char* n):IVNodeGlyphFactory(n){}
	SoNode*		map(Node*);				//impl inh	
	
	protected:
	RT_TYPE_INFO("IVBasicNodeGlyphFactory")			//RTTI	
	};



class	IVBasicEdgeGlyphFactory : public IVEdgeGlyphFactory
	{
	public:
			IVBasicEdgeGlyphFactory(const char* n):IVEdgeGlyphFactory(n){}
	SoNode*		map(Edge*,const char*,const char*,const char*);	//impl inh
	
	protected:
	RT_TYPE_INFO("IVBasicEdgeGlyphFactory")			//RTTI	
	};


//--------  Inline implementations  -------------------------------------------


inline void IVTclGlyphFactory::setTclCallback(const char* cb)
{  tcl_cb = cb;  }

inline const STRING& IVTclGlyphFactory::getTclCallback() const
{ return tcl_cb;  } 


#endif
