#ifndef PR_IV_UTILS_H
#define PR_IV_UTILS_H

//    IVUtils.h:	Various utility functions based on GL/Inventor. 
//
//
//


class SbViewVolume;
class SbVec3f;

#include <Inventor/SbBasic.h>
#include "Genrl/string.h"


SbBool IVViewVolumeIntersect(const SbViewVolume&,const SbVec3f&);
//This is actually an implementation of SbViewVolume::intersect(SbVec3f)
//as present in the original Inventor. Since Coin3D v1.0.2 does not implement
//this method, we provide it here (the code is just the original Inventor code)
//Actually, if we wanted, we could use a #define to switch on the original
//SbViewVolume::intersect() on platforms where we have it, e.g. when we use the
//original Inventor. However, the code is simpler like this.


static inline unsigned int makeRGBA(float r,float g,float b)	//REMARK:  This code detects the RGBA/ABGR
{								//	   byte ordering used by Inventor by testing for
   #ifdef IV_USES_RGBA						//	   the symbol IV_USES_RGBA. We define this symbol ourselves
								//	   in UI_imp.h by testing for a combination of #define's
   return (((unsigned int)(255*r))<<24)+			//	   to determine the Inventor/Coin3D version we are using.
   	  (((unsigned int)(255*g))<<16)+			//	 
	  (((unsigned int)(255*b))<<8) +
	  (((unsigned int)(255*1)));

   #else							//	   Inventor uses ABGR

   return (((unsigned int)(255*1))<<24)+			
   	  (((unsigned int)(255*b))<<16)+
	  (((unsigned int)(255*g))<<8) +
	  (((unsigned int)(255*r)));

   #endif
}

STRING rgb2tkcolor(float,float,float);				//Makes a Tk-compatible #rrggbb color string from a RGB color spec
int    tkcolor2rgb(const char*,float&,float&,float&);		//Extracts the RGB components of a Tk-compatible #rrggbb color spec
								//Returns 1 if string conformant & extraction ok else 0

void rainbowColormap(float,float&,float&,float&);               //Given a value in 0..1, returns its mapping via a 'standard' rainbow
                                                                //colormap as 3 RGB values in 0..1.

void glutBitmapString(float,float,const char*);                 //Draws a string at 2D pos (x,y) using GLUT's bitmap font

int  glutStringLength(const char*);                             //Returns the length, in pixels, the string-arg would have if
                                                                //drawn by glutBitmapString().

int  glutStringHeight(const char*);                             //Returns the height, in pixels, the string-arg would have if drawn.


class   IVZBufferHit                                            //Implements a z-buffer-hit, i.e. a (small) portion of a window's Z-buffer.
        {                                                       //Used for pick-point communication between EventHandler and IVViewer.
        public:
                       IVZBufferHit():zb(0),dx(0),dy(0),is_empty(1) {}
                      ~IVZBufferHit() { delete[] zb; }
        int            dimX() const   { return dx;   }
        int            dimY() const   { return dy;   } 
        float&         operator()(int x,int y) { return zb[y*dx+x]; }
        void           init(int,int,int,int);
        int            empty() const  { return is_empty; }

        private:

        float*         zb;
        int            dx,dy;
        int            is_empty;
        };

#endif

