#ifndef PR_NODE_H
#define PR_NODE_H

//	EdgeSet:	Models a set of Edges. This is an implementation-class for Node.
//			It is more lightweight than a Selection and is usually employed in local
//			operations on a Node's edges.
//	Node:		Models a Node. A Node is an AttrSet plus several EdgeSets, all given
//			in some Selection context.
//

#include "Element.h"
#include "Genrl/types.h"
#include "Genrl/darray.h"
#include <map>			//instead of hash_map


class Edge;
class Node;
class Filter;
class AttrSetType;
class Selection;

class EdgeSet : public DARRAY<Edge*>
		{
		public:
		
				EdgeSet() {}
			BOOL	isEmpty() const { return Count()==0; }
			Edge*	to(Node*);					//Returns an edge to arg in this, if any
			Edge*	from(Node*);	
		};	
		
class NodeSet : public DARRAY<Node*>
		{
		};		
					

class Node : public Element
		{
		public:
			enum DIRECTION  { IN=0, OUT=1, BOTH=2 };
		
		
					Node(Selection*,int=-1,const AttrSetType* =0);		
										//Ctor: make node with given id and type and add it to given Selection
										//(if id given, caller MUST ensure it is unique)
										//If no type is given, the 'unknown' type of the current DomainModel will be used.
			void		destroy();				//Public dtor
		
										//SET/GET METHODS
			EdgeSet&	getEdges(Selection*,DIRECTION=BOTH);	//get all in/out edges in given selection
			const EdgeSet&	getEdges(Selection*,DIRECTION=BOTH) const;
			EdgeSet		getEdges(Selection*,Filter*);		//get edges selected by custom Filter
			Edge*		connectTo(Node*, Selection*,const AttrSetType* =0,int id=-1);		
										//connect this->arg, in given context. Returns newly created Edge.
										//3rd arg specifies the optional type for that edge. If none given,
										//then the unknown-edge-type of the current DomainModel is used.
                                                                                //4th arg specifies the edge-id, if desired.
			void		delFromSelection(Selection*);		//enh inh (ONLY FOR Selection)


		private:

			friend  class     Edge;				//for disconnectIn,disconnectOut
			
			void		  delEdge(Edge*,Selection*);	//
			void		  addInEdge(Edge*,Selection*);
			void		  addOutEdge(Edge*,Selection*);
			void		  addEdge(Edge*,Selection*);
	  		      	         ~Node();			//Dtor: destructs this and all Edges used by this

		
			std::multimap<unsigned long,EdgeSet*>   
					  edges, in_edges, out_edges;	//edges, kept per Selection-object
			BOOL		  in_dtor;			//flag to accelerate dtor
			
			static EdgeSet 	  empty_set;			//return-value in the empty set case
		}; 	   	  	





inline EdgeSet& Node::getEdges(Selection* s,DIRECTION io)
{  
   std::multimap<unsigned long,EdgeSet*>::iterator i = (io==IN) ? in_edges.find((unsigned long)s)  :
   						  (io==OUT)? out_edges.find((unsigned long)s) :
   						  	     edges.find((unsigned long)s);
   return (i!= ((io==IN) ? in_edges.end()  :
                (io==OUT)? out_edges.end() :
                           edges.end()))? *(EdgeSet*)(*i).second : empty_set;
}

inline const EdgeSet& Node::getEdges(Selection* s,DIRECTION io) const
{  return ((Node*)this)->getEdges(s,io);  }

#endif
