#ifndef PR_OBS_LIST_H
#define PR_OBS_LIST_H

//	ObserverList:		Maintains a list of all persistent Observer objects in the system.
//				This class is managed by GraphManager. 
//				Observers (with names) can be added to the list. 
//				ObserverList is managed at a higher level by GraphManager which ensures that
//				observers are updated wheneven selection data changes in the SelectionList.
//				Observers are owned by the ObserverList, so they are destroyed upon removal.
//

#include "Observer.h"
#include "Genrl/darray.h"


class SelectionList;

class ObserverList	 
		  {
		  public:
		  
		  	 	ObserverList(SelectionList*);		       //CONSTR/DESTR 	
		  	       ~ObserverList();			       	       //tor: destroy all observers in this
		  	       						       //
					
									       //QUERY METHODS
		  int		numObservers() const;			       //get #observers in list
		  Observer*	getObserver(int);			       //iterate over observers
		  Observer*     getObserver(const char*);		       //get observer by name (return 0 if none found)

		  							       //MANAGEMENT METHODS
		  void		addObserver(Observer*);		       	       //add new observer to list (destroy any existing observer with same name)
		  void		delObserver(const char*);	               //destroys observer from list (if in list)
		  void		delAllObservers();			       //destroys all non-persistent observers from this
		  void		update();				       //updates all observers
		  void		doUpdates();
		  	
		  private:	
		  
		  DARRAY<Observer*>	obs;
		  SelectionList*	sel;
		  BOOL			dirty;
		  };	 		


inline int ObserverList::numObservers() const
{  return obs.Count();  }

inline Observer* ObserverList::getObserver(int i) 
{  return obs[i];  }


#endif
