#ifndef PR_OPERATION_H
#define PR_OPERATION_H

//	Operation:	Superclass for all operations on Graphs. An Operation has mostly one (static) method, execute().
//			Conceptually an Operation has the following four types of arguments:
//
//			- Graph
//			- Selection names (for the selections it has to use as inputs)
//			- Selection names (for the selections it has to use as outputs)
//			- extra parameters (operation-specific)
//
//			WriteOperations are those operations that modify _anything_ in the Graph/Selections.
//			WriteOperations are the only parts of the system which create/destroy Nodes/Edges/Levels and Selections.
//			For this reason, mechanisms are implemented atop of WriteOperations to ensure that they maintain the
//			system in a consistent state. To ensure this, an operation should always:
//
//			- first call WriteOperation::begin();
//			- do whatever it wants with Graph-parts and/or Selections (read/write/destroy/create)
//			- call WriteOperation::end() upon completion.
//
//			At the operation end, the system will automatically 
//			a) remove all nodes/edges/levels deleted during an operation from the Selections in the selection-list
//			b) add the newly created Selections to the selection-list
//			c) update all Observers so they display the new system state


class   Operation       {
			public:
			
			
			
			
			};	


class   WriteOperation : public Operation
			{
			public:
			
			
			
			protected:
			
			static void	begin();		//for subclasses to call	
			static void	end();			//for subclasses to call

			private:			
			static int	level;			//internally used by begin(),end()
			};
		
			

#endif
