#ifndef PR_OPERATIONS_H
#define PR_OPERATIONS_H

//      This file contains various operations implemented by the toolkit that don't fall into the SelectOp category.
//	That is, we keep here the other operations, as follows:
//
//
//	EDIT 		These operations modify various aspects of the Graph dataset (connectivity,attributes,etc).
//	OPERATIONS: 	Available implementations:
//
//			LoadOp:				  Load graph completely from file. 
//			       ::readGraphEd(f,p):        read from GraphEd file with pos-attrs p, if specified.
//			       ::readOwnFormat(f):	  read from own format. Attrs are read automatically.
//			       ::readRSF(f):		  read from unstructured RSF format. Current dom-model must be a valid RSF model.
//			EditOp:				  Graph editing operations.	
//			       ::renameSelection(s,name): rename selection s to name
//			AttrOp:
//			       ::computeLevel(sel,attr):  compute an integer level-attribute, which is 0 for root of sel, 1 for its children, etc.
//			
//	READ
//	OPERATIONS:	These operations only read the Graph data. They usually export it to other systems in different formats.
//			Available implementations:	
//
//			SaveOp:				   save selection+attrs as file.
//			      ::writeGV(s,f,pos,b1,b2,e):  save sel s in .dot file f. Optionally save geometric attrs 'pos'. If b1, save position.
//							   If b2, save sizes too. e gives an optional edge-length.
//			      ::writeGraphEd(s,f,pos,b1):  save sel s in GraphEd file f. Optionally save geometric attrs 'pos'. If b1, save position.
//			      ::writeOwnFormat(s,f):	   save sel s in own-format file f	
//
//


#include "Operation.h"
#include "Genrl/types.h"
#include "Genrl/string.h"
#include "SelectOp.h"
#include "LayoutOp.h"

class   Selection;
class   Node;
class   Edge;
class   Graph;
class	DirectionFilter;


			
//********  EDIT OPERATIONS  *************************************************

			
class   LoadOp : public WriteOperation
			{
			public:
			        
                                static void  clearAtLoad(BOOL);
                                static BOOL  isClearAtLoad();
                                static void  reset();
				static int   readGraphEd(const char* file,const char* posname=0);
				static int   readOwnFormat(const char* file,Selection* =0);
				static int   readRSF(const char* file);
                        protected:

                                static int   readString(FILE*,char*);
                                static BOOL  clear_at_load;
			};
			
class	EditOp : public WriteOperation			
			{
			public:
				static void  renameSelection(Selection*,const char* newsel);
				static BOOL  cluster(Selection*,Node*&,int id=-1);	//clusters nodes in sel in parent w. given id. 
											//Returns ok or failure and the newly-made parent
				static void  reset();
			};
			
class	AttrOp : public WriteOperation
			{
			public:
			
				static void  computeLevel(Selection* inp,DirectionFilter*,const char* lev_attr);
			};				
			

//********  READ OPERATIONS  ****************************************************


class SaveOp : public Operation
			{
			public:
			
				static int   writeGV(Selection*,const char* file,const char* pos=0,BOOL wr_pos=FALSE,
					             BOOL wr_sz=FALSE,BOOL flip_edges=FALSE,const char* level=0,float esz=-1);
				static int   writeGraphEd(Selection*,const char* file,const char* pos=0,BOOL wr_pos=FALSE,
							  BOOL wr_ids=FALSE);
				static int   writeOwnFormat(Selection*,const char*);
				static int   writeRSF(Selection*,const char*);
                                static int   writeChaco(Selection*,const char*);

                        private:

                                static void   writeRSFNode(FILE*,Node*);
                                static void   writeRSFEdge(FILE*,Edge*,Selection*);
                                static STRING getRSFNodeName(Node*);
			};




#endif
