#ifndef PR_PATH_QUALIFIER_H
#define PR_PATH_QUALIFIER_H

//      PathQualifier:          Provides system-wide path qualification services.
//                              Useful when a component is given a filename whichis not absolute, and
//                              it desires to qualify it in a simple, transparent way. Services:
//                              qualify(p):  qualifies 'p' to an absolute path, by trying in following order:
//                                           -to qualify 'p' with the 'current path'.
//                                           -to qualify 'p' with the 'system paths'.
//                                           -to qualify 'p' with the exec's own path (pwd)
//
//                              setCurrentPath(): sets the 'current path', see above. Clients which want to
//                                                make some filenames relative to some path should call this,
//                                                e.g. when loading a file which can refer to relative files.
//                              tail(p):     get tail of path p, i.e. its filename+ext                                
//                                              


#include <Genrl/string.h>
#include <Genrl/darray.h>


class   PathQualifier
        {
        public:


        static void            reset();                         //Bring this back to initial state
        static STRING          qualify(const char*);            //Qualify arg with crt-path/sys-paths/exec-path
        static STRING          setCurrentPath(const char*);     //Set current-path for later qualification
        static void            addSystemPath(const char*);
        static STRING          tail(const char*);               //Get filename+ext out of path
        static STRING          removeExtension(const char*);    //Cut ext from path
        static STRING          dir(const char*);                //Cut filename+ext from path
        static STRING          cannonical(const char*);         //Makes a Tcl platform-indep path from a platform-dep rep


        private:

        static DARRAY<STRING>  paths;
        static STRING          crt_path;
        };



#endif


