#ifndef PR_RSF_ATTR_SET_TYPE_H
#define PR_RSF_ATTR_SET_TYPE_H

//  RSFAttrSetType:	Concrete implementation of AttrSetType. This models an AttrSetType (nodetype or edgetype)
//			for a Rigi RSFDomainModel. Basically this enforces the constraint that all
//			nodes/edges have the same type by simply delegating the operations of this to
//			a GenericAttrSetType implementation object. For these reasons, RSFDomainModel
//			is so far the only one that constructs RSFAttrSetTypes.
//  
//  RSFArcType:	 	Specialisation of RSFAttrSetType for arctypes. Simply extends the info in 
//			RSFAttrSetType with two strings being the names of the second and third args of
//			a line in a Rigiarc file - namely the 'from' and 'to' nodetypes. These attributes
//			are the empty strings, if they were not specified in Rigiarc.
//

#include "AttrSetType.h"
#include <Genrl/string.h>


class   AttrType;
class   GenericAttrSetType;


class	RSFAttrSetType : public AttrSetType
	{
	public:
	
	
			     RSFAttrSetType(const char*,		   //Ctor (set name,impl object)
					    const GenericAttrSetType*);    //ONLY FOR RSFDomainModel
					       
		       	    ~RSFAttrSetType();				   //Dtor

	const AttrType*	     getAttrType(const char*) const;		   //impl inh
	const AttrType*	     getAttrType(int) const;			   //impl inh
	int		     numAttrTypes() const;			   //impl inh

	protected:

	const   GenericAttrSetType* imp;
	};



class 	RSFArcType : public RSFAttrSetType                                //Specializes RSFAttrSetType for arcs
	{
	public:
	
			    RSFArcType(const char*,			  //Ctor
			    	       const GenericAttrSetType*,	  //ONLY for RSFDomainModel
			    	       const char*,
			    	       const char*);
			    	  
	const char*	    fromNodeType() const;			  //Types of from/to nodes for this arc-type  	
	const char*	    toNodeType()   const;			  //or "" if not constrained 
	
	private:
	
	STRING		   from_nodetype,to_nodetype;			  //names of Rigiarc nodetypes
	};			    	  
			    	       		
	
	
inline RSFAttrSetType::RSFAttrSetType(const char* n,const GenericAttrSetType* imp_)
	 	      :AttrSetType(n),imp(imp_)
{  }


inline RSFArcType::RSFArcType(const char* n,const GenericAttrSetType* imp,
			      const char* from,const char* to)
		  :RSFAttrSetType(n,imp),from_nodetype(from),to_nodetype(to)
{   }

inline const char* RSFArcType::fromNodeType() const
{  return from_nodetype;  }

inline const char* RSFArcType::toNodeType() const
{  return to_nodetype;  }

		  	      

#endif
