#ifndef PR_RSF_DOMAIN_MODEL
#define PR_RSF_DOMAIN_MODEL

//   RSFDomainModel:		Specialization of DomainModel that accepts Rigi RSF domain models.
//				This gets thus constructed from a Rigi set of files (Rigiattr,Riginode,Rigiarc).
//				This uses a RSFAttrSetType specialization of AttrSetType. 
//				RSFAttrSetType has the same interface as AttrSetType. However, the idea
//				is that the RSFDomainModel-RSFAttrSetType combination enforces that
//				all nodetypes/arctypes have the same attr-types. Moreover, these are
//				implemented in a more efficient manner than e.g. GenericDomainModel
//	
//   REMARK: Assumptions 	- syntax strictly conforming to RSF syntax
//   made when parsing RSF	- node types (arg 2,3 of Rigiarc lines) must exist in the Riginode file
//   domain files:		- arc type names (Rigiarc) are different than node type names (Riginode)
//


#include "DomainModel.h"
#include <stdio.h>


class   RSFAttrSetType;		//The concrete AttrSetType this uses.
class   GenericAttrSetType;	



class	RSFDomainModel : public DomainModel
	{
	public:
	
				RSFDomainModel(const char*);		//Ctor (name)
		       	       ~RSFDomainModel();			//Dtor
	int			constr(const char*,			//Construct this from Riginode,
				       const char*,			//Rigiarc, and Rigiattr domain files.
				       const char*);	       	     	//Returns 1 if ok, else 0
        void                    save(FILE*);                            //Impl inh
	
	private:

	int			read_node_file(FILE*);
	int			read_arc_file(FILE*);
	int			read_attr_file(FILE*);
	GenericAttrSetType	*nodetype,				//Implem objs for the unique
			        *arctype;				//nodetype/arctype in the RSF domain model
	
	};






#endif

