#ifndef PR_SELLIST_H
#define PR_SELLIST_H

//	SelectionList:	Maintains a list of all persistent Selection objects in the system.
//			This class is managed by GraphManager. 
//			Selections (with names) can be added to the list. Besides these selections,
//			the list maintains two particular selections: the full-selection
//			and the empty selection. The full selection contains the whole graph.
//			The empty selection is always empty. The names for these two selections are
//			reserved. 
//			SelectionList is managed at a higher level by GraphManager which ensures that
//			selections are updated wheneven nodes/edges are removed from the Graph.
//
//

#include "Selection.h"
#include "Genrl/darray.h"
#include "Genrl/types.h"

class Graph;

class SelectionList	 
		  {
		  public:
		  
		  	 	SelectionList();			       //CONSTR/DESTR 	
		  	       ~SelectionList();			       //Ctor: creates a FullSelection and a NullSelection
		  	       						       //Dtor: deletes all Selections in this
					
									       //QUERY METHODS
		  int		numSelections() const;			       //get # selection in list
		  Selection*	getSelection(int);			       //iterate over selections
		  Selection*    getSelection(const char*);		       //get selection by name (return NullSelection if none found)
		  Selection* 	getFullSelection();			       //get the complete selection
		  Selection* 	getNullSelection();			       //get the empty selection

		  							       //MANAGEMENT METHODS
		  void		addSelection(Selection*);		       //add new selection to list. Destroys any existing selection with same name
		  							       // (if the arg is physically already in the list, nothing happens)
		  void		delAllSelections();			       //delete	all selections in this except ALL, NONE
		  void		reset();				       //del all selections and also all nodes in the full selection
                  void          save(FILE*,const char*);                       //save Tcl-cmds to restore this from *.sel file <arg2>
                  void          saveDefinition(const char*);                   //deep-save all sels in *.sel file 
                  void          read(const char*);                             //read selections from *.sel file back to this
		  BOOL		delSelection(const char*);		       //destroys selection given by name from list. Returns 1 if op completed ok.
		  BOOL		renameSelection(Selection*,const char*);       //renames selection. Deletes selection with new name if any.
		  
		  	
		  private:	
		  
                  Selection*    readSelection(FILE*);                          //Read new selection from .sel file into the system
                                                                               //Returns 0 if no selection could be read.

		  DARRAY<Selection*>		sels;
		  };	 		


inline int SelectionList::numSelections() const
{  return sels.Count();  }

inline Selection* SelectionList::getSelection(int i) 
{  return sels[i];  }

inline Selection* SelectionList::getFullSelection()
{  return getSelection(Selection::ALL);  }			
   
inline Selection* SelectionList::getNullSelection()
{  return getSelection(Selection::NONE);  }
   

#endif
