#ifndef PR_SO_CACHED_FILE_H
#define PR_SO_CACHED_FILE_H

// SoCachedFile:	Utility class for Open Inventor. A SoCachedFile is a group holding the contents of an
//			Inventor file. In contrast to directly reading and using a file via SoDB::read(), 
//			SoCachedFile caches the contents of this read and hashes it by file-name. That is, if a file
//			is read several times via SoCachedFile::read(), one gets the same cached scenegraph, which is 
//			quite efficient. The cache gets automatically flushed when that scenegraph is destroyed by 
//			Inventor (i.e. when it is unref'd the last time). The only assumption users must make when using
//			SoCachedFile is that the contents of files given by filename never change. If they do change, users 
//			can flush the cache by calling SoCachedFile::flush(). This does NOT destroy any of the previously
//			cached nodes, but it will simply forget that those filenames were cached. flush() can be called
//			also for a given file-name to flush only the entry for that file - i.e. to inform the cache that
//			the cache for that file is no longer valid.
//			The main purpose of this class is making reading many small files more efficient.

#include <Inventor/nodes/SoGroup.h>
#include <map>
#include "Genrl/string.h"


class	SoCachedFile : SoGroup					//Inheritance takes care of deleting nodes from cache
	{
	public:

		static SoNode* read(const char*);		//replaces SoDB::read()
		static void    flush(const char* =0);		//flushes cache when some or all file(s) changed

	protected:

			       SoCachedFile(SoNode*,const char*);
			      ~SoCachedFile(); 

	private:
		STRING	       file_name;
		static std::map<STRING,SoNode*> cache;
		static std::map<SoNode*,std::map<STRING,SoNode*>::iterator> cache2;
	};











#endif



