#ifndef PR_SPACE_PARTITION_H
#define PR_SPACE_PARTITION_H


#include <set>
#include "Genrl/types.h"

class	Node;

template <class T>
class	SpacePartition
	{
	public:
	struct Item	{  
			   float ox,oy,x,y; T data;
			   Item(float ox_,float oy_,float x_,float y_,T d =0)
			        :ox(ox_),oy(oy_),x(x_),y(y_), data(d) { }
			   BOOL overlap(const Item& it) const
			   {
			     if (intersect(it.ox,it.oy,it.ox,it.y)) return TRUE;
			     if (intersect(it.ox,it.oy,it.x, it.oy)) return TRUE;
			     if (intersect(it.x, it.oy,it.x, it.y)) return TRUE;
			     if (intersect(it.ox,it.y, it.x, it.y)) return TRUE;
			     return FALSE;	
			   }
			   BOOL intersect(float x3, float y3, float x4, float y4) const
			   {
			     unsigned char code1 = 0, code2 = 0;
			     if (x3<ox) code1 |=8; if (y3>y ) code1 |=4;
			     if (x3>x ) code1 |=2; if (y3<oy) code1 |=1;
			     if (x4<ox) code2 |=8; if (y4>y ) code2 |=4;
			     if (x4>x ) code2 |=2; if (y4<oy) code2 |=1;
			     return (((code1|code2)==0) || !((code1&code2)!=0));
			   }
			};


	
			SpacePartition(float,float,float,float,float);
		       ~SpacePartition();
	void		insert(const Item&);
	void		remove(T);
	BOOL		intersect(const Item&);


	struct  Pred
	{ 
	   bool operator()(const Item& i1,const Item& i2) const 
	   { return &i1<&i2; }
	};




	private:
	typedef std::set<Item,Pred>
			Items;
	Items*		get(int,int);
	int		pos2idx(float,float,int&,int&);
	float		cell_sz;
	int		nx,ny;
	float		xmin,ymin,xmax,ymax;

			
	Items*		tbl;

	};



template <class T> inline  SpacePartition<T>::Items* SpacePartition<T>::get(int i,int j)
{  return (i>=0 && i<nx && j>=0 && j<ny)? &tbl[nx*i+j] : 0; }


template <class T>
SpacePartition<T>::SpacePartition(float xmin_,float ymin_,float xmax_,float ymax_,float cell_sz_)
	       :xmin(xmin_),xmax(xmax_),ymin(ymin_),ymax(ymax_),cell_sz(cell_sz_)
{
   nx = int((xmax-xmin)/cell_sz);
   ny = int((ymax-ymin)/cell_sz);

   tbl = new Items[nx*ny];
}


template <class T>
SpacePartition<T>::~SpacePartition()
{
   delete[] tbl;
}


template <class T>
inline int SpacePartition<T>::pos2idx(float x,float y,int& i,int& j)
{
  if (x<xmin || x>xmax || y<ymin || y>ymax) return 0;
  i = int(x/cell_sz); j = int(y/cell_sz);
  return 1;
}



template <class T>
void SpacePartition<T>::insert(const Item& it)
{
   int i,j; if (!pos2idx(it.ox,it.oy,i,j)) return;
  
   get(i,j)->insert(it);
}




template <class T>
BOOL SpacePartition<T>::intersect(const Item& it) 
{
  int i,j;
  if (!pos2idx(it.ox,it.oy,i,j)) return FALSE;

  Items* items = get(i,j);
  for(Items::iterator iter = items->begin(); iter!=items->end(); iter++)
  {
     const Item& tst = *iter;
     if (tst.overlap(it)) return TRUE;
     if (it.overlap(tst)) return TRUE;
  }
  return FALSE;
}






#endif

