#ifndef PR_TREE_TRAVERSAL_H
#define PR_TREE_TRAVERSAL_H

//   TreeTraversal:	A traversal that iterates through all (in)direct descendants of
//			a set of Nodes in a Graph. The descendants are computed via the 
//			user-provided Filter node-filter. This lets one have a flexible definition of
//			the notion of 'descendant'.
//			TreeTraversal ensures that the order in which its Nodes/Edges are retrieved
//			by its iterators is a breadth-first traversal from the given selection.
//
//



#include "Genrl/darray.h"
#include "Genrl/dqueue.h"
#include "Node.h"

class Selection;
class Edge;
class Filter;

class TreeTraversal  
		  {
		  public:  
		  	 	TreeTraversal(Selection* inp,Selection* graph,Filter*);
		  	
		  class NodeIterator 
		  		{
		  		public:
		  			 NodeIterator(DARRAY<Node*>&,TreeTraversal*);
		  		   Node* next();
		  		private: 
		  			 DQUEUE<Node*> q; TreeTraversal* tt;
		  		};

		  class EdgeIterator
		  		{
		  		public:
		  			 EdgeIterator(DARRAY<Node*>&,TreeTraversal*);
		  		   Edge* next();
		  		private: 
		  			 DQUEUE<Node*> q; int idx; EdgeSet edge_set; TreeTraversal* tt;
		  		};

		  NodeIterator* iterNodes();			       			
		  EdgeIterator* iterEdges();			       

		  private:	
		  
		  friend class	NodeIterator;
		  friend class	EdgeIterator;
		  
		  void		recalcNodesEdges();
		  DARRAY<Node*>	root;
		  Filter*	filter;
		  Selection*	all;
		  };


inline TreeTraversal::NodeIterator* TreeTraversal::iterNodes()
{  return new NodeIterator(root,this);  }

inline TreeTraversal::EdgeIterator* TreeTraversal::iterEdges()
{  return new EdgeIterator(root,this);  }

		  
#endif
