#ifndef PR_UI_H
#define PR_UI_H

//  UI.h:	Simple adapter that brings both X11 and Windows-based Inventor declarations
//		to a single naming convention. Several class-names are introduced here:
//
//		SoUIExaminerViewer
//		SoUIViewer
//		SoUIComponent
//
//		Client code should use these instead of the platform-specific
//		SoWin... or SoXt... classes.
//


#include "Genrl/string.h"
#include "Genrl/darray.h"
#include "Genrl/types.h"

class	UIViewerBase							//Baseclass for our own extensions to the
	{								//Inventor viewers. This is platform-independent
	public:

	
	enum PAN_DIRECTION	{					//Panning directions
				   INCR_X,DECR_X,INCR_Y,DECR_Y
				};

	virtual		       ~UIViewerBase();				//Dtor	
	virtual void		pan(PAN_DIRECTION);			//Pan (translate) the camera with one 
									//step in the X/Y directions
	static void		setPanStep(float);			//Set/get the panning step as fraction of the
	static float		getPanStep();				//viewport-size in that dimension. Good values are 0.1..1
	virtual void		addButton(const char*, const char*,	//Adds a btn to the viewer's UI (def=nop). For every button,
					  const char*,BOOL);		//the icon-filename, Tcl-proc-callback, button-tooltip,
									//and check-or-not style) are specifiable.
	virtual void		addActivateCommand(const char*);	//Add Tcl proc to be executed when the viewer gets the focus
									//(subclasses might enhance)
	
	protected:
				UIViewerBase();				//Ctor
	DARRAY<STRING>		commands;				//Tcl commands for all user buttons
	DARRAY<STRING>		tooltips;
	STRING			cmd_activate;				//Tcl command called when this window becomes active
	static float		pan_step;				//Panning step
	static const int        ICON_SIZE;				//Size of the (square) user-buttons on the viewer
	};


#ifdef WIN32
//MS Windows: Binds the SoUI* names to SoWin* names

class   SoUIExaminerViewer;
#define SoUIViewer	SoWinViewer
#define SoUIComponent	SoWinComponent
#define SoUI		SoWin
#else
//X/UNIX: Binds the SoUI* names to SoXt* names

class   SoUIExaminerViewer;
#define SoUIViewer	SoXtViewer
#define SoUIComponent	SoXtComponent
#define SoUI		SoXt
#endif

inline void UIViewerBase::setPanStep(float ps)
{  pan_step = ps;  }

inline float UIViewerBase::getPanStep()
{  return pan_step;  }

#endif

